package medstore.model;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import medstore.math.BigDec;

public class HashTableModel extends AbstractTableModel implements TableModel{
    private Vector data;
    private String[] cols;
    private String[] keys;
    public HashTableModel(Vector data, String[] colNames) {
        cols = colNames;
        if(data == null){
            this.data =  new Vector();
            return;
        }
        
        this.data = data;
    }
    public Object getValueAt(int rowIndex, int columnIndex){
        try {
            LinkedHashMap map = (LinkedHashMap)data.get(rowIndex);
            return map.values().toArray()[columnIndex];
        } catch(Exception e) {
            return null;
        }
    }
    public void setValueAt(Object aValue, int rowIndex, int columnIndex){
        if ( data == null) {
            return;
        }
        LinkedHashMap map = (LinkedHashMap)data.get(rowIndex);
        map.put(map.keySet().toArray()[columnIndex],aValue);
        data.set(rowIndex,map);
    }
    public int getColumnCount(){
        return cols.length;
    }
    public int getRowCount(){
        try {
            return data.size();
        } catch(Exception e) {
            return 0;
        }
        
    }
    public String getColumnName(int columnIndex) {
        try {
            return cols[columnIndex].toString();
        } catch(Exception e) {
            return null;
        }
    }
    
    public LinkedHashMap getRowAt(int rowIndex){
        try{
            return (LinkedHashMap)data.get(rowIndex);
        } catch(Exception e){
            return null;
        }
    }
    
    public String[] getCols() {
        return cols;
    }
    public int getContentIndex(Object key, Object value){
        try{
            for(int i = 0; i < data.size(); i++){
                LinkedHashMap row = (LinkedHashMap)data.get(i);
                if(String.valueOf(value).equalsIgnoreCase(String.valueOf(row.get(key)))){
                    return i;
                }
            }
        } catch(Exception e){
            
        }
        return -1;
    }
    public void setData(Object obj){
        try{
            if(data!=null)
                data.clear();
            data = (Vector)obj;
            fireTableDataChanged();
        } catch(Exception e){
            
        }
    }
    public Vector getDataVector(){
        return data;
    }
    public void addRow(LinkedHashMap  row ) {
        if(data==null)
            data = new Vector();
        
        data.add(row);
        fireTableDataChanged();
    }
    public void removeRow(int index) {
        if(data==null || data.size()<=0)
            return;
        data.remove(index);
        fireTableDataChanged();
    }
    public void removeRowByKey(String key, Object value) {
        if(data==null || data.size()<=0)
            return;
        try{
            for(int i = 0; i < data.size(); i++){
                LinkedHashMap row = (LinkedHashMap)data.get(i);
                if(String.valueOf(value).equalsIgnoreCase(String.valueOf(row.get(key)))){                                        
                    data.remove(i);                    
                    fireTableDataChanged();
                    return;
                }
            }
        } catch(Exception e){
            
        }        
    }
    public void clearData(){
        if(data !=null)
            data.removeAllElements();
        data = new Vector();
    }
    public void formatDate(String mapKey, String format){
        try{
            if(data!=null && data.size()>0){
                for(int i = 0; i < data.size(); i++){
                    LinkedHashMap row = (LinkedHashMap)data.get(i);
                    String date = fromTimestamp(row.get(mapKey),format);
                    row.put(mapKey,date);
                }
            }
        } catch(Exception e){
            
        }
    }
    public void formatDecimal(String mapKey){
        try{
            if(data!=null && data.size()>0){
                for(int i = 0; i < data.size(); i++){
                    LinkedHashMap row = (LinkedHashMap)data.get(i);
                    String date = scaledRoundHalfUp(row.get(mapKey),2);
                    row.put(mapKey,date);
                }
            }
        } catch(Exception e){
            
        }
    }
    public static String scaledRoundHalfUp(Object value, int scale){
        try {
            value = (value==null || "null".equalsIgnoreCase(String.valueOf(value).trim()) || String.valueOf(value).trim().length()<=0) ? "0.00" : value;
            BigDecimal bdm = new BigDecimal(value.toString());
            bdm = bdm.setScale(scale,BigDecimal.ROUND_HALF_UP);
            return bdm.toString();
        } catch(Exception e) {
            return String.valueOf(value);
        }
    }
    private boolean ascending = false;
    private int oldIndex = 0;
    public void sortColumn(int colIndex) {
        ascending = !ascending;
        if(colIndex!= oldIndex){
            ascending = true;
        }
        Collections.sort(data, new ColumnSorter(colIndex, ascending));
        oldIndex = colIndex;
        fireTableDataChanged();
    }
    
    public static String fromTimestamp(Object value, String format){
        try {
            SimpleDateFormat sdf = new SimpleDateFormat(format);
            Timestamp t = (Timestamp)value;
            Date d = new Date(t.getTime());
            return sdf.format(d);
        } catch(Exception e) {
            return String.valueOf(value);
        }
    }
}
