package medstore.orders;

import holders.User;
import holders.orders.Order;
import java.awt.Dialog;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import medstore.forms.FormConvertor;
import medstore.model.HashTableModel;
import medstore.model.TableUtils;

public class OrdersPanel extends javax.swing.JPanel implements SearchOrdersBase{
    private BaseOrdersPanel base;
    private User user;
    private SearchOrders searchOrders;
    private MacroEditDialog macroEditDialog;
    private Dialog parWin;
    String columnNames[] = {"Order No","Date","Doc. Type","Netto","Total","Payed"};
    
    public OrdersPanel() {
        initComponents();
        tblOrders.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt){
                if( evt.getSource() == tblOrders ) {
                    if ((evt.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
                        int nRow = tblOrders.rowAtPoint(evt.getPoint());
                        LinkedHashMap row = (LinkedHashMap)((HashTableModel)tblOrders.getModel()).getRowAt(nRow);
                        base.setOrder(row);
                    }
                }
            }
        });
        TableUtils.setTableHeader(tblOrders);
        TableUtils.setColumnSorter(tblOrders);
        tblOrders.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tblOrders.getTableHeader().setReorderingAllowed(false);
        tblOrders.setModel(new HashTableModel(null,columnNames));
        final Window w = SwingUtilities.windowForComponent(this);
        parWin  = FormConvertor.convertor(w);
        macroEditDialog = new MacroEditDialog(parWin);
    }
    
    public String getMacroText(){
        return macroEditDialog.getMacroText();
    }
    public void setSearchOrder(String v_nr){
        base.loadOrders(v_nr);
    }
    public User getUser(){
        return user;
    }
    
    public void setUser(User user) {
        this.user = user;
    }
    
    public void setBase(BaseOrdersPanel base) {
        this.base = base;
        macroEditDialog.setBase(base);
        setUser(this.base.getUser());
        
    }
    public void setOrdersModel(Vector data){
        int[] width = {200,200,200,120,160,120};
        for( int i = 0; i < data.size(); i++ ) {
            LinkedHashMap map = (LinkedHashMap)data.get(i);
            String doc_type = String.valueOf(map.get(Order.DOC_TYPE_IDNT));
            doc_type = doc_type.equalsIgnoreCase("1") ? "Invoice" : "Credit Note";
            
            String v_nr = String.valueOf(map.get(Order.V_NR));
            String price = String.valueOf(map.get(Order.TOTL_SUM));
            String netto = String.valueOf(map.get(Order.NETO_SUM));
            
            String pref = doc_type.equalsIgnoreCase("Invoice") ? "" : "-";
            
            try {
                if( Double.parseDouble(price) > 0 )
                    map.put(Order.TOTL_SUM,pref+price);
            } catch(Exception e) {
                
            }
            
            try {
                if( Double.parseDouble(netto) > 0 )
                    map.put(Order.NETO_SUM,pref+netto);
            } catch(Exception e) {
                
            }
            map.put(Order.DOC_TYPE_IDNT,doc_type);
            data.set(i,map);
        }
        HashTableModel model = new HashTableModel(data,columnNames);
        
        model.formatDate(Order.INVC_DATE,"dd.MM.yyyy");
        model.formatDecimal(Order.TOTL_SUM);
        model.formatDecimal(Order.PAY_SUM);
        model.formatDecimal(Order.NETO_SUM);
        
        TableUtils.setTableColumnsWidth(tblOrders,width);
        tblOrders.setModel(model);
        TableUtils.setHorizontalAlignment(tblOrders, 0, javax.swing.JTextField.LEFT );
        TableUtils.setHorizontalAlignment(tblOrders, 1, javax.swing.JTextField.LEFT );
        TableUtils.setHorizontalAlignment(tblOrders, 2, javax.swing.JTextField.LEFT );
        TableUtils.setHorizontalAlignment(tblOrders, 3, javax.swing.JTextField.RIGHT );
        TableUtils.setHorizontalAlignment(tblOrders, 4, javax.swing.JTextField.RIGHT );
        TableUtils.setHorizontalAlignment(tblOrders, 5, javax.swing.JTextField.RIGHT );
        tblOrders.updateUI();
    }
    public Object getOrder(String v_nr){
        int nRow = ((HashTableModel)tblOrders.getModel()).getContentIndex(Order.V_NR,v_nr);
        return ((HashTableModel)tblOrders.getModel()).getRowAt(nRow);
    }
    public int hasCreditNote(String v_nr){
        int nRow = ((HashTableModel)tblOrders.getModel()).getContentIndex(Order.INVOICEID,v_nr);
        return nRow;
    }
    public JButton getBtnPrintInvoice() {
        return btnPrintInvoice;
    }
    
    public JButton getBtnPrintPacking() {
        return btnPrintPacking;
    }
    
    public JButton getBtnCreditNote() {
        return btnCreditNote;
    }
    
    public JTable getTblOrders() {
        return tblOrders;
    }
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        tblOrders = new javax.swing.JTable();
        findOrder = new javax.swing.JButton();
        btnMacros = new javax.swing.JButton();
        btnPrintInvoice = new javax.swing.JButton();
        btnPrintPacking = new javax.swing.JButton();
        btnCreditNote = new javax.swing.JButton();

        tblOrders.setFont(new java.awt.Font("Arial", 1, 12));
        tblOrders.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        tblOrders.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                tblOrdersKeyReleased(evt);
            }
        });

        jScrollPane1.setViewportView(tblOrders);

        findOrder.setFont(new java.awt.Font("Arial", 1, 12));
        findOrder.setText("Find Order");
        findOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                findOrderActionPerformed(evt);
            }
        });

        btnMacros.setFont(new java.awt.Font("Arial", 1, 12));
        btnMacros.setText("Macros");
        btnMacros.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMacrosActionPerformed(evt);
            }
        });

        btnPrintInvoice.setFont(new java.awt.Font("Arial", 1, 12));
        btnPrintInvoice.setText("Print Invoice");
        btnPrintInvoice.setEnabled(false);
        btnPrintInvoice.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPrintInvoiceActionPerformed(evt);
            }
        });

        btnPrintPacking.setFont(new java.awt.Font("Arial", 1, 12));
        btnPrintPacking.setText("Print Packing");
        btnPrintPacking.setEnabled(false);
        btnPrintPacking.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPrintPackingActionPerformed(evt);
            }
        });

        btnCreditNote.setFont(new java.awt.Font("Arial", 1, 12));
        btnCreditNote.setText("Credit Note");
        btnCreditNote.setEnabled(false);
        btnCreditNote.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCreditNoteActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 554, Short.MAX_VALUE)
            .add(layout.createSequentialGroup()
                .add(findOrder)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btnMacros)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 49, Short.MAX_VALUE)
                .add(btnCreditNote, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 101, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btnPrintInvoice)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btnPrintPacking))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 154, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(18, 18, 18)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(findOrder)
                    .add(btnPrintPacking)
                    .add(btnPrintInvoice)
                    .add(btnCreditNote)
                    .add(btnMacros))
                .addContainerGap(19, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    private void btnCreditNoteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCreditNoteActionPerformed
        base.creditNote();
    }//GEN-LAST:event_btnCreditNoteActionPerformed
    
    private void tblOrdersKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_tblOrdersKeyReleased
        int nRow = tblOrders.getSelectedRow();
        LinkedHashMap row = (LinkedHashMap)((HashTableModel)tblOrders.getModel()).getRowAt(nRow);
        base.setOrder(row);
    }//GEN-LAST:event_tblOrdersKeyReleased
    
    private void btnPrintPackingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPrintPackingActionPerformed
        try {
            base.print(2);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(parWin,"Can't print packing!\nError: " + ex.getMessage());
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnPrintPackingActionPerformed
    
    private void btnPrintInvoiceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPrintInvoiceActionPerformed
        try {
            base.print(1);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(parWin,"Can't print Invoice!\nError: " + ex.getMessage());
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnPrintInvoiceActionPerformed
    
    private void btnMacrosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMacrosActionPerformed
        if(macroEditDialog==null){
            macroEditDialog = new MacroEditDialog(parWin);
            macroEditDialog.setBase(base);
        }
        if(parWin == null){
            final Window w = SwingUtilities.windowForComponent(this);
            parWin  = FormConvertor.convertor(w);
            macroEditDialog = new MacroEditDialog(parWin);
            macroEditDialog.setBase(base);
        }
        macroEditDialog.setVisible(true);
    }//GEN-LAST:event_btnMacrosActionPerformed
    
    private void findOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_findOrderActionPerformed
        if(searchOrders==null){
            final Window w = SwingUtilities.windowForComponent(this);
            Dialog parWin  = FormConvertor.convertor(w);
            searchOrders = new SearchOrders(this,parWin);
        }
        
        searchOrders.setVisible(true);
    }//GEN-LAST:event_findOrderActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCreditNote;
    private javax.swing.JButton btnMacros;
    private javax.swing.JButton btnPrintInvoice;
    private javax.swing.JButton btnPrintPacking;
    private javax.swing.JButton findOrder;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tblOrders;
    // End of variables declaration//GEN-END:variables
    
}
