
package medstore.orders;

import holders.orders.Order;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.*;
import medstore.forms.CommonDialog;
import medstore.model.HashTableModel;
import medstore.model.TableUtils;
import medstore.remote.io.forms.RemoteOrderSearch;
import medstore.verifiers.UniversalInputVerifier;

public class SearchOrders extends CommonDialog {
    public static int ORDER_KEY_INDEX=0;
    
    Vector vecResults;
    SearchOrdersBase base;
    RemoteOrderSearch order_search;
    
    String columnNamesOrders[] = {"Order No","Date","Doc. Type","Netto","Total","Payed"};
    
    public SearchOrders(final SearchOrdersBase base, Dialog parent) {
        super(parent,true);
        setTitle("Find order");
        setLocationByPlatform(true);
        initComponents();
        tblOrders.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt){
                if( evt.getSource() == tblOrders ) {
                    if ((evt.getModifiers() & InputEvent.BUTTON1_MASK) != 0 && evt.getClickCount()>1) {
                        int nRow = tblOrders.rowAtPoint(evt.getPoint());
                        submit(nRow);
                    }
                }
            }
        });
        TableUtils.setColumnSorter(tblOrders);
        TableUtils.setTableHeader(tblOrders);
        tblOrders.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tblOrders.getTableHeader().setReorderingAllowed(false);
        
        this.base = base;
        order_search = new RemoteOrderSearch(base.getUser());
        editFromDate.setInputVerifier(new UniversalInputVerifier(UniversalInputVerifier.UNI_DATE_VERIFIER));
        editToDate.setInputVerifier(new UniversalInputVerifier(UniversalInputVerifier.UNI_DATE_VERIFIER));
        tblOrders.getTableHeader().setReorderingAllowed(false);
        
    }
    private void submit(int nRow){
        LinkedHashMap row = (LinkedHashMap)((HashTableModel)tblOrders.getModel()).getRowAt(nRow);
        String v_nr = String.valueOf(row.get(Order.V_NR));
        String prsn_key = String.valueOf(row.get(Order.PATN_IDNT));
        base.setSearchOrder(v_nr);
        setVisible(false);
    }
    public void setOrdersModel(Vector data){
        if(data!=null){
            HashTableModel model = new HashTableModel(data,columnNamesOrders);
            model.formatDate(Order.INVC_DATE,"dd.MM.yyyy");
            model.formatDecimal(Order.TOTL_SUM);
            model.formatDecimal(Order.PAY_SUM);
            model.formatDecimal(Order.NETO_SUM);
            tblOrders.setModel(model);
            
            for( int i = 0; i < model.getRowCount(); i++ ) {
                String v_nr = String.valueOf(model.getValueAt(i, 0));
                String price = String.valueOf(model.getValueAt(i,3));
                String type =  String.valueOf(model.getValueAt(i,2));
                String pref = type.equalsIgnoreCase("Invoice") ? "" : "-";
                
                try {
                    if( Double.parseDouble(price) > 0 );
                    model.setValueAt(pref+price, i, 3);
                } catch(Exception e) {
                    
                }
                price = String.valueOf(model.getValueAt(i,4));
                try {
                    if( Double.parseDouble(price) > 0 );
                    model.setValueAt(pref+price, i, 4);
                } catch(Exception e) {
                    
                }
                model.setValueAt(type, i, 2);
            }
            
            TableUtils.setHorizontalAlignment(tblOrders, 0, javax.swing.JTextField.LEFT );
            TableUtils.setHorizontalAlignment(tblOrders, 1, javax.swing.JTextField.LEFT );
            TableUtils.setHorizontalAlignment(tblOrders, 2, javax.swing.JTextField.LEFT );
            TableUtils.setHorizontalAlignment(tblOrders, 3, javax.swing.JTextField.RIGHT );
            TableUtils.setHorizontalAlignment(tblOrders, 4, javax.swing.JTextField.RIGHT );
            TableUtils.setHorizontalAlignment(tblOrders, 5, javax.swing.JTextField.RIGHT );
            
            model.fireTableDataChanged();
        } else{
            tblOrders.setModel(null);
        }
        try {
            tblOrders.updateUI();
        } catch (Exception ex) {
        }
    }

    public JTable getTblOrders() {
        return tblOrders;
    }

    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jPanel1 = new javax.swing.JPanel();
        jLabel18 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        editFromDate = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        editToDate = new javax.swing.JTextField();
        editOrderNo = new javax.swing.JTextField();
        btnSearch = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        tblOrders = new javax.swing.JTable();
        jLabel1 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Search Orders");
        setFont(new java.awt.Font("Arial", 0, 10));
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Search by", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 14)));
        jPanel1.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jPanel1.setMaximumSize(new java.awt.Dimension(469, 90));
        jPanel1.setMinimumSize(new java.awt.Dimension(469, 90));
        jPanel1.setPreferredSize(new java.awt.Dimension(469, 70));
        jLabel18.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel18.setText("< >");

        jLabel2.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel2.setText("Order No.");
        jLabel2.setMaximumSize(new java.awt.Dimension(80, 18));
        jLabel2.setMinimumSize(new java.awt.Dimension(80, 18));
        jLabel2.setPreferredSize(new java.awt.Dimension(80, 18));

        jLabel3.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel3.setText("From(date)");
        jLabel3.setMinimumSize(new java.awt.Dimension(80, 18));
        jLabel3.setPreferredSize(new java.awt.Dimension(80, 18));

        editFromDate.setFont(new java.awt.Font("Tahoma", 0, 13));
        editFromDate.setMinimumSize(new java.awt.Dimension(80, 21));
        editFromDate.setPreferredSize(new java.awt.Dimension(80, 21));
        editFromDate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editFromDateActionPerformed(evt);
            }
        });

        jLabel4.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel4.setText("To(date)");
        jLabel4.setMinimumSize(new java.awt.Dimension(80, 18));
        jLabel4.setPreferredSize(new java.awt.Dimension(80, 18));

        editToDate.setFont(new java.awt.Font("Tahoma", 0, 13));
        editToDate.setMinimumSize(new java.awt.Dimension(80, 21));
        editToDate.setPreferredSize(new java.awt.Dimension(80, 21));
        editToDate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editToDateActionPerformed(evt);
            }
        });

        editOrderNo.setFont(new java.awt.Font("Tahoma", 0, 13));
        editOrderNo.setMinimumSize(new java.awt.Dimension(80, 21));
        editOrderNo.setPreferredSize(new java.awt.Dimension(80, 21));
        editOrderNo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editOrderNoActionPerformed(evt);
            }
        });

        btnSearch.setFont(new java.awt.Font("Arial", 1, 12));
        btnSearch.setText("Search");
        btnSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSearchActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(11, 11, 11)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(editOrderNo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(20, 20, 20)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(editFromDate, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(10, 10, 10)
                .add(jLabel18)
                .add(10, 10, 10)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(editToDate, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(21, 21, 21)
                .add(btnSearch)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(11, 11, 11)
                        .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(editOrderNo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(11, 11, 11)
                        .add(jLabel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(editFromDate, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(29, 29, 29)
                        .add(jLabel18))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(11, 11, 11)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(btnSearch)
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(editToDate, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                .add(11, 11, 11))
        );

        jScrollPane1.setFont(new java.awt.Font("Tahoma", 1, 13));
        tblOrders.setFont(new java.awt.Font("Arial", 1, 12));
        tblOrders.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Order No", "Date", "Doc. Type", "Netto", "Total", "Payed"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        tblOrders.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                tblOrdersKeyPressed(evt);
            }
        });

        jScrollPane1.setViewportView(tblOrders);

        jLabel1.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel1.setText("Orders found:");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(10, 10, 10)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 489, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 110, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 489, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(10, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(10, 10, 10)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(10, 10, 10)
                .add(jLabel1)
                .add(2, 2, 2)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(40, Short.MAX_VALUE))
        );
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-517)/2, (screenSize.height-337)/2, 517, 337);
    }// </editor-fold>//GEN-END:initComponents
    
    private void tblOrdersKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_tblOrdersKeyPressed
        if( evt.getSource() == tblOrders && evt.getKeyCode() == evt.VK_ENTER) {
            int nRow = tblOrders.getSelectedRow();
            submit(nRow);
        }
    }//GEN-LAST:event_tblOrdersKeyPressed
    
    private void editToDateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editToDateActionPerformed
        btnSearch.doClick();
        editToDate.requestFocus();
    }//GEN-LAST:event_editToDateActionPerformed
    
    private void editFromDateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editFromDateActionPerformed
        btnSearch.doClick();
        editFromDate.requestFocus();
    }//GEN-LAST:event_editFromDateActionPerformed
    
    private void editOrderNoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editOrderNoActionPerformed
        btnSearch.doClick();
        editOrderNo.requestFocus();
    }//GEN-LAST:event_editOrderNoActionPerformed
    
    private void btnSearchActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSearchActionPerformed
        if(order_search == null)
            order_search = new RemoteOrderSearch(base.getUser());
        String v_nr = editOrderNo.getText();
        String date_from = editFromDate.getText();
        String date_to = editToDate.getText();
        if((date_from.length()==0 && date_to.length()!=0)) {
            JOptionPane.showMessageDialog(this,"Invalid From(date) filter!");
            return;
        }
        if((date_from.length()!=0 && date_to.length()==0)) {
            JOptionPane.showMessageDialog(this,"Invalid To(date) filter!");
            return;
        }
        if(!UniversalInputVerifier.DATE_Verifier(editFromDate)){
            JOptionPane.showMessageDialog(this,"Invalid From(date) filter!");
            return;
        }
        if(!UniversalInputVerifier.DATE_Verifier(editToDate)){
            JOptionPane.showMessageDialog(this,"Invalid To(date) filter!");
            return;
        }
        try {
            if(v_nr.length()>0){
                Integer.parseInt(v_nr);
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(this,"Please, enter valid integer for Order number!");
            return;
        }
        try {
            Cursor hourglassCursor = new Cursor(Cursor.WAIT_CURSOR);
            setCursor(hourglassCursor);
            Vector data = order_search.findOrder(v_nr,date_from,date_to);
            setOrdersModel(data);
        } catch (Exception ex) {
            ex.printStackTrace();
        }finally{
            Cursor normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
            setCursor(normalCursor);
        }
    }//GEN-LAST:event_btnSearchActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnSearch;
    private javax.swing.JTextField editFromDate;
    private javax.swing.JTextField editOrderNo;
    private javax.swing.JTextField editToDate;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tblOrders;
    // End of variables declaration//GEN-END:variables
}