package medstore.orders.fullfilment;

import holders.orders.Order;
import java.awt.Component;
import java.awt.Rectangle;
import java.io.File;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import medstore.Base;
import medstore.String.StringUtils;
import medstore.auftrag.AuftragData;
import medstore.forms.FormConvertor;
import medstore.model.ComboDataModel;
import medstore.model.HashTableModel;
import medstore.model.TableUtils;
import medstore.orders.ShowOrderInfo;
import medstore.print.orders.OrderPrintUtils;
import medstore.remote.io.auftrag.FileIO;
import medstore.remote.io.auftrag.RemoteAuftrag;
import medstore.remote.io.fullfill.RemoteFullFill;
import medstore.remote.io.stores.RemoteStores;
import medstore.stock.Stock;
import medstore.stock.StockOnHand;
import medstore.verifiers.UniversalInputVerifier;

public class OrderFullfilment extends JFrame implements BaseFullfilment{
    
    private volatile Thread mainThread = null;
    private int curRow = -1;
    private Base base;
    
    Vector orderData;
    
    private Thread ordersThread;
    
    private StockOnHand stockOnHand;
    private ShowOrderInfo showOrderInfo;
    
    public OrderFullfilment(Base base) {
        
        initComponents();
        
        this.base = base;
        
        {
            int[] widths = {250,250,250,120};
            TableUtils.setTableColumnsWidth(tblActions, widths);
        }
        
        {
            int[] width = {100,150,100};
            TableUtils.setTableColumnsWidth(tblOrders,width);
        }
        radioGroupNoAct.add(radioToday);
        radioGroupNoAct.add(radioAll);
        radioToday.setSelected(true);
        prepareStockTable();
        setOrdersModel(null);
        initStore();
        startThread();
        
        orderData = null;
    }
    
    public void startThread(){
        
        ordersThread = new Thread(new Runnable() {
            public void run() {                
                while(ordersThread == Thread.currentThread()){
                    System.out.println("running in");                    
                    try{
                        final Rectangle rect = tblOrders.getVisibleRect();
                        tblPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Orders List - reloading ... "));
                        setUnfiled();
                        java.awt.EventQueue.invokeLater(new Runnable() {
                            public void run(){
                                tblOrders.scrollRectToVisible(rect);
                                if(curRow>-1 && tblOrders.getRowCount() > 0){
                                    curRow = Math.min(curRow,tblOrders.getRowCount()-1);
                                    tblOrders.setRowSelectionInterval(curRow,curRow);
                                }
                                
                            }
                        });
                        tblPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Orders List"));
                    } catch(Exception e){
                        
                    }
                    try {
                        ordersThread.sleep(1000*15);
                    } catch (InterruptedException ex) {
                        
                    }
                }
            }
        });
        ordersThread.start();
    }
    
    public void stopThread(){
         System.out.println("stoped");
        try{
            ordersThread.interrupt();
        } catch(Exception e){
            
        }
        ordersThread = null;
        
    }
    
    public void setUnfiled(){
        updateOrdersModel(getUnfiled());
    }
    
    public Vector getUnfiled(){
        try {
            boolean all = false;
            if(radioAll.isSelected()){
                all = true;
            }
            return RemoteFullFill.getUnFiled(base.getUser().getUserMap(), all);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(this,"Can't load orderlist");
            ex.printStackTrace();
            return null;
        }
    }
    
    public void prepareStockTable(){
        setStockModel(null);
    }
    
    public void setOrdersModel(Vector data){
        
        String columnNames[] = {"Order No","Date","Doc. Type"};
        
        if(data!=null)
            data = formatOrdersRows(data);
        HashTableModel model = new HashTableModel(data,columnNames);
        model.formatDate(view_flfl_ordr_noact.invc_date,"dd.MM.yyyy");
        tblOrders.setModel(model);
        
    }
    public void updateOrdersModel(Vector data){
        try{
            HashTableModel model = ((HashTableModel)tblOrders.getModel());
            data = formatOrdersRows(data);
            model.setData(data);
            model.fireTableDataChanged();
        }
        catch(Exception e){
            setOrdersModel(data);
        }
    }
    public void setStockModel(Vector data){
        String[] columns = {"Order No.","Stock Name","Barcode","Quantity"};
        data = formatStockRows(data);
        HashTableModel model = new HashTableModel(data,columns);
        tblActions.setModel(model);
    }
    
    public void addStockRow(LinkedHashMap row){
        
        ((HashTableModel)tblActions.getModel()).addRow(row);
        ((HashTableModel)tblActions.getModel()).fireTableDataChanged();
    }
    
    public Vector formatOrdersRows(Vector data){
        if(data == null || data.size()<=0)
            return data;
        for(int i = 0; i < data.size(); i ++){
            LinkedHashMap row = (LinkedHashMap)data.get(i);
            Object v_nr = row.get(view_flfl_ordr_noact.v_nr);
            Object invc_date = row.get(view_flfl_ordr_noact.invc_date);
            Object type_name = row.get(view_flfl_ordr_noact.type_name);
            
            LinkedHashMap new_row = new LinkedHashMap();
            new_row.put(view_flfl_ordr_noact.v_nr, v_nr);
            new_row.put(view_flfl_ordr_noact.invc_date, invc_date);
            new_row.put(view_flfl_ordr_noact.type_name, type_name);
            
            data.set(i,new_row);
        }
        return data;
    }
    
    public Vector formatStockRows(Vector data){
        if(data == null || data.size()<=0)
            return data;
        for(int i = 0; i < data.size(); i ++){
            LinkedHashMap row = (LinkedHashMap)data.get(i);
            Object v_nr = row.get(Order.V_NR);
            Object stck_name = row.get(Stock.stck_name);
            Object stck_brcd = row.get(Stock.stck_brcd);
            Object p_menge = row.get(Stock.p_menge);
            Object p_pos = row.get(Stock.p_pos);
            Object stck_key = row.get(Stock.stck_key);
            
            LinkedHashMap new_row = new LinkedHashMap();
            new_row.put(Order.V_NR, v_nr);
            new_row.put(Stock.stck_name, stck_name);
            new_row.put(Stock.stck_brcd, stck_brcd);
            new_row.put(Stock.p_menge, p_menge);
            new_row.put(Stock.p_pos, p_pos);
            new_row.put(Stock.stck_key, stck_key);
            
            data.set(i,new_row);
        }
        return data;
    }
    
    public void initStore(){
        try {
            Vector data = RemoteStores.getPackingRooms(base.getUser().getUserMap());
            ComboDataModel model = new ComboDataModel(data);
            comboStore.setModel(model);
            
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    private boolean isFilled(boolean show){
        try{
            String orderId = editOrderNo.getText();
            if(RemoteFullFill.isOrderFilled(base.getUser().getUserMap(),orderId)){
                if(show)
                    JOptionPane.showMessageDialog(this,"This order (No. " + orderId + ") is already fulfilled!");
                return true;
            }
            
        } catch(Exception e){
            
        }
        return false;
    }
    
    private synchronized void validateUI(){
        
        editStockBarcode.setText("");
        editOrderNo.setText("");
        editOrderNo.setEnabled(true);
        editStockCount.setText("1");
        editStockBarcode.setText("");
        btnCancelOrder.setEnabled(false);
        btnShowOrder.setEnabled(false);
        btnOrderSubmit.setEnabled(false);
        
        tblActions.updateUI();
        setStockModel(null);
        
        tblOrders.setEnabled(true);
        btnPrint.setEnabled(false);
        
        editOrderNo.requestFocus();
        
        startThread();
    }
    
    public void addToFullFill(){
        String orderId = editOrderNo.getText();
        if(isFilled(true))
            return;
        if(Integer.parseInt(orderId) == -1){
            JOptionPane.showMessageDialog(this,"You havent submit your order number yet!\nPlease enter order number and hit enter key!");
            return;
        }
        try{
            Integer.parseInt(editOrderNo.getText().trim());
        } catch(Exception e){
            JOptionPane.showMessageDialog(this,"Please, enter valid Order Number!");
            return;
        }
        try{
            Integer.parseInt(editStockCount.getText().trim());
        } catch(Exception e){
            JOptionPane.showMessageDialog(this,"Please, enter valid integer for stock count!");
            return;
        }
        
        if(!UniversalInputVerifier.BARCODE_Verifier(editStockBarcode)){
            JOptionPane.showMessageDialog(this,"Please, enter valid barcode!");
            return;
        }
        if(((HashTableModel)tblActions.getModel()).getContentIndex(Stock.stck_brcd, editStockBarcode.getText())>0){
            JOptionPane.showMessageDialog(this,
                    "You have already checked this stock for that order!\n" +
                    "You can press the 'Show Order' for more info.");
            return;
        }
        
        String barcode = editStockBarcode.getText().trim();
        String count = editStockCount.getText().trim();
        String order_no = editOrderNo.getText().trim();
        
        int nRow = stockRowExist(orderData, barcode, count, order_no);
        
        if( nRow < 0 ){
            JOptionPane.showMessageDialog(this, "There is no such record for this order!");
            return;
        }
        
        LinkedHashMap row = (LinkedHashMap)orderData.get(nRow);
        ((HashTableModel)tblActions.getModel()).addRow(row);
        
        editStockBarcode.setText("");
        editStockCount.setText("");
        editStockCount.requestFocus();
        
    }
    
    public void fullfill(){
        if(isFilled(true))
            return;
        if(orderData.size() != tblActions.getRowCount()){
            JOptionPane.showMessageDialog(null,"Your current stock count doesn't match the order stock count!\nYou can press the 'Show Order' for more info..");
            return;
        }
        
        int storeId = ((ComboDataModel)comboStore.getModel()).getCObjectAt(comboStore.getSelectedIndex()).nItemDBKey;
        int nRowCount = tblActions.getRowCount();
        
        HashMap data = new HashMap();
        data.put("stor_key",storeId);
        data.put("stock",((HashTableModel)tblActions.getModel()).getDataVector());
        try {
            String result = RemoteFullFill.fullfill(base.getUser().getUserMap(),data);
            JOptionPane.showMessageDialog(this,result);
            HashMap auftrag = RemoteAuftrag.loadAuftrag(base.getUser().getUserMap(),editOrderNo.getText().trim());
            
            String v_nr = StringUtils.clearString(auftrag.get("v_nr"));
            String prsn_appl = StringUtils.clearString(auftrag.get("prsn_appl"));
            String prsn_fnam = StringUtils.clearString(auftrag.get("prsn_fnam"));
            String prsn_lnam = StringUtils.clearString(auftrag.get("prsn_lnam"));
            String altr_name = StringUtils.clearString(auftrag.get("altr_name"));
            String patn_idnt = StringUtils.clearString(auftrag.get("patn_idnt"));
            String patn_strt = StringUtils.clearString(auftrag.get("patn_strt"));
            String patn_city = StringUtils.clearString(auftrag.get("patn_city"));
            String patn_code = StringUtils.clearString(auftrag.get("patn_code"));
            boolean packing = Boolean.valueOf(StringUtils.clearString(auftrag.get("packing"),"false"));            
            AuftragData auftragData = new AuftragData(v_nr, prsn_appl, prsn_fnam, 
                    prsn_lnam, altr_name, patn_idnt, patn_strt, patn_city, patn_code);
            FileIO.WriteToFile(new File("c:/auftrag.txt"),auftragData.toString(),false);
            
            removeRow(v_nr);
            validateUI();
            
            startThread();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    public void removeRow(String v_nr){
        HashTableModel model = (HashTableModel)tblOrders.getModel();
        model.removeRowByKey(Order.V_NR, v_nr);
        model.fireTableDataChanged();
        tblOrders.updateUI();
    }
    public static int stockRowExist(Vector data, String barcode, String count, String order_no){
        
        if(data == null || data.size()<=0)
            return -1;
        
        for(int i = 0; i < data.size(); i ++){
            
            LinkedHashMap row = (LinkedHashMap)data.get(i);
            
            String v_nr = String.valueOf(row.get(Order.V_NR)).trim();
            String stck_brcd = String.valueOf(row.get(Stock.stck_brcd)).trim();
            String p_menge = String.valueOf(row.get(Stock.p_menge)).trim();
            
            if(stck_brcd.equalsIgnoreCase(barcode.trim()) &&
                    p_menge.equalsIgnoreCase(count.trim()) &&
                    v_nr.equalsIgnoreCase(order_no.trim())){
                return i;
            }
            
            
        }
        
        return -1;
    }
    
    public Vector getOrderDetails(String v_nr) {
        try {
            return RemoteFullFill.loadOrderDetails(base.getUser().getUserMap(),editOrderNo.getText());
        } catch (Exception ex) {            
            return null;
        }
    }
    
    public void focusOrderNo(){
        try{
            Integer.parseInt(editOrderNo.getText());
            if(editOrderNo.isEnabled()){
                throw new Exception();
            }
        } catch(Exception e){
            editOrderNo.requestFocus();
        }
    }
    
    public void stockOnHand(){
        int storeId = ((ComboDataModel)comboStore.getModel()).getCObjectAt(comboStore.getSelectedIndex()).nItemDBKey;
        if(stockOnHand == null)
            stockOnHand = new StockOnHand(base, storeId);
        else
            stockOnHand.setStore(storeId);
        stockOnHand.setVisible(true);
    }
    
    public void showOrderInfo(){
        if(showOrderInfo==null){
            showOrderInfo = new ShowOrderInfo(FormConvertor.convertor(this));
        }
        showOrderInfo.setModal(false);
        showOrderInfo.setModel(getOrderDetails(editOrderNo.getText()));
        showOrderInfo.setVisible(true);
    }
    public void print() throws Exception{
        OrderPrintUtils op;
        String v_nr = editOrderNo.getText();        
        op = new OrderPrintUtils(v_nr, base.getUser());        
        op.printInvoice(false);
        op.printPacking(false);
       
    }
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        btngOperations = new javax.swing.ButtonGroup();
        jpmUndo = new javax.swing.JPopupMenu();
        jmiUndo = new javax.swing.JMenuItem();
        radioGroupNoAct = new javax.swing.ButtonGroup();
        scrAllStocks = new javax.swing.JScrollPane();
        tblActions = new javax.swing.JTable();
        lblStocks = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        editOrderNo = new javax.swing.JTextField();
        lblStore = new javax.swing.JLabel();
        comboStore = new javax.swing.JComboBox();
        radioToday = new javax.swing.JRadioButton();
        radioAll = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        lblStockCount = new javax.swing.JLabel();
        editStockCount = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        editStockBarcode = new javax.swing.JTextField();
        btnAddToFullfill = new javax.swing.JButton();
        btnPrint = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        btnOrderSubmit = new javax.swing.JButton();
        btnCancelOrder = new javax.swing.JButton();
        btnShowOrder = new javax.swing.JButton();
        buttonStockQuery = new javax.swing.JButton();
        tblPanel = new javax.swing.JScrollPane();
        tblOrders = new javax.swing.JTable();

        jmiUndo.setText("Undo");
        jpmUndo.add(jmiUndo);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Fullfil Order");
        scrAllStocks.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        tblActions.setFont(new java.awt.Font("Tahoma", 0, 13));
        tblActions.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        tblActions.setSelectionBackground(new java.awt.Color(125, 162, 246));
        scrAllStocks.setViewportView(tblActions);

        lblStocks.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        lblStocks.setText("Current operations");

        jLabel2.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel2.setText("OrderNo");

        editOrderNo.setFont(new java.awt.Font("Tahoma", 0, 13));
        editOrderNo.setMinimumSize(new java.awt.Dimension(60, 20));
        editOrderNo.setPreferredSize(new java.awt.Dimension(60, 20));
        editOrderNo.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                editOrderNoKeyPressed(evt);
            }
        });

        lblStore.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        lblStore.setText("Take stocks from:");

        comboStore.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        comboStore.setBorder(null);
        comboStore.setMinimumSize(new java.awt.Dimension(200, 22));
        comboStore.setPreferredSize(new java.awt.Dimension(200, 22));

        radioToday.setFont(new java.awt.Font("Tahoma", 1, 13));
        radioToday.setLabel("Today");
        radioToday.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radioTodayActionPerformed(evt);
            }
        });

        radioAll.setFont(new java.awt.Font("Tahoma", 1, 13));
        radioAll.setLabel("All");
        radioAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radioAllActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(editOrderNo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(comboStore, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lblStore)))
                .add(43, 43, 43)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(radioToday)
                    .add(radioAll))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(radioToday)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(radioAll))
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(lblStore))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(editOrderNo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(comboStore, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
        );

        lblStockCount.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        lblStockCount.setText("Count");

        editStockCount.setFont(new java.awt.Font("Tahoma", 0, 13));
        editStockCount.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        editStockCount.setMinimumSize(new java.awt.Dimension(60, 20));
        editStockCount.setPreferredSize(new java.awt.Dimension(60, 20));

        jLabel1.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel1.setText("Barcode");
        jLabel1.setRequestFocusEnabled(false);

        editStockBarcode.setFont(new java.awt.Font("Tahoma", 0, 13));
        editStockBarcode.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        editStockBarcode.setMinimumSize(new java.awt.Dimension(180, 20));
        editStockBarcode.setPreferredSize(new java.awt.Dimension(180, 20));
        editStockBarcode.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                editStockBarcodeFocusGained(evt);
            }
        });

        btnAddToFullfill.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnAddToFullfill.setText("Add to fullfilment");
        btnAddToFullfill.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddToFullfillActionPerformed(evt);
            }
        });

        btnPrint.setFont(new java.awt.Font("Microsoft Sans Serif", 1, 13));
        btnPrint.setText("Print");
        btnPrint.setEnabled(false);
        btnPrint.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnPrint.setMaximumSize(new java.awt.Dimension(76, 23));
        btnPrint.setMinimumSize(new java.awt.Dimension(76, 23));
        btnPrint.setPreferredSize(new java.awt.Dimension(76, 23));
        btnPrint.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPrintActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .add(lblStockCount)
                .add(29, 29, 29)
                .add(jLabel1)
                .add(197, 197, 197)
                .add(btnPrint, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
            .add(jPanel3Layout.createSequentialGroup()
                .add(editStockCount, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(6, 6, 6)
                .add(editStockBarcode, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(3, 3, 3)
                .add(btnAddToFullfill))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .add(2, 2, 2)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(8, 8, 8)
                        .add(lblStockCount))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(7, 7, 7)
                        .add(jLabel1))
                    .add(btnPrint, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(3, 3, 3)
                        .add(editStockCount, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(3, 3, 3)
                        .add(editStockBarcode, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(btnAddToFullfill)))
        );

        btnOrderSubmit.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnOrderSubmit.setText("Submit Order");
        btnOrderSubmit.setEnabled(false);
        btnOrderSubmit.setMaximumSize(new java.awt.Dimension(117, 26));
        btnOrderSubmit.setMinimumSize(new java.awt.Dimension(117, 26));
        btnOrderSubmit.setPreferredSize(new java.awt.Dimension(117, 26));
        btnOrderSubmit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOrderSubmitActionPerformed(evt);
            }
        });

        btnCancelOrder.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnCancelOrder.setText("Cancel Order");
        btnCancelOrder.setEnabled(false);
        btnCancelOrder.setMaximumSize(new java.awt.Dimension(117, 26));
        btnCancelOrder.setMinimumSize(new java.awt.Dimension(117, 26));
        btnCancelOrder.setPreferredSize(new java.awt.Dimension(117, 26));
        btnCancelOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelOrderActionPerformed(evt);
            }
        });

        btnShowOrder.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnShowOrder.setText("Show Order");
        btnShowOrder.setEnabled(false);
        btnShowOrder.setMaximumSize(new java.awt.Dimension(109, 26));
        btnShowOrder.setMinimumSize(new java.awt.Dimension(109, 26));
        btnShowOrder.setPreferredSize(new java.awt.Dimension(109, 26));
        btnShowOrder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnShowOrderActionPerformed(evt);
            }
        });

        buttonStockQuery.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        buttonStockQuery.setText("Stock on hand");
        buttonStockQuery.setMaximumSize(new java.awt.Dimension(123, 26));
        buttonStockQuery.setMinimumSize(new java.awt.Dimension(123, 26));
        buttonStockQuery.setPreferredSize(new java.awt.Dimension(123, 26));
        buttonStockQuery.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonStockQueryActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel5Layout = new org.jdesktop.layout.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel5Layout.createSequentialGroup()
                .add(buttonStockQuery, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 146, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btnShowOrder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 145, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(btnCancelOrder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 149, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(62, 62, 62)
                .add(btnOrderSubmit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 146, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(382, 382, 382))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel5Layout.createSequentialGroup()
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(buttonStockQuery, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnOrderSubmit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnShowOrder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnCancelOrder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        tblPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Orders List"));
        tblPanel.setMinimumSize(new java.awt.Dimension(300, 64));
        tblPanel.setPreferredSize(new java.awt.Dimension(300, 64));
        tblOrders.setFont(new java.awt.Font("Arial", 1, 12));
        tblOrders.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        tblOrders.setMinimumSize(new java.awt.Dimension(300, 64));
        tblOrders.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tblOrdersMouseClicked(evt);
            }
        });

        tblPanel.setViewportView(tblOrders);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 952, Short.MAX_VALUE)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .add(10, 10, 10)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(lblStocks, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 180, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jPanel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(tblPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 276, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, scrAllStocks, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 671, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(288, 288, 288))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 8, Short.MAX_VALUE)
                        .add(jPanel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(lblStocks))
                    .add(tblPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 138, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(scrAllStocks, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 120, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(10, 10, 10)
                .add(jPanel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(20, 20, 20))
        );
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-698)/2, (screenSize.height-369)/2, 698, 369);
    }// </editor-fold>//GEN-END:initComponents

    private void radioAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radioAllActionPerformed
        try{            
            stopThread();
            curRow = -1;
            startThread();
        } catch(Exception e){
            e.printStackTrace();
        }
    }//GEN-LAST:event_radioAllActionPerformed

    private void radioTodayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radioTodayActionPerformed
        try{            
            stopThread();
            curRow = -1;
            startThread();
        } catch(Exception e){
            e.printStackTrace();
        }
    }//GEN-LAST:event_radioTodayActionPerformed

    private void btnPrintActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPrintActionPerformed
        try {
            print();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnPrintActionPerformed

    private void btnShowOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnShowOrderActionPerformed
        showOrderInfo();
    }//GEN-LAST:event_btnShowOrderActionPerformed

    private void buttonStockQueryActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonStockQueryActionPerformed
        stockOnHand();
    }//GEN-LAST:event_buttonStockQueryActionPerformed
    
    private void editStockBarcodeFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_editStockBarcodeFocusGained
        focusOrderNo();
    }//GEN-LAST:event_editStockBarcodeFocusGained
    
    private void btnOrderSubmitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOrderSubmitActionPerformed
        focusOrderNo();
        fullfill();
    }//GEN-LAST:event_btnOrderSubmitActionPerformed
    
    private void btnAddToFullfillActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddToFullfillActionPerformed
        focusOrderNo();
        addToFullFill();
    }//GEN-LAST:event_btnAddToFullfillActionPerformed
    
    private void btnCancelOrderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelOrderActionPerformed
        int result = JOptionPane.showConfirmDialog((Component)this,"Do you really want to cancel the order?","Confirm Order Canceling",JOptionPane.YES_NO_OPTION);
        if( result == JOptionPane.YES_OPTION){
            validateUI();
        }
    }//GEN-LAST:event_btnCancelOrderActionPerformed
    
    private void editOrderNoKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_editOrderNoKeyPressed
        int chKeyPressed = evt.getKeyCode();
        if(chKeyPressed == evt.VK_ENTER || chKeyPressed == evt.VK_TAB) {
            
            try{
                Integer.parseInt(editOrderNo.getText().trim());
            } catch(Exception e){
                JOptionPane.showMessageDialog(this,"Please, enter valid Order Number!");
                return;
            }
            if(isFilled(true))
                return;
            
            orderData = getOrderDetails(editOrderNo.getText());
            orderData = formatStockRows(orderData);
            
            editOrderNo.setEnabled(false);
            tblOrders.setEnabled(false);
            editStockCount.requestFocus();
            editStockCount.setSelectionStart(0);
            editStockCount.setSelectionEnd(1);
            btnShowOrder.setEnabled(true);
            btnCancelOrder.setEnabled(true);
            btnOrderSubmit.setEnabled(true);
            btnPrint.setEnabled(true);
            stopThread();
        }
        
        
    }//GEN-LAST:event_editOrderNoKeyPressed
    
    private void tblOrdersMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tblOrdersMouseClicked
        try{
            if(!tblOrders.isEnabled())
                return;
            int row = tblOrders.getSelectedRow();
            curRow = row;
            String strOrderIdnt = String.valueOf(tblOrders.getModel().getValueAt(row,0));
            editOrderNo.setText(strOrderIdnt);
            editOrderNo.requestFocus();
        } catch(Exception e){
            
        }
    }//GEN-LAST:event_tblOrdersMouseClicked
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JButton btnAddToFullfill;
    private javax.swing.JButton btnCancelOrder;
    private javax.swing.JButton btnOrderSubmit;
    private javax.swing.JButton btnPrint;
    private javax.swing.JButton btnShowOrder;
    private javax.swing.ButtonGroup btngOperations;
    private javax.swing.JButton buttonStockQuery;
    private javax.swing.JComboBox comboStore;
    private javax.swing.JTextField editOrderNo;
    private javax.swing.JTextField editStockBarcode;
    private javax.swing.JTextField editStockCount;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JMenuItem jmiUndo;
    private javax.swing.JPopupMenu jpmUndo;
    private javax.swing.JLabel lblStockCount;
    private javax.swing.JLabel lblStocks;
    private javax.swing.JLabel lblStore;
    private javax.swing.JRadioButton radioAll;
    private javax.swing.ButtonGroup radioGroupNoAct;
    private javax.swing.JRadioButton radioToday;
    private javax.swing.JScrollPane scrAllStocks;
    private javax.swing.JTable tblActions;
    private javax.swing.JTable tblOrders;
    private javax.swing.JScrollPane tblPanel;
    // End of variables declaration//GEN-END:variables
    
}
