package medstore.person;

import holders.person.Person;
import java.awt.Dialog;
import java.awt.event.WindowEvent;
import java.util.HashMap;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import medstore.forms.CommonDialog;
import medstore.model.ComboDataModel;
import medstore.remote.io.forms.RemotePersonSearch;
import medstore.verifiers.UniversalInputVerifier;

public class AddPerson extends CommonDialog{
    
    public static final int ADD_MODE = 1;
    public static final int EDIT_MODE = 2;
    Person person;
    int nMode;
    RemotePersonSearch rps;
    BasePersonPanel base;
    private boolean isChanged = false;
    
    public AddPerson(Person person, Dialog parent, int mode) {        
        super(parent,true);
        setTitle("Person Details");
        try {
            person = person == null ? new Person() : person;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        this.person = person;
        nMode = mode;
        setSize(500,500);
        initComponents();
        loadPerson();
        
        setLocationRelativeTo(parent);
        isChanged = false;
        editBirthDate.setInputVerifier(new UniversalInputVerifier(UniversalInputVerifier.UNI_DATE_VERIFIER));
    }
    
    public Person getPerson() {
        return person;
    }
    
    public void loadCountries(){
        loadCountries(15);
    }
    public void loadCountries(int id){
        try {
            if(rps==null){
                rps = new RemotePersonSearch(base.getUser());
            }
            Vector data = rps.CountryData();
            ComboDataModel model = new ComboDataModel(data);
            model.selectIndexByKey(id);
            comboCountry.setModel(model) ;
            comboCountry.updateUI();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    public void loadUserTypes(){
        try {
            if(rps==null){
                rps = new RemotePersonSearch(base.getUser());
            }
            Vector data = rps.UserTypeData();
            ComboDataModel model = new ComboDataModel(data);
            comboPersonType.setModel(model) ;
            comboPersonType.updateUI();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    public void setBase(BasePersonPanel base) {
        this.base = base;
        provisionersPanel.setBase(base);        
    }
   
    public void loadPerson(){
        
        editFirstName.setText(person.prsn_fnam());
        editSurname.setText(person.prsn_lnam());
        comboAppeal.getModel().setSelectedItem(person.prsn_appl());
        editCLCode.setText(person.clcode());
        comboPersonType.getModel().setSelectedItem(person.type_abbr());
        editDiscount.setText(person.prsn_dscn());
        chkbProv1Appr.setSelected(person.prv1_good());
        chkbProv2Appr.setSelected(person.prv1_good());
        chkbCanPurchase.setSelected(person.alow_prch());
        chkbVATFree.setSelected(person.vatfree());
        editAlternateName.setText(person.altr_name());
        editStreet.setText(person.prsn_strt());
        editCode.setText(person.city_code());
        editCity.setText(person.prsn_city());
        comboCountry.getModel().setSelectedItem(person.cntr_name());        
        editUIN.setText(person.prsn_key());
        editBirthDate.setText(person.birthdate());
        editPrivatePhone.setText(person.priv_phon());
        editWorkPhone.setText(person.work_phon());
        editMobilePhone.setText(person.mobi_phon());
        editFax.setText(person.prsn_fax());
        editEmail.setText(person.prsn_eml());
        edpAddInfo.setText(person.prsn_info());
        
        Person prov1 = person.getProv1();
        Person prov2 = person.getProv2();
        Person prov3 = person.getProv3();
        
        provisionersPanel.setProv1(prov1.prsn_key(),
                prov1.prsn_fnam() + ", " + prov1.prsn_lnam(),person.prv1_comm());
        
        provisionersPanel.setProv2(prov2.prsn_key(),
                prov2.prsn_fnam() + ", " + prov2.prsn_lnam(),person.prv2_comm());
        
        provisionersPanel.setProv3(prov3.prsn_key(),
                prov3.prsn_fnam() + ", " + prov3.prsn_lnam(),person.prv3_comm());
        
        
    }
    public void savePerson() throws Exception{
        try {
            HashMap map = getBasePrsnMap();
            
            HashMap data = null;
            
            if(editFirstName.getText().trim().length()<=0 || editFirstName.getText().length()>50){
                JOptionPane.showMessageDialog(null,"Please enter valid First Name(50 characters max)!");
                return;
            }
            if(editSurname.getText().length()>50){
                JOptionPane.showMessageDialog(null,"Please enter valid Surname!(50 characters max)");
                editSurname.requestFocus();
                return;
            }
            if(editCity.getText().length()>50){
                JOptionPane.showMessageDialog(null,"Please enter valid City Name(50 characters max)!");
                editCity.requestFocus();
                return;
            }
            if(editCode.getText().length()>50){
                JOptionPane.showMessageDialog(null,"Please enter valid City Code(100 characters max)!");
                editCode.requestFocus();
                return;
            }            
            if(editCLCode.getText().length()>30){
                JOptionPane.showMessageDialog(null,"Please enter valid Client/Firm No(30 characters max)!");
                editCLCode.requestFocus();
                return;
            }
            if(editAlternateName.getText().length()>100){
                JOptionPane.showMessageDialog(null,"Please enter valid Alternative Name(100 characters max)!");
                editAlternateName.requestFocus();
                return;
            }
            
            if( !UniversalInputVerifier.SCALED2_PERSENT_Verifier(editDiscount) ){
                JOptionPane.showMessageDialog(null,"Please enter valid Discount value(0-100)!");
                return;
            }
            
            if( !UniversalInputVerifier.SCALED2_PERSENT_Verifier(provisionersPanel.getEditProv1Pers()) ){
                JOptionPane.showMessageDialog(null,"Please enter valid Provision 1 percent value(0-100%)!");
                return;
            }
            if( !UniversalInputVerifier.SCALED2_PERSENT_Verifier(provisionersPanel.getEditProv2Pers()) ){
                JOptionPane.showMessageDialog(null,"Please enter valid Provision 2 percent value(0-100%)!");
                return;
            }
            if( !UniversalInputVerifier.SCALED2_PERSENT_Verifier(provisionersPanel.getEditProv3Pers())){
                JOptionPane.showMessageDialog(null,"Please enter valid Provision 3 percent value(0-100%)!");
                return;
            }
            if(editBirthDate.getText().length()<=0){
                editBirthDate.setText("01.01.2010");
            }
            if(!UniversalInputVerifier.DATE_Verifier((JComponent)editBirthDate)){
                JOptionPane.showMessageDialog(this,"Please enter valid Birth date!");
                return;
            }
            if(editPrivatePhone.getText().length()>50){
                JOptionPane.showMessageDialog(this,"Please enter valid Private phone (50 characters max)!");
                editPrivatePhone.requestFocus();
                return;
            }
            if(editWorkPhone.getText().length()>50){
                JOptionPane.showMessageDialog(this,"Please enter valid Work phone (50 characters max)!");
                editWorkPhone.requestFocus();
                return;
            }
            if(editMobilePhone.getText().length()>50){
                JOptionPane.showMessageDialog(this,"Please enter valid Mobile phone (50 characters max)!");
                editMobilePhone.requestFocus();
                return;
            }
            if(editFax.getText().length()>50){
                JOptionPane.showMessageDialog(this,"Please enter valid Fax (50 characters max)!");
                editFax.requestFocus();
                return;
            }
            if(editEmail.getText().length()>60){
                JOptionPane.showMessageDialog(this,"Please enter valid E-mail (60 characters max)!");
                editEmail.requestFocus();
                return;
            }
            if(edpAddInfo.getText().length()>400){
                JOptionPane.showMessageDialog(this,"Please enter valid Additional Info (400 characters max)!");
                edpAddInfo.requestFocus();
                return;
            }
            
            if(nMode == ADD_MODE){
                data = RemotePersonSearch.savePerson(base.getUser(),map);
                nMode = EDIT_MODE;
            } else{
                if(nMode == EDIT_MODE){
                    data = RemotePersonSearch.updatePerson(base.getUser(),map);
                } else{
                    throw new Exception("Unknown mode!");
                }
            }
            
            
            populateChanges(data);
            dispatchEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSING));
            dispose();
            
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null,ex.getMessage());
            ex.printStackTrace();
        }
    }
    public void populateChanges(HashMap data){
        person = new Person((HashMap)data.get("person"));
        Person prov1 = new Person((HashMap)data.get("prov1"));
        Person prov2 = new Person((HashMap)data.get("prov2"));
        Person prov3 = new Person((HashMap)data.get("prov3"));
        
        person.setProv1(prov1);
        person.setProv2(prov2);
        person.setProv3(prov3);
        
        loadPerson();
    }
    public HashMap getBasePrsnMap() throws Exception{
        
        person = person == null ? new Person(new HashMap()) : person;
        person.prsn_fnam(editFirstName.getText());
        person.prsn_lnam(editSurname.getText());
        person.prsn_appl(comboAppeal.getModel().getSelectedItem());
        person.clcode(editCLCode.getText());
        int type_key = ((ComboDataModel)comboPersonType.getModel()).getCObjectAt(comboPersonType.getSelectedIndex()).nItemDBKey;
        person.type_idnt(new Integer(type_key));
        person.prsn_dscn(editDiscount.getText());
        person.prv1_good(chkbProv1Appr.isSelected());
        person.prv2_good(chkbProv2Appr.isSelected());
        person.alow_prch(chkbCanPurchase.isSelected());
        person.vatfree(chkbVATFree.isSelected());
        person.altr_name(editAlternateName.getText());
        person.prsn_strt(editStreet.getText());
        person.city_code(editCode.getText());
        person.prsn_city(editCity.getText());
        int ctry_key = ((ComboDataModel)comboCountry.getModel()).getCObjectAt(comboCountry.getSelectedIndex()).nItemDBKey;
        person.cntr_idnt(new Integer(ctry_key));        
        person.prsn_key(editUIN.getText());
        if(!UniversalInputVerifier.DATE_Verifier(editBirthDate)){
            throw new Exception("Invalid Birth date");
        }
        person.birthdate(editBirthDate.getText());
        person.priv_phon(editPrivatePhone.getText());
        person.work_phon(editWorkPhone.getText());
        person.mobi_phon(editMobilePhone.getText());
        person.prsn_fax(editFax.getText());
        person.prsn_eml(editEmail.getText());
        person.prsn_info(edpAddInfo.getText());
        
        Person prov1 = person.getProv1();
        Person prov2 = person.getProv2();
        Person prov3 = person.getProv3();
        
        person.prv1_idnt(provisionersPanel.getProv1UIN());
        person.prv1_comm(provisionersPanel.getProv1Pers());
        
        person.prv2_idnt(provisionersPanel.getProv2UIN());
        person.prv2_comm(provisionersPanel.getProv2Pers());
        
        person.prv3_idnt(provisionersPanel.getProv3UIN());
        person.prv3_comm(provisionersPanel.getProv3Pers());
        HashMap map = person.getPerson();
        map.put("cret_user_idnt",base.getUser().getUID());
        map.put("mdfc_user_idnt",base.getUser().getUID());
        
        return map;
    }
    public int getMode() {
        return nMode;
    }
    public boolean isChanged(){
        return isChanged;
    }
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jPanel4 = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        editWorkPhone = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        editMobilePhone = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        editFax = new javax.swing.JTextField();
        editPrivatePhone = new javax.swing.JTextField();
        editEmail = new javax.swing.JTextField();
        jScrollPane5 = new javax.swing.JScrollPane();
        edpAddInfo = new javax.swing.JEditorPane();
        jLabel4 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        jpanGenInfo = new javax.swing.JPanel();
        editUIN = new javax.swing.JTextField();
        editDiscount = new javax.swing.JTextField();
        editFirstName = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        editStreet = new javax.swing.JTextField();
        editCity = new javax.swing.JTextField();
        editCode = new javax.swing.JTextField();
        jLabel10 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        editSurname = new javax.swing.JTextField();
        chkbCanPurchase = new javax.swing.JCheckBox();
        comboAppeal = new javax.swing.JComboBox();
        comboPersonType = new javax.swing.JComboBox();
        comboCountry = new javax.swing.JComboBox();
        chkbProv1Appr = new javax.swing.JCheckBox();
        chkbProv2Appr = new javax.swing.JCheckBox();
        jLabel12 = new javax.swing.JLabel();
        editCLCode = new javax.swing.JTextField();
        chkbVATFree = new javax.swing.JCheckBox();
        editAlternateName = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        btnCancel = new javax.swing.JButton();
        jPanel6 = new javax.swing.JPanel();
        jLabel19 = new javax.swing.JLabel();
        editBirthDate = new javax.swing.JTextField();
        btnSave = new javax.swing.JButton();
        provisionersPanel = new medstore.orders.ProvisionersPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Add Person");

        jPanel4.setLayout(null);

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Additional Info", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 14)));
        jPanel4.setFont(new java.awt.Font("MS Sans Serif", 1, 14));
        jLabel8.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel8.setText("Private phone");
        jPanel4.add(jLabel8);
        jLabel8.setBounds(10, 20, 150, 18);

        jLabel1.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel1.setText("Work phone");
        jPanel4.add(jLabel1);
        jLabel1.setBounds(170, 20, 160, 18);

        editWorkPhone.setFont(new java.awt.Font("Tahoma", 0, 13));
        jPanel4.add(editWorkPhone);
        editWorkPhone.setBounds(170, 40, 160, 21);

        jLabel2.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel2.setText("Mobile Phone");
        jPanel4.add(jLabel2);
        jLabel2.setBounds(10, 70, 150, 18);

        editMobilePhone.setFont(new java.awt.Font("Tahoma", 0, 13));
        jPanel4.add(editMobilePhone);
        editMobilePhone.setBounds(10, 90, 150, 21);

        jLabel3.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel3.setText("Fax");
        jPanel4.add(jLabel3);
        jLabel3.setBounds(170, 70, 160, 18);

        editFax.setFont(new java.awt.Font("Tahoma", 0, 13));
        jPanel4.add(editFax);
        editFax.setBounds(170, 90, 160, 21);

        editPrivatePhone.setFont(new java.awt.Font("Tahoma", 0, 13));
        jPanel4.add(editPrivatePhone);
        editPrivatePhone.setBounds(10, 40, 150, 21);

        editEmail.setFont(new java.awt.Font("Tahoma", 0, 13));
        jPanel4.add(editEmail);
        editEmail.setBounds(10, 140, 320, 21);

        edpAddInfo.setFont(new java.awt.Font("Tahoma", 0, 13));
        jScrollPane5.setViewportView(edpAddInfo);

        jPanel4.add(jScrollPane5);
        jScrollPane5.setBounds(10, 190, 320, 70);

        jLabel4.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel4.setText("E-mail");
        jPanel4.add(jLabel4);
        jLabel4.setBounds(10, 120, 320, 18);

        jLabel18.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel18.setText("Additional Info");
        jPanel4.add(jLabel18);
        jLabel18.setBounds(10, 170, 320, 18);

        jpanGenInfo.setLayout(null);

        jpanGenInfo.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "General Info", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 14)));
        jpanGenInfo.setFont(new java.awt.Font("MS Sans Serif", 1, 14));
        editUIN.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        editUIN.setEnabled(false);
        jpanGenInfo.add(editUIN);
        editUIN.setBounds(240, 310, 70, 23);

        editDiscount.setFont(new java.awt.Font("Tahoma", 0, 13));
        editDiscount.setText("0");
        jpanGenInfo.add(editDiscount);
        editDiscount.setBounds(260, 100, 60, 21);

        editFirstName.setColumns(5);
        editFirstName.setFont(new java.awt.Font("Tahoma", 0, 13));
        jpanGenInfo.add(editFirstName);
        editFirstName.setBounds(10, 50, 100, 21);

        jLabel5.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel5.setText("UIN");
        jLabel5.setName("lblOPUIN");
        jpanGenInfo.add(jLabel5);
        jLabel5.setBounds(240, 290, 95, 18);

        jLabel6.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel6.setText("Appeal");
        jLabel6.setName("lblOPApeal");
        jpanGenInfo.add(jLabel6);
        jLabel6.setBounds(230, 30, 90, 18);

        jLabel7.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel7.setText("Type");
        jLabel7.setName("lblOPType");
        jpanGenInfo.add(jLabel7);
        jLabel7.setBounds(140, 80, 100, 18);

        jLabel9.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel9.setText("Discount %");
        jLabel9.setName("lblOPDiscount");
        jpanGenInfo.add(jLabel9);
        jLabel9.setBounds(260, 80, 70, 18);

        editStreet.setFont(new java.awt.Font("Tahoma", 0, 13));
        jpanGenInfo.add(editStreet);
        editStreet.setBounds(10, 260, 210, 21);

        editCity.setFont(new java.awt.Font("Tahoma", 0, 13));
        jpanGenInfo.add(editCity);
        editCity.setBounds(10, 310, 90, 21);

        editCode.setFont(new java.awt.Font("Tahoma", 0, 13));
        jpanGenInfo.add(editCode);
        editCode.setBounds(230, 260, 90, 21);

        jLabel10.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel10.setText("First Name");
        jLabel10.setName("lblOPFName");
        jpanGenInfo.add(jLabel10);
        jLabel10.setBounds(10, 30, 110, 18);

        jLabel11.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel11.setText("Surname");
        jLabel11.setName("lblOPLName");
        jpanGenInfo.add(jLabel11);
        jLabel11.setBounds(120, 30, 110, 18);

        jLabel14.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel14.setText("Street");
        jLabel14.setName("lblOPStreet");
        jpanGenInfo.add(jLabel14);
        jLabel14.setBounds(10, 240, 210, 18);

        jLabel15.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel15.setText("Code");
        jLabel15.setName("lblOPCode");
        jpanGenInfo.add(jLabel15);
        jLabel15.setBounds(230, 240, 90, 18);

        jLabel16.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel16.setText("City");
        jLabel16.setName("lblOPCity");
        jpanGenInfo.add(jLabel16);
        jLabel16.setBounds(10, 290, 90, 18);

        jLabel17.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel17.setText("Country");
        jLabel17.setName("lblOPCountry");
        jpanGenInfo.add(jLabel17);
        jLabel17.setBounds(110, 290, 120, 18);

        editSurname.setFont(new java.awt.Font("Tahoma", 0, 13));
        jpanGenInfo.add(editSurname);
        editSurname.setBounds(120, 50, 100, 21);

        chkbCanPurchase.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        chkbCanPurchase.setText("Can purchase");
        chkbCanPurchase.setMargin(new java.awt.Insets(0, 0, 0, 0));
        chkbCanPurchase.setName("chkOPCanPurchase");
        jpanGenInfo.add(chkbCanPurchase);
        chkbCanPurchase.setBounds(172, 130, 150, 20);

        comboAppeal.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        comboAppeal.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Frau", "Herr", "Frau Dr.", "Herr Dr.", "Mevrouw", "Meneer", "Mister", "Misses" }));
        jpanGenInfo.add(comboAppeal);
        comboAppeal.setBounds(230, 50, 90, 22);

        comboPersonType.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jpanGenInfo.add(comboPersonType);
        comboPersonType.setBounds(140, 100, 80, 22);

        comboCountry.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jpanGenInfo.add(comboCountry);
        comboCountry.setBounds(110, 310, 130, 22);

        chkbProv1Appr.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        chkbProv1Appr.setText("Prov.1 appropriate");
        chkbProv1Appr.setName("CLProv1Appropriate");
        jpanGenInfo.add(chkbProv1Appr);
        chkbProv1Appr.setBounds(10, 160, 160, 20);

        chkbProv2Appr.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        chkbProv2Appr.setText("Prov.2 appropriate");
        chkbProv2Appr.setName("CLProv2Appropriate");
        jpanGenInfo.add(chkbProv2Appr);
        chkbProv2Appr.setBounds(170, 160, 150, 20);

        jLabel12.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel12.setText("Client/Firm No");
        jLabel12.setName("lblOPFirmNo");
        jpanGenInfo.add(jLabel12);
        jLabel12.setBounds(10, 80, 130, 18);

        editCLCode.setFont(new java.awt.Font("MS Sans Serif", 0, 13));
        editCLCode.setMinimumSize(new java.awt.Dimension(11, 21));
        editCLCode.setPreferredSize(new java.awt.Dimension(11, 21));
        jpanGenInfo.add(editCLCode);
        editCLCode.setBounds(10, 100, 100, 21);

        chkbVATFree.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        chkbVATFree.setText("VAT Free ");
        chkbVATFree.setName("chkOPVatFree");
        jpanGenInfo.add(chkbVATFree);
        chkbVATFree.setBounds(10, 130, 160, 20);

        editAlternateName.setFont(new java.awt.Font("Tahoma", 0, 13));
        jpanGenInfo.add(editAlternateName);
        editAlternateName.setBounds(10, 210, 210, 21);

        jLabel13.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel13.setText("Alternative Name");
        jLabel13.setName("lblOPAltName");
        jpanGenInfo.add(jLabel13);
        jLabel13.setBounds(10, 190, 210, 18);

        btnCancel.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnCancel.setText("Cancel");
        btnCancel.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnCancel.setName("btnClCancel");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        jPanel6.setLayout(null);

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Birth Info", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 14)));
        jLabel19.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel19.setText("Birth Date");
        jPanel6.add(jLabel19);
        jLabel19.setBounds(10, 30, 100, 18);

        editBirthDate.setColumns(5);
        editBirthDate.setFont(new java.awt.Font("Tahoma", 0, 13));
        jPanel6.add(editBirthDate);
        editBirthDate.setBounds(10, 50, 100, 21);

        btnSave.setFont(new java.awt.Font("Arial", 1, 12));
        btnSave.setText("Save");
        btnSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSaveActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(170, 170, 170)
                        .add(btnCancel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(60, 60, 60)
                        .add(btnSave, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(10, 10, 10)
                        .add(jpanGenInfo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 340, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 340, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jPanel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 340, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(layout.createSequentialGroup()
                        .addContainerGap()
                        .add(provisionersPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 680, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(9, 9, 9))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(10, 10, 10)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jpanGenInfo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 350, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(layout.createSequentialGroup()
                        .add(jPanel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 80, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jPanel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 270, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(110, 110, 110)
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(80, 80, 80)
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(provisionersPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(btnCancel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnSave)))
        );
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-707)/2, (screenSize.height-566)/2, 707, 566);
    }// </editor-fold>//GEN-END:initComponents
    
    private void btnSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveActionPerformed
        
        try {
            savePerson();
            isChanged = true;
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null,ex.getMessage());
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnSaveActionPerformed
    
    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        dispose();
    }//GEN-LAST:event_btnCancelActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnSave;
    private javax.swing.JCheckBox chkbCanPurchase;
    private javax.swing.JCheckBox chkbProv1Appr;
    private javax.swing.JCheckBox chkbProv2Appr;
    private javax.swing.JCheckBox chkbVATFree;
    private javax.swing.JComboBox comboAppeal;
    private javax.swing.JComboBox comboCountry;
    private javax.swing.JComboBox comboPersonType;
    private javax.swing.JTextField editAlternateName;
    private javax.swing.JTextField editBirthDate;
    private javax.swing.JTextField editCLCode;
    private javax.swing.JTextField editCity;
    private javax.swing.JTextField editCode;
    private javax.swing.JTextField editDiscount;
    private javax.swing.JTextField editEmail;
    private javax.swing.JTextField editFax;
    private javax.swing.JTextField editFirstName;
    private javax.swing.JTextField editMobilePhone;
    private javax.swing.JTextField editPrivatePhone;
    private javax.swing.JTextField editStreet;
    private javax.swing.JTextField editSurname;
    private javax.swing.JTextField editUIN;
    private javax.swing.JTextField editWorkPhone;
    private javax.swing.JEditorPane edpAddInfo;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JPanel jpanGenInfo;
    private medstore.orders.ProvisionersPanel provisionersPanel;
    // End of variables declaration//GEN-END:variables
    
    
}
