package medstore.person;
import holders.User;
import java.awt.AWTEvent;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import medstore.SharedConstants;
import medstore.forms.CommonDialog;
import medstore.model.ComboDataModel;
import medstore.model.ComboDataRow;
import medstore.model.HashTableModel;
import medstore.model.TableUtils;

import medstore.remote.io.forms.RemotePersonSearch;

public class PersonSearch extends CommonDialog implements SharedConstants{
    public final static int PATIENT_SEARCH = 1;
    public final static int DOCTOR_SEARCH = 2;
    public final static int DEALER_SEARCH = 3;
    public final static int PROV3_SEARCH = 4;
    public final static int PROV_GOOD_SEARCH = 5;
    
    public final static String DOCTOR_STRING = "DR";
    public final static String DEALER_STRING = "MA";
    public final static String PATIENT_STRING = "PAT";
    
    public final static int UIN_INDEX = 0;
    public final static int NAME_INDEX = 1;
    public final static int PROV1_IDNT_INDEX = 5;
    public final static int PROV1_COMM_INDEX = 6;
    public final static int PROV2_IDNT_INDEX = 7;
    public final static int PROV2_COMM_INDEX = 8;
    
    
    private int[] width = {50,180,30,80,200};
    
    private User user;
    
    LinkedHashMap personRow;
    
    RemotePersonSearch rps;
    private BasePerson base;
    
    public PersonSearch(final BasePerson base, Dialog parent) throws Exception {
        super(parent,true);
        setTitle("Person search");
        super.setSize(600, 600);
        this.user = base.getUser();
        personRow = null;
        this.base = base;
        
        initComponents();
        rps = new RemotePersonSearch(user);
        loadCountries();
        loadUserTypes();
        tblPersons.getTableHeader().setReorderingAllowed(false);
        tblPersons.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt){
                if( evt.getSource() == tblPersons ) {
                    if ((evt.getModifiers() & InputEvent.BUTTON1_MASK) != 0 && evt.getClickCount()>1) {
                        int nRow = tblPersons.rowAtPoint(evt.getPoint());
                        submit(nRow);
                    }
                }
            }
        });
        TableUtils.setColumnSorter(tblPersons);
        TableUtils.setTableHeader(tblPersons);
        tblPersons.getTableHeader().setReorderingAllowed(false);
        tblPersons.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    }
    private void submit(int nRow){
        if(nRow<0)
            return;
        
        String strUIN = ((LinkedHashMap)((HashTableModel)tblPersons.getModel()).getRowAt(nRow)).get("prsn_key").toString();
        base.loadPerson(strUIN);
        setVisible(false);
    }
    public void selectComboPrsType(int key) {
        ((ComboDataModel)comboPrsType.getModel()).selectIndexByKey(key);
    }
    public void setEnableComboPrsType(boolean b) {
        comboPrsType.setEnabled(b);
    }
    public void loadCountries(){
        try {
            Vector data = rps.CountryData();
            ComboDataModel model = new ComboDataModel(data);
            model.insertCObjectAt(0,new ComboDataRow("All",-1));
            comboCountry.setModel(model) ;
            comboCountry.updateUI();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    public void loadUserTypes(){
        try {
            Vector data = rps.UserTypeData();
            ComboDataModel model = new ComboDataModel(data);
            model.insertCObjectAt(0,new ComboDataRow("All",-1));
            comboPrsType.setModel(model) ;
            comboPrsType.updateUI();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        scrPersons = new javax.swing.JScrollPane();
        tblPersons = new javax.swing.JTable();
        editKeywords = new javax.swing.JTextField();
        btnSearch = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        comboCountry = new javax.swing.JComboBox();
        comboPrsType = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        editUid = new javax.swing.JTextField();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Persons search");
        scrPersons.setFont(new java.awt.Font("Arial", 1, 12));
        tblPersons.setFont(new java.awt.Font("Arial", 1, 12));
        tblPersons.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "UIN", "Name", "Type", "Country", "Address"
            }
        ));
        tblPersons.setName("tblClSearchPersons");
        tblPersons.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                tblPersonsKeyPressed(evt);
            }
        });

        scrPersons.setViewportView(tblPersons);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 8;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 10, 10);
        getContentPane().add(scrPersons, gridBagConstraints);

        editKeywords.setFont(new java.awt.Font("Tahoma", 0, 13));
        editKeywords.setMinimumSize(new java.awt.Dimension(260, 21));
        editKeywords.setPreferredSize(new java.awt.Dimension(260, 21));
        editKeywords.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editKeywordsActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 10, 0, 0);
        getContentPane().add(editKeywords, gridBagConstraints);

        btnSearch.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnSearch.setText("Search");
        btnSearch.setName("btnSearch");
        btnSearch.setPreferredSize(new java.awt.Dimension(73, 26));
        btnSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSearchActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 11);
        getContentPane().add(btnSearch, gridBagConstraints);

        jLabel1.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel1.setText("Enter Keywords:");
        jLabel1.setName("lblEnterKeyWords");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 37;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 10, 0, 0);
        getContentPane().add(jLabel1, gridBagConstraints);

        comboCountry.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        comboCountry.setMaximumSize(new java.awt.Dimension(20, 24));
        comboCountry.setMinimumSize(new java.awt.Dimension(20, 24));
        comboCountry.setPreferredSize(new java.awt.Dimension(20, 24));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.ipadx = 92;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 10, 0, 0);
        getContentPane().add(comboCountry, gridBagConstraints);

        comboPrsType.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        comboPrsType.setMaximumSize(new java.awt.Dimension(10, 24));
        comboPrsType.setMinimumSize(new java.awt.Dimension(10, 24));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.ipadx = 72;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 10, 0, 10);
        getContentPane().add(comboPrsType, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel2.setText("Country");
        jLabel2.setName("lblOPCountry");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 11;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 10, 0, 0);
        getContentPane().add(jLabel2, gridBagConstraints);

        jLabel3.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel3.setText("Person Type");
        jLabel3.setName("lblPersonType");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 11;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 10, 0, 0);
        getContentPane().add(jLabel3, gridBagConstraints);

        jLabel4.setFont(new java.awt.Font("Tahoma", 1, 12));
        jLabel4.setText("No.:");
        jLabel4.setMaximumSize(new java.awt.Dimension(34, 21));
        jLabel4.setMinimumSize(new java.awt.Dimension(34, 21));
        jLabel4.setName("lblUID");
        jLabel4.setPreferredSize(new java.awt.Dimension(34, 21));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 12, 0, 12);
        getContentPane().add(jLabel4, gridBagConstraints);

        editUid.setFont(new java.awt.Font("Tahoma", 1, 12));
        editUid.setMaximumSize(new java.awt.Dimension(100, 21));
        editUid.setMinimumSize(new java.awt.Dimension(100, 21));
        editUid.setPreferredSize(new java.awt.Dimension(100, 21));
        editUid.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editUidActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 40, 0, 40);
        getContentPane().add(editUid, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-619)/2, (screenSize.height-305)/2, 619, 305);
    }// </editor-fold>//GEN-END:initComponents
    
    private void tblPersonsKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_tblPersonsKeyPressed
        if( evt.getSource() == tblPersons && evt.getKeyCode() == evt.VK_ENTER) {            
            int nRow = tblPersons.getSelectedRow();
            submit(nRow);
        }
    }//GEN-LAST:event_tblPersonsKeyPressed
    
    private void editUidActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editUidActionPerformed
        btnSearch.doClick();
        editUid.requestFocus();
    }//GEN-LAST:event_editUidActionPerformed
    
    private void editKeywordsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editKeywordsActionPerformed
        btnSearch.doClick();
        editKeywords.requestFocus();
    }//GEN-LAST:event_editKeywordsActionPerformed
    
    private void btnSearchActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSearchActionPerformed
        try {
            int nCountryId = ((ComboDataModel)comboCountry.getModel()).getCObjectAt(comboCountry.getSelectedIndex()).nItemDBKey;
            int nTypeId = ((ComboDataModel)comboPrsType.getModel()).getCObjectAt(comboPrsType.getSelectedIndex()).nItemDBKey;
            String uid = editUid.getText().trim();
            try {
                if(uid.trim().length()>0)
                    Long.parseLong(uid);
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(this,"UIN - Only numbers (without white spaces)");
                uid = "";
                return;
            }
            Cursor hourglassCursor = new Cursor(Cursor.WAIT_CURSOR);
            setCursor(hourglassCursor);
            Vector data = rps.findPerson(editKeywords.getText(),nCountryId,nTypeId,editUid.getText());
            String columnNames[] = {"UIN","Name","Type","Country","Address"};
            HashTableModel model = new HashTableModel(data,columnNames);
            tblPersons.setModel(model);
            TableUtils.setTableColumnsWidth(tblPersons, width);
            tblPersons.updateUI();
        } catch (Exception ex) {
            ex.printStackTrace();
        }finally{
            Cursor normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
            setCursor(normalCursor);
        }
    }//GEN-LAST:event_btnSearchActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnSearch;
    private javax.swing.JComboBox comboCountry;
    private javax.swing.JComboBox comboPrsType;
    private javax.swing.JTextField editKeywords;
    private javax.swing.JTextField editUid;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JScrollPane scrPersons;
    private javax.swing.JTable tblPersons;
    // End of variables declaration//GEN-END:variables
    
}
