package medstore.print.orders;

import java.math.BigDecimal;
import java.util.ArrayList;

public class Invoice implements java.io.Serializable{
    private ArrayList articles;
    private ArrayList articleCounts;
    
    private Customer customer;
    private String date;
    private String invoiceNumber;
    private String invoiceVersand;
    private String invoiceDiscount;
    private String invoiceSum;
    private String invoiceVat;
    private String invoiceTotalSum;
    private String invoiceVatPerc;
    private String customerId;
    private String invoiceBarcode;
    private String invoiceDicountPerc;
    private int type = -1; //1 Invoice, 2 Credit node
    
    public Invoice(final Customer customer, final String date,
            final String invoiceNumber, final String invoiceVersand, final String invoiceDiscount,
            final String invoiceSum, final String invoiceVat, final String invoiceTotalSum, final String invoiceVatPerc, final String customerId,
            final String invoiceBarcode, final String invoiceDicountPerc) {
        this.customer = customer;
        this.date = date;
        this.invoiceNumber = invoiceNumber;
        this.invoiceVersand = invoiceVersand;
        this.invoiceDiscount = invoiceDiscount;
        this.invoiceSum = invoiceSum;
        this.invoiceVat = invoiceVat;
        this.invoiceTotalSum = invoiceTotalSum;
        this.invoiceVatPerc = invoiceVatPerc;
        this.customerId = customerId;
        this.invoiceBarcode = invoiceBarcode;
        this.invoiceDicountPerc = invoiceDicountPerc;
        this.articles = new ArrayList();
        this.articleCounts = new ArrayList();
    }
    public void setType(int type){
        this.type = type;
    }
    public synchronized void addArticle( Article article) {
        final int index = articles.indexOf(article);
        if (index == -1) {
            articles.add(article);
            articleCounts.add(new Integer(1));
        } else {
            final Integer oldCount = (Integer) articleCounts.get(index);
            articleCounts.set(index, new Integer(oldCount.intValue() + 1));
        }
    }
    
    public synchronized void removeArticle(final Article article) {
        final int index = articles.indexOf(article);
        if (index != -1) {
            final Integer oldCount = (Integer) articleCounts.get(index);
            if (oldCount.intValue() == 1) {
                articleCounts.remove(index);
                articles.remove(index);
            } else {
                articleCounts.set(index, new Integer(oldCount.intValue() - 1));
            }
        }
    }
    public synchronized void setArticles(ArrayList articles){
        for(int i = 0; i<articles.size(); i++){
            addArticle((Article)articles.get(i));
        }
    }
    
    public Article getArticle(final int index) {
        return (Article) articles.get(index);
    }
    
    public int getArticleCount(final int index) {
        final Integer i = (Integer) articleCounts.get(index);
        return i.intValue();
    }
    
    public int getArticleCount() {
        return articles.size();
    }
    
    public Customer getCustomer() {
        return customer;
    }
    
    public String getDate() {
        return date;
    }
    
    public String getInvoiceNumber() {
        return invoiceNumber;
    }
    public String getInvoiceVersand() {
        return invoiceVersand;
    }
    public String getInvoiceDiscount() {
        return invoiceDiscount;
    }
    public String getInvoiceVat() {
        return this.invoiceVat;
    }
    public String getInvoiceSum() {
        return this.invoiceSum;
    }
    public String getInvoiceVatPerc() {
        return this.invoiceVatPerc;
    }
    public String getInvoiceTotalSum() {
        return this.invoiceTotalSum;
    }
    public String getCustomerInvoiceId() {
        return this.customerId;
    }
    public String getInvoiceBarcode(){
        return invoiceBarcode;
    }
    public String getInvoiceDicountPerc(){
        return invoiceDicountPerc;
    }
}
