package medstore.print.orders;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class InvoiceTableModel extends AbstractTableModel implements java.io.Serializable{
    private static Class[] COLUMN_TYPES =
    {
        Invoice.class,
                String.class, String.class, String.class,
                String.class, String.class, String.class,
                String.class, String.class, String.class,
                Date.class,   String.class, String.class,
                String.class, String.class, String.class,
                Float.class,  Integer.class,
                String.class, String.class, String.class,
                String.class, String.class, String.class, String.class, 
                String.class, String.class, String.class
    };
    
    private static String[] COLUMN_NAMES =
    {
        "invoice",
                "customer.firstName", "customer.lastName", "customer.street",
                "customer.town", "customer.postalCode", "customer.country",
                "customer.salutation","customer.altAddressCaption","customer.altAddress",
                "customer.macro","invoice.date", "invoice.number",
                "article.number", "article.name", "article.price",
                "article.count", "article.total",
                "invoice.versand","invoice.discount","invoice.sum","invoice.vat",
                "invoice.totalSum","invoice.vatperc","invoice.customerId","invoice.barcode","customer.invAltName",
                "invoice.invoiceDicountPerc", "customer.packmacro"
    };
    
    private transient Invoice[] invoicePerRow;
    private transient Article[] articlesPerRow;
    private transient int[] articleCountPerRow;
    
    private ArrayList invoices;
    private int totalSize;
    
    public InvoiceTableModel() {
        invoices = new ArrayList();
    }
    
    public void addInvoice(final Invoice invoice) {
        invoices.add(invoice);
        invalidateCaches();
        fireTableDataChanged();
    }
    
    public void removeInvoice(final Invoice invoice) {
        invoices.remove(invoice);
        invalidateCaches();
        fireTableDataChanged();
    }
    
    public Invoice getInvoice(final int invoice) {
        return (Invoice) invoices.get(invoice);
    }
    
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < invoices.size(); i++) {
            final Invoice inv = getInvoice(i);
            size += inv.getArticleCount();
        }
        this.totalSize = size;
        this.invoicePerRow = null;
        this.articlesPerRow = null;
        this.articleCountPerRow = null;
    }
    
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    
    private void fillCache() {
        if (invoicePerRow != null && articlesPerRow != null) {
            // nothing to do...
            return;
        }
        // ensure that we have enough space ...
        this.invoicePerRow = new Invoice[totalSize];
        this.articlesPerRow = new Article[totalSize];
        this.articleCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = invoices.size();
        for (int i = 0; i < invoiceSize; i++) {
            final Invoice inv = (Invoice) invoices.get(i);
            final int articleCount = inv.getArticleCount();
            for (int ac = 0; ac < articleCount; ac++) {
                invoicePerRow[currentRow] = inv;
                articlesPerRow[currentRow] = inv.getArticle(ac);
                articleCountPerRow[currentRow] = inv.getArticleCount(ac);
                currentRow += 1;
            }
        }
    }
    
   
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        // just make sure we can access the invoices by the array
        fillCache();
        final Invoice inv = invoicePerRow[rowIndex];
        final Article art = articlesPerRow[rowIndex];
        
        switch (columnIndex) {
            case 0:
                return inv;
            case 1:
                return inv.getCustomer().getFirstName();
            case 2:
                return inv.getCustomer().getLastName();
            case 3:
                return inv.getCustomer().getStreet();
            case 4:
                return inv.getCustomer().getTown();
            case 5:
                return inv.getCustomer().getPostalCode();
            case 6:
                return inv.getCustomer().getCountry();
            case 7:
                return inv.getCustomer().getSalutation();
            case 8:
                return inv.getCustomer().getAltAddressCaption();
            case 9:
                return inv.getCustomer().getAltAddress();
            case 10:
                return inv.getCustomer().getMacro();
            case 11:
                return inv.getDate();
            case 12:
                return inv.getInvoiceNumber();
            case 13:
                return art.getArticleNumber();
            case 14:
                return art.getName();
            case 15:
                return art.getPrice();
            case 16:
                return art.getArticleCount();
            case 17:
                return art.getArticleTotal();
            case 18:
                return inv.getInvoiceVersand();
            case 19:
                return inv.getInvoiceDiscount();
            case 20:
                return inv.getInvoiceSum();
            case 21:
                return inv.getInvoiceVat();
            case 22:
                return inv.getInvoiceTotalSum();
            case 23:
                return inv.getInvoiceVatPerc();
            case 24:
                return inv.getCustomerInvoiceId();
            case 25:
                return inv.getInvoiceBarcode();
            case 26:
                return inv.getCustomer().getInvAltName();
            case 27: 
                return inv.getInvoiceDicountPerc();
            case 28: 
                return inv.getCustomer().getPackMacro();
        }
        throw new IndexOutOfBoundsException("ColumnIndex is invalid: " + columnIndex);
    }
}
