package medstore.print.orders;

import medstore.remote.io.RemoteURL;
import holders.User;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import medstore.String.StringUtils;
import medstore.math.BigDec;
import medstore.print.PrintPreview;
import medstore.print.RepotUtil;
import medstore.remote.io.ServletWriter;
import org.jfree.report.JFreeReport;
import org.jfree.report.modules.gui.print.PrintUtil;

public class OrderPrintUtils {
    
    
    private boolean packing = false;
    private boolean hidediscont = true;
    User user;
    Invoice invoice;
    
    public OrderPrintUtils(String strOrderIdnt, User user) throws Exception{
        packing = false;
        this.user = user;
        invoice = loadOrderInfo(strOrderIdnt);
    }
    
    
    
    
    private Invoice loadOrderInfo(String v_nr) throws Exception{
        packing = false;
        
        HashMap map = new HashMap();
        map.put("v_nr",v_nr);
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)map};
        URL url = new URL(RemoteURL.printInvoice);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        HashMap data = (HashMap)result.readObject();
        
        HashMap customer_map = (HashMap)data.get("customer");
        
        String firstName = StringUtils.clearString(customer_map.get("firstName"));
        String lastName = StringUtils.clearString(customer_map.get("lastName"));
        String salutation = StringUtils.clearString(customer_map.get("salutation"));
        String street = StringUtils.clearString(customer_map.get("street"));
        String postalCode = StringUtils.clearString(customer_map.get("postalCode"));
        String town = StringUtils.clearString(customer_map.get("town"));
        String country = StringUtils.clearString(customer_map.get("country"));
        String altAddressCaption = StringUtils.clearString(customer_map.get("altAddressCaption"));
        String invAltAddress = StringUtils.clearString(customer_map.get("invAltAddress"));
        String invmacro = StringUtils.clearString(customer_map.get("invmacro"));
        String invAltName = StringUtils.clearString(customer_map.get("invAltName"));
        String packmacro = StringUtils.clearString(customer_map.get("packmacro"));
        
        String date = StringUtils.clearString(data.get("date"));
        String invoiceNumber = StringUtils.clearString(data.get("invoiceNumber"));
        String invoiceVersand = StringUtils.clearString(data.get("invoiceVersand"));
        String invoiceDiscount = StringUtils.clearString(data.get("invoiceDiscount"));
        String invoiceSum = StringUtils.clearString(data.get("invoiceSum"));
        String invoiceVat = StringUtils.clearString(data.get("invoiceVat"));
        String invoiceTotalSum = StringUtils.clearString(data.get("invoiceTotalSum"));
        String invoiceVatPerc = StringUtils.clearString(data.get("invoiceVatPerc"));
        String customerId = StringUtils.clearString(data.get("customerId"));
        String invoiceBarcode = StringUtils.clearString(data.get("invoiceBarcode"));
        String invoiceDicountPerc = StringUtils.clearString(data.get("invoiceDicountPerc"));
        
        Customer customer = new Customer(firstName, lastName, salutation, street, postalCode, town, country,
                altAddressCaption, invAltAddress, invmacro, invAltName, packmacro);
        Invoice invoice = new Invoice(customer, date, invoiceNumber, invoiceVersand, invoiceDiscount, invoiceSum,
                invoiceVat, invoiceTotalSum, invoiceVatPerc, customerId, invoiceBarcode, invoiceDicountPerc);
        
        int invoiceType = new Integer(data.get("invoiceType").toString());
        invoice.setType(invoiceType);
        
        ArrayList articles_remote = (ArrayList)data.get("articles");
        ArrayList articles = new ArrayList();
        
        for(int i = 0; i < articles_remote.size(); i++){
            HashMap art = (HashMap)articles_remote.get(i);
            String articleNumber = StringUtils.clearString(art.get("articleNumber"));
            String name = StringUtils.clearString(art.get("name"));
            String price = StringUtils.clearString(art.get("price"));
            String count = StringUtils.clearString(art.get("count"));
            String total = StringUtils.clearString(art.get("total"));
            
            price = BigDec.scaledRoundHalfUp(price,2);
            total = BigDec.scaledRoundHalfUp(total,2);
            
            final Article article = new Article(articleNumber, name, price, count, total);
            articles.add(article);
        }
        invoice.setArticles(articles);
        
        packing = StringUtils.clearString(invAltAddress,"").length() > 0;
        
        HashMap settings = (HashMap)data.get("settings");
        hidediscont = ((Boolean)settings.get("hidediscont")).booleanValue();
        return invoice;
    }
    
    public void printInvoice(boolean preview)throws Exception{
        
        InvoiceTableModel data = new InvoiceTableModel();
        data.addInvoice(invoice);
        SimplePrintUtils spUtils = new SimplePrintUtils(data, "/medstore/print/orders/invoice.xml","Invoice");
        final JFreeReport report = spUtils.createReport();
        
        try{
            spUtils.ShowHideItems(report, hidediscont);
        } catch(Exception e){
            e.printStackTrace();
        }
        if(preview==false){
            PrintUtil.print(report);
            return;
        }
        PrintPreview prv = new PrintPreview(report);
        prv.setSize(700,800);
        prv.setVisible(true);
    }
    
    public void printPacking(boolean preview)throws Exception{
        if(packing == false)
            return;
        
        InvoiceTableModel data = new InvoiceTableModel();
        data.addInvoice(invoice);
        
        SimplePrintUtils spUtils = new SimplePrintUtils(data, "/medstore/print/orders/packing.xml","Packing");
        
        final JFreeReport report = spUtils.createReport();
        
        try {
            spUtils.ShowHideItems(report, false);
        } catch(Exception e) {
        }
        
        if(preview==false){
            PrintUtil.print(report);
            return;
        }
        PrintPreview prv = new PrintPreview(report);
        prv.setSize(700,800);
        prv.setVisible(true);
        
    }
    
}