package medstore.remote.io.forms;

import medstore.remote.io.RemoteURL;
import holders.User;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.net.URL;
import java.util.HashMap;
import java.util.Vector;
import javax.swing.JComboBox;
import medstore.model.ComboDataModel;
import medstore.remote.io.ServletWriter;

public class RemotePersonSearch {
    public static final String COUNTRY_DATA_KEY = "ctry";
    public static final String USER_TYPES_KEY = "user_types";
    
    private User user;
    private HashMap data;
    
    public RemotePersonSearch(User user) throws Exception{
        this.user = user;
        loadData();
    }
    public void loadData() throws Exception{
        URL url = new URL(RemoteURL.PersonSearch);
        Serializable obj = (Serializable)user.getUserMap();
        ObjectInputStream result = ServletWriter.postObject(url,obj);
        data = (HashMap)result.readObject();
        
    }
    public HashMap getData(){
        return data;
    }
    public Vector CountryData() throws Exception{
        return (Vector)data.get(COUNTRY_DATA_KEY) ;
    }
    
    public Vector UserTypeData() throws Exception{
        return (Vector)data.get(USER_TYPES_KEY) ;
    }
    
    public Vector findPerson(String keyWords, int ctry, int user_type, String uid) throws Exception{
        HashMap map = new HashMap();
        keyWords = keyWords == null ? "" : keyWords.trim();
        map.put("keyWords", keyWords);
        map.put("user_type",user_type);
        map.put("ctry",ctry);
        map.put("uid",uid);
        
        URL url = new URL(RemoteURL.findPerson);
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)map};
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        
        return (Vector)result.readObject();
    }
    
    public static HashMap loadPerson(User user, String uid) throws Exception{
        HashMap map = new HashMap();
        map.put("uid",uid);
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)map};
        URL url = new URL(RemoteURL.loadPerson);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        
        return (HashMap)result.readObject();
    }
    public static HashMap loadPersonByOrder(User user, String v_nr) throws Exception{
        HashMap map = new HashMap();
        map.put("v_nr",v_nr);
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)map};
        URL url = new URL(RemoteURL.loadPersonByOrder);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        
        return (HashMap)result.readObject();
    }
    public static String placeOrder(User user, HashMap data) throws Exception{        
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)data};
        URL url = new URL(RemoteURL.placeOrder);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        String v_nr = String.valueOf(result.readObject());
        return v_nr;
    }
    public static Vector loadPercent(User user) throws Exception{        
        Serializable[] obj = {(Serializable)user.getUserMap()};
        URL url = new URL(RemoteURL.loadPercent);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        Vector data = (Vector)result.readObject();
        return data;
    }
    public static HashMap savePerson(User user, HashMap person) throws Exception{        
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)person};
        URL url = new URL(RemoteURL.savePerson);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        String prsn_key = (String)result.readObject();
        HashMap data = loadPerson(user,prsn_key);        
        return (HashMap)data;
    }
    public static HashMap updatePerson(User user, HashMap person) throws Exception{        
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)person};
        URL url = new URL(RemoteURL.updatePerson);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        String prsn_key = (String)result.readObject();
        HashMap data = loadPerson(user,prsn_key);        
        return (HashMap)data;
    }
    public static HashMap payOrder(User user, HashMap data) throws Exception{        
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)data};
        URL url = new URL(RemoteURL.payOrder);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);
        String v_nr = (String)result.readObject();
        data = loadPerson(user,v_nr);        
        return (HashMap)data;
    }
    public static String creditNote(User user, HashMap data) throws Exception{        
        Serializable[] obj = {(Serializable)user.getUserMap(),(Serializable)data};
        URL url = new URL(RemoteURL.creditNote);
        ObjectInputStream result = ServletWriter.postObjects(url,obj);        
        return (String)result.readObject();
    }
}
