package medstore.stock;
import java.awt.Dialog;
import java.awt.Rectangle;
import java.awt.event.InputEvent;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import medstore.String.StringUtils;
import medstore.forms.CommonDialog;
import medstore.model.HashTableModel;
import medstore.model.TableUtils;

public class StockSelectionForm extends CommonDialog {
    public static final int STOCK_ABRV_INDEX = 0;
    public static final int STOCK_NAME_INDEX = 1;
    public static final int STOCK_KEY_INDEX = 2;
    
    private Vector data;
    private BaseStockSelection base;
    
    private LinkedHashMap lastSelected;
    
    String[] stockColNames = new String [] {"Stock Name","Abbr."};
    
    public StockSelectionForm(BaseStockSelection base, Dialog parent) {
        super(parent,true);
        setTitle("Stock selection!");
        initComponents();
        TableUtils.setTableHeader(tblStocks);
        TableUtils.setColumnSorter(tblStocks);
        tblStocks.getTableHeader().setReorderingAllowed(false);
        tblStocks.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        data = createModel((Vector)base.getStocks());
        setStockModel((Vector)data.clone());
        this.base = base;
        lastSelected = null;
    }
    public void setSelected(LinkedHashMap row){
        if(row.getClass()!=LinkedHashMap.class){
            return;
        }
        lastSelected = row;
    }
    public Vector createModel(Vector data){
        Vector copy = data == null ? data : (Vector)data.clone();
        
        for(int i = 0; i < copy.size(); i++){
            LinkedHashMap row = (LinkedHashMap)copy.get(i);
            LinkedHashMap new_row = formatRow(row);
            copy.set(i,new_row);
        }
        return copy;
    }
    public void setStockModel(Vector data){
        try{
            Vector copy = data == null ? data : (Vector)data.clone();
            
            HashTableModel model = new HashTableModel(copy,stockColNames);
            tblStocks.setModel(model);
            ((AbstractTableModel)tblStocks.getModel()).fireTableDataChanged();
            try{
                tblStocks.updateUI();
            } catch(Exception e){
                
            }
        } catch(Exception e){
            JOptionPane.showMessageDialog(null,"Can't set stock model!");
        }
    }
    public static LinkedHashMap formatRow(LinkedHashMap row){
        String name = StringUtils.clearString(row.get(Stock.stck_name));
        String abbr = StringUtils.clearString(row.get(Stock.stck_abrv));
        String key = StringUtils.clearString(row.get(Stock.stck_key));
        LinkedHashMap new_row = new LinkedHashMap();
        new_row.put(Stock.stck_name,name);
        new_row.put(Stock.stck_abrv,abbr);
        new_row.put(Stock.stck_key,key);
        return new_row;
    }
    public void submit(){
        int nRow = tblStocks.getSelectedRow();
        if(nRow < 0){
            JOptionPane.showMessageDialog(null,"Please select a stock first!");
            return;
        }
        LinkedHashMap row = ((HashTableModel)tblStocks.getModel()).getRowAt(nRow);
        base.setSelectedStock(row);
        setVisible(false);
    }
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        scrStocks = new javax.swing.JScrollPane();
        tblStocks = new javax.swing.JTable();
        btnSubmit = new javax.swing.JButton();
        editSearchText = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("Stock Selection");
        scrStocks.setFocusable(false);
        scrStocks.setFont(new java.awt.Font("Tahoma", 1, 13));
        tblStocks.setFont(new java.awt.Font("Arial", 1, 12));
        tblStocks.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Abbr.", "Stock Name"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        tblStocks.setName("tblStockSelStocks");
        tblStocks.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                tblStocksKeyPressed(evt);
            }
        });
        tblStocks.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                tblStocksMousePressed(evt);
            }
        });

        scrStocks.setViewportView(tblStocks);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 297;
        gridBagConstraints.ipady = 123;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 10, 10, 10);
        getContentPane().add(scrStocks, gridBagConstraints);

        btnSubmit.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        btnSubmit.setText("Select");
        btnSubmit.setName("btnStockSelSelect");
        btnSubmit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSubmitActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.ipadx = 17;
        gridBagConstraints.ipady = -1;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 10, 0);
        getContentPane().add(btnSubmit, gridBagConstraints);

        editSearchText.setFont(new java.awt.Font("Tahoma", 0, 13));
        editSearchText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editSearchTextActionPerformed(evt);
            }
        });
        editSearchText.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                editSearchTextKeyPressed(evt);
            }
        });
        editSearchText.addCaretListener(new javax.swing.event.CaretListener() {
            public void caretUpdate(javax.swing.event.CaretEvent evt) {
                editSearchTextCaretUpdate(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 10, 0, 10);
        getContentPane().add(editSearchText, gridBagConstraints);

        jLabel1.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel1.setText("Search string:");
        jLabel1.setName("lblStockSelSearchStr");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 22;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 0);
        getContentPane().add(jLabel1, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        jLabel2.setText("Stocks");
        jLabel2.setName("lblStockSelStocks");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.ipadx = 39;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 0);
        getContentPane().add(jLabel2, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-348)/2, (screenSize.height-308)/2, 348, 308);
    }// </editor-fold>//GEN-END:initComponents
    
    private void editSearchTextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editSearchTextActionPerformed
        int nRow = tblStocks.getSelectedRow();
        if(nRow < 0){
            return;
        }
        submit();
    }//GEN-LAST:event_editSearchTextActionPerformed
    
    private void btnSubmitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSubmitActionPerformed
        submit();
    }//GEN-LAST:event_btnSubmitActionPerformed
    
    private void tblStocksMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tblStocksMousePressed
        if( evt.getSource() == tblStocks){
            if ((evt.getModifiers() & InputEvent.BUTTON1_MASK) != 0 && evt.getClickCount()>1) {
                submit();
            }
        }
    }//GEN-LAST:event_tblStocksMousePressed
    
    private void editSearchTextKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_editSearchTextKeyPressed
        if( evt.getKeyCode()!= evt.VK_ENTER )
            return;
        submit();
    }//GEN-LAST:event_editSearchTextKeyPressed
    
    private void tblStocksKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_tblStocksKeyPressed
        if( evt.getKeyCode()!= evt.VK_ENTER )
            return;
        submit();
    }//GEN-LAST:event_tblStocksKeyPressed
    
    private void editSearchTextCaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_editSearchTextCaretUpdate
        if(evt.getSource() == editSearchText){
            String strSearchText = editSearchText.getText();
            if(strSearchText.trim().length()<=0){
                ((HashTableModel)tblStocks.getModel()).setData(data.clone());
                tblStocks.updateUI();
                return;
            }
            
            
            tblStocks.getSelectionModel().setSelectionInterval(0,0);
            tblStocks.scrollRectToVisible(new Rectangle(0,0,0,0));
            ((HashTableModel)tblStocks.getModel()).clearData();
            
            for(int i = 0; i < data.size(); i++){
                String name = StringUtils.clearString(((LinkedHashMap)data.get(i)).get("stck_name"));
                if(name.toUpperCase().startsWith(strSearchText.toUpperCase())){
                    LinkedHashMap row = formatRow((LinkedHashMap)data.get(i));
                    ((HashTableModel)tblStocks.getModel()).addRow(row);
                    
                }
            }
            tblStocks.updateUI();
        }
    }//GEN-LAST:event_editSearchTextCaretUpdate
        // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnSubmit;
    private javax.swing.JTextField editSearchText;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane scrStocks;
    private javax.swing.JTable tblStocks;
    // End of variables declaration//GEN-END:variables
}