package medstore.stock.nomenclature;

import holders.User;
import java.awt.Dialog;
import java.awt.Window;
import java.awt.event.InputEvent;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import medstore.Base;
import medstore.String.StringUtils;
import medstore.forms.FormConvertor;
import medstore.model.HashTableModel;
import medstore.model.TableUtils;
import medstore.remote.io.stock.RemoteStocks;
import medstore.stock.Stock;
import medstore.stock.StockList;

public class StockListForm extends javax.swing.JFrame implements StockBase{
    private Base base;
    AddEditStock addEditStock;
    
    private String[] colnames = {"Name","ABBREV.","Group","Barcode","Content Barcode","Count In Packet","Stock Limit","BUY PRICE","MIDDLE PRICE","SELL PRICE"};
    
    public StockListForm(Base base) {
        initComponents();
        setSize(979, 349);
        tblAllStocks.getTableHeader().setReorderingAllowed(false);
        tblAllStocks.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        TableUtils.setColumnSorter(tblAllStocks);
        TableUtils.setTableHeader(tblAllStocks);
        this.base = base;
        setLocationByPlatform(true);
        
        addEditStock = null;
    }
    
    public void saveStock(){
        LinkedHashMap row = addEditStock.getProduct();
        int mode = addEditStock.getMode();
        
        HashMap data = new HashMap();
        data.put("product",row);
        try {
            if(mode == AddEditStock.ADD_MODE){
                RemoteStocks.saveStock(base.getUser(),data);
            }
            else{
                RemoteStocks.updateStock(base.getUser(),data);
            }
            loadProducts();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void loadProducts(){
        Vector data = new Vector();
        try {
            data = RemoteStocks.stockList(getUser());
            setProducts(data);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(this,"Can't load products!\nError:"+ex.getMessage());
            ex.printStackTrace();
            dispose();
        }
    }
    public Vector getComposite(){
        Vector data = ((HashTableModel)tblAllStocks.getModel()).getDataVector();
        if(data==null || data.size()<=0){
            return null;
        }
        Vector composite_data = new Vector();
        for(int i = 0; i < data.size(); i ++){
            LinkedHashMap map = (LinkedHashMap)data.get(i);
            map = StockList.formatRow(map);
            String composite = StringUtils.clearString(map.get(Stock.composite),"0");
            if(composite.equalsIgnoreCase("1")){
                composite_data.add(map);
            }            
        }
        return composite_data;
    }
    public void editStock(){
        int nRow = tblAllStocks.getSelectedRow();
        if(nRow<0)
            return;
        LinkedHashMap row = ((HashTableModel)tblAllStocks.getModel()).getRowAt(nRow);
        Dialog parent = FormConvertor.convertor((Window)this);
        addEditStock = new AddEditStock(parent, AddEditStock.EDIT_MODE);
        addEditStock.setBase(this);
        addEditStock.setProducts(row);
        addEditStock.setComposite(getComposite());
        addEditStock.setVisible(true);
    }
    
    public void addStock(){        
        Dialog parent = FormConvertor.convertor((Window)this);
        addEditStock = new AddEditStock(parent, AddEditStock.ADD_MODE);
        addEditStock.setBase(this);
        addEditStock.setProducts(new LinkedHashMap());
        addEditStock.setComposite(getComposite());
        addEditStock.setVisible(true);
    }
    public void setProducts(Vector data){
        if(data==null || data.size()<=0){
            return;
        }
        for(int i = 0; i < data.size(); i ++){
            LinkedHashMap map = (LinkedHashMap)data.get(i);
            map = StockList.formatRow(map);
            data.set(i,map);
        }
        HashTableModel model = new HashTableModel(data,colnames);
        model.formatDecimal(Stock.pric_buy);
        model.formatDecimal(Stock.pric_mid);
        model.formatDecimal(Stock.pric_sell);
        tblAllStocks.setModel(model);
        
        TableUtils.setColumnWidth( tblAllStocks, 0, 255 );//NAME
        TableUtils.setColumnWidth( tblAllStocks, 1, 78 );//STCK_ABRV
        TableUtils.setColumnWidth( tblAllStocks, 2, 50 );//GROUP
        TableUtils.setColumnWidth( tblAllStocks, 3, 80 );//STCK_BRCD
        TableUtils.setColumnWidth( tblAllStocks, 4, 140 );//CNTN_BCOD
        TableUtils.setColumnWidth( tblAllStocks, 5, 130 );//STCK_PCNT
        TableUtils.setColumnWidth( tblAllStocks, 6, 90 );//STCK_LIMT
        TableUtils.setColumnWidth( tblAllStocks, 7, 90 );//PRIC_BUY
        TableUtils.setColumnWidth( tblAllStocks, 8, 119 );//PRIC_MID
        TableUtils.setColumnWidth( tblAllStocks, 9, 86 );//PRIC_SELL
        
        tblAllStocks.updateUI();
        
    }
    public Base getBase() {
        return base;
    }
    
    public void setBase(Base base) {
        this.base = base;
    }
    
    public User getUser(){
        return base.getUser();
    }
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        scrAllStocks = new javax.swing.JScrollPane();
        tblAllStocks = new javax.swing.JTable();
        jPanel2 = new javax.swing.JPanel();
        btnAddStock = new javax.swing.JButton();
        btnEditStock = new javax.swing.JButton();
        btnDeleteStock = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        scrAllStocks.setFont(new java.awt.Font("MS Sans Serif", 1, 13));
        scrAllStocks.setPreferredSize(new java.awt.Dimension(454, 404));
        tblAllStocks.setFont(new java.awt.Font("Tahoma", 0, 13));
        tblAllStocks.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        tblAllStocks.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                tblAllStocksMouseClicked(evt);
            }
        });

        scrAllStocks.setViewportView(tblAllStocks);

        jPanel2.setLayout(new java.awt.GridBagLayout());

        btnAddStock.setFont(new java.awt.Font("Arial", 1, 12));
        btnAddStock.setText("Add");
        btnAddStock.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddStockActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        jPanel2.add(btnAddStock, gridBagConstraints);

        btnEditStock.setFont(new java.awt.Font("Arial", 1, 12));
        btnEditStock.setText("Edit");
        btnEditStock.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEditStockActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        jPanel2.add(btnEditStock, gridBagConstraints);

        btnDeleteStock.setFont(new java.awt.Font("Arial", 1, 12));
        btnDeleteStock.setText("Delete");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        jPanel2.add(btnDeleteStock, gridBagConstraints);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(scrAllStocks, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 622, Short.MAX_VALUE)
            .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 622, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(scrAllStocks, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 314, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 39, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(19, 19, 19))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void btnAddStockActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddStockActionPerformed
        addStock();
    }//GEN-LAST:event_btnAddStockActionPerformed
    
    private void btnEditStockActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEditStockActionPerformed
        editStock();
    }//GEN-LAST:event_btnEditStockActionPerformed
            private void tblAllStocksMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_btnEditStock1actionPerformed
                            if ((evt.getModifiers() & InputEvent.BUTTON1_MASK) != 0 && evt.getClickCount()>1) {//GEN-LAST:event_btnEditStock1actionPerformed
                                                editStock();//GEN-FIRST:event_tblAllStocksMouseClicked
                            }
    }//GEN-LAST:event_tblAllStocksMouseClicked
                // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAddStock;
    private javax.swing.JButton btnDeleteStock;
    private javax.swing.JButton btnEditStock;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane scrAllStocks;
    public static javax.swing.JTable tblAllStocks;
    // End of variables declaration//GEN-END:variables
    
}
