package medstore.verifiers;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Vector;
import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JTextField;
import medstore.SharedConstants;
import medstore.date.DateUtil;

public class UniversalInputVerifier extends InputVerifier implements SharedConstants{
    
    public static final int UNI_STRING_VERIFIER = 1;
    public static final int UNI_INTEGER_PARAMS = 2;
    public static final int UNI_DATE_VERIFIER = 3;
    
    private int nComponentID;
    JDialog parent;
    Vector vecParams;
    public UniversalInputVerifier(int nComponentID){
        this.nComponentID = nComponentID;        
    }
    
    
    public boolean verify(JComponent input) {
        switch (nComponentID){
            case UNI_DATE_VERIFIER: return DATE_Verifier(input);
            default: return false;
        }        
    }
       
    public static boolean DATE_Verifier(JComponent input){
        JTextField tf = (JTextField) input;
        String value = tf.getText();
        int nLength = value.length();
        if(nLength == 0)
            return true;
        for(int i = 0; i < nLength; i++){
            if(!Character.isDigit(value.charAt(i)) && value.charAt(i)!='-' && value.charAt(i)!='.')
                return false;
        }
        String currDateFields[] = DateUtil.parseDateString(value);
        if(currDateFields.length!=3)
            return false;
        int day = Integer.parseInt(currDateFields[0]);
        int month = Integer.parseInt(currDateFields[1]);
        int year =  Integer.parseInt(currDateFields[2]);
        if(currDateFields[0].length() > 2 || currDateFields[1].length()>2 ||
                currDateFields[2].length()==3 || currDateFields[2].length() >4)
            return false;
        if(month < 1 || month > 12)
            return false;
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MONTH, month-1);
        if(day < 1 || day > calendar.getActualMaximum(Calendar.DAY_OF_MONTH))
            return false;
        if((year < 0 || year >99))
            return false;
        String strDate = currDateFields[0] + "." + currDateFields[1] + "." + currDateFields[2];
        ((JTextField)input).setText(strDate);
        return true;
    }
    
    public static boolean SCALED2_PERSENT_Verifier(JComponent input){
        JTextField tf = (JTextField) input;
        String value = tf.getText();
        if(value.length()==0)
            return false;
        try{
            
            BigDecimal nPersent = new BigDecimal(value);
            if(nPersent.doubleValue() < 0 || nPersent.doubleValue() >100)
                return false;
        }catch(NumberFormatException nfe){
            return false;
        }
        return true;
    }
    
    public static boolean BARCODE_Verifier(JComponent input){
        JTextField tf = (JTextField) input;
        String value = tf.getText();
        if(value.length() == 0)
            return false;
        if(value.length() > 12){            
            return false;
        }
        
        return true;
    }
}
