/*
 * PrintUtilities.java
 *
 * Created on , 2005,  16, 19:32
 */

package medstore;

import java.awt.*;
import javax.swing.*;
import java.awt.print.*;
import java.util.*;
import javax.swing.text.*;

public class PrintUtilities implements Printable {    
    private Component componentToBePrinted;
    private double relative;
    Vector m_lines;
    
    
    public PrintUtilities(Component componentToBePrinted, double relative) {
        this.componentToBePrinted = componentToBePrinted;
        this.relative = relative;
    }
    
    public int print(Graphics g, PageFormat pf, int pageIndex) {
        int response = NO_SUCH_PAGE;
        Graphics2D g2 = (Graphics2D) g;
        // for faster printing, turn off double buffering
        disableDoubleBuffering(componentToBePrinted);
        Dimension d = componentToBePrinted.getSize(); //get size of document
        double panelWidth = d.width; //width in pixels
        double panelHeight = d.height; //height in pixels
        double pageHeight = pf.getImageableHeight(); //height of printer page
        double pageWidth = pf.getImageableWidth(); //width of printer page
        double scale = pageWidth / panelWidth;
        int totalNumPages = (int) Math.ceil(scale * panelHeight / pageHeight);
        
        ///////////////////////////////////////////////////////////////////////
        FontMetrics fm = g.getFontMetrics();
        int hLine = fm.getHeight();     
        ///////////////////////////////////////////////////////////////////////
        
        // make sure not print empty pages
        if (pageIndex >= totalNumPages) {
            response = NO_SUCH_PAGE;
        } else {
            // shift Graphic to line up with beginning of print-imageable region
            g2.translate( pf.getImageableX(), pf.getImageableY() );
            // shift Graphic to line up with beginning of next page to print
            g2.translate( 0f, -pageIndex * pageHeight + pageIndex * relative);
            // scale the page so the width fits...
            g2.scale( scale, scale );
            componentToBePrinted.paint(g2); //repaint the page for printing
            enableDoubleBuffering(componentToBePrinted);
            response = Printable.PAGE_EXISTS;
        }
        return response;
    }
    
    public static void disableDoubleBuffering(Component c) {
        RepaintManager currentManager = RepaintManager.currentManager(c);
        currentManager.setDoubleBufferingEnabled(false);
    }
    
    public static void enableDoubleBuffering(Component c) {
        RepaintManager currentManager = RepaintManager.currentManager(c);
        currentManager.setDoubleBufferingEnabled(true);
    }
    
    ////////////////
    
    /*public int print(Graphics g, PageFormat pf, int pageIndex) {
        int response = NO_SUCH_PAGE;
        Graphics2D g2 = (Graphics2D) g;
        // for faster printing, turn off double buffering
        disableDoubleBuffering(componentToBePrinted);
        Dimension d = componentToBePrinted.getSize(); //get size of document
        double panelWidth = d.width; //width in pixels
        double panelHeight = d.height; //height in pixels
        double pageHeight = pf.getImageableHeight(); //height of printer page
        double pageWidth = pf.getImageableWidth(); //width of printer page
        double scale = pageWidth / panelWidth;
        int totalNumPages = (int) Math.ceil(scale * panelHeight / pageHeight);
        
        ///////////////////////////////////////////////////////////////////////
        FontMetrics fm = g.getFontMetrics();
        int hLine = fm.getHeight();     
        int maxPageHeight = ( (int)pageHeight / hLine ) * hLine;
        ///////////////////////////////////////////////////////////////////////
        
        // make sure not print empty pages
        if ( pageIndex >= totalNumPages ) {
            response = NO_SUCH_PAGE;
        } else {
            // shift Graphic to line up with beginning of print-imageable region
            g2.translate( pf.getImageableX(), pf.getImageableY() );
            // shift Graphic to line up with beginning of next page to print
            //g2.translate( 0f, -pageIndex * pageHeight + pageIndex * relative);
            g2.translate( 0f, -pageIndex * maxPageHeight + pageIndex * relative);
            // scale the page so the width fits...
            g2.scale( scale, scale );
            g2.setClip( 0, (int)(-pageIndex * maxPageHeight + pageIndex * relative), (int)pageWidth, maxPageHeight );
            componentToBePrinted.paint(g2); //repaint the page for printing
            //g2.translate( 0f, -pageIndex * maxPageHeight + hLine /*+ pageIndex * relative);
            enableDoubleBuffering(componentToBePrinted);
            response = Printable.PAGE_EXISTS;
        }
        return response;
    }*/    
}

