/*
 * PrinterPageableText.java
 *
 * Created on , 2005,  24, 5:30
 */
package medstore;

import java.awt.print.*;
import java.awt.*;
import java.util.*;
import javax.swing.text.*;

public class PrinterPageableText
    implements Pageable, Printable, SharedConstants 
{
    JTextComponent m_componnent;
    ArrayList m_pages;
    PageFormat m_pageFormat;
    double m_textWidth;
    String m_header;
    int m_reportType;
    float m_fontSize;
    
    public PrinterPageableText( JTextComponent componnent, 
            String header, int reportType )
    {
        m_componnent = componnent;     
        m_header = header;
        m_reportType = reportType;
        m_fontSize = PRINT_FONT_SIZE;
    }
    
    public void print() throws PrinterException 
    {
        PrinterJob printerJob = PrinterJob.getPrinterJob();
        m_pageFormat = printerJob.defaultPage();
        Paper paper = new Paper();
        paper.setImageableArea( 28.346456692913385, 28.346456692913385, 
                538.5826771653543, 785.1968503937007 );
        paper.setSize( 595.275590551181, 841.8897637795276 );
        m_pageFormat.setPaper( paper );        
        m_pageFormat = printerJob.pageDialog(m_pageFormat);
        int pageHeight = (int) m_pageFormat.getImageableHeight();
        Font font = m_componnent.getFont().deriveFont( m_fontSize );
        int lineHeight = this.m_componnent.getFontMetrics(font).getHeight();
        PrinterPagesGenerator pagesGen = 
                new PrinterPagesGenerator( m_componnent.getText(), 
                m_header, m_reportType, pageHeight/lineHeight);            
        m_pages = pagesGen.getPages();
        printerJob.setPageable(this);
        if (!printerJob.printDialog())
          return;
        printerJob.print();
    }
    
    ////////////////////////////////////////////////////////////////////////////
    public int print( Graphics pageGraphics, PageFormat pageFormat, int pageIndex) 
      throws PrinterException 
    {
        int pageWidth = (int) pageFormat.getImageableWidth();
        int pageHeight = (int) pageFormat.getImageableHeight();
        pageGraphics.setFont( m_componnent.getFont().deriveFont( m_fontSize ) );
        //calculate maxWidth
        if( pageIndex == 0 )
            calculateTextWidth( pageGraphics, pageGraphics.getFontMetrics() );
        // scale if needed
        if ( pageWidth < m_textWidth ) 
        {
            correctTextWidth( pageGraphics, pageGraphics.getFontMetrics(), pageWidth);            
        }
        FontMetrics fm = pageGraphics.getFontMetrics();
        int lineHeight = fm.getHeight();
        if (pageIndex >= m_pages.size())
            return NO_SUCH_PAGE;
        pageGraphics.translate( (int) pageFormat.getImageableX(),
                (int) pageFormat.getImageableY() );   
        pageGraphics.setClip( 0, 0, pageWidth, pageHeight);
        int x = 0;
        int y = fm.getAscent();
        ArrayList lines = (ArrayList) m_pages.get(pageIndex);
        for (int i = 0; i < lines.size(); i++) 
        {
            assert y < pageHeight;
            String str = (String) lines.get(i);
            pageGraphics.drawString(str, x, y);
            y += lineHeight;
        }        
        return PAGE_EXISTS;
    }
    
    /*
     *public int print( Graphics pageGraphics, PageFormat pageFormat, int pageIndex) 
      throws PrinterException 
    {
        int pageWidth = (int) pageFormat.getImageableWidth();
        int pageHeight = (int) pageFormat.getImageableHeight();
        pageGraphics.setFont( m_componnent.getFont().deriveFont( m_fontSize ) );
        //calculate maxWidth
        if( pageIndex == 0 )
            calculateTextWidth( pageGraphics, pageGraphics.getFontMetrics() );
        // scale if needed
        if ( pageWidth < m_textWidth ) 
        {
            correctTextWidth( pageGraphics, pageGraphics.getFontMetrics(), pageWidth);
            //double scale = pageWidth / (m_textWidth + 20);
            //( (Graphics2D) pageGraphics).scale(scale, scale);
        }
        FontMetrics fm = pageGraphics.getFontMetrics();
        int lineHeight = fm.getHeight();
        //calculate pages again
        if( pageIndex == 0 )
        {
            //PrinterPagesGenerator pagesGen = 
            //    new PrinterPagesGenerator( m_componnent.getText(),
            //    m_header, m_reportType, pageHeight/lineHeight );
            //m_pages = pagesGen.getPages();
        }
        //TODO check if new pages count is equal to old pages count    
        if (pageIndex >= m_pages.size())
            return NO_SUCH_PAGE;
        pageGraphics.translate( (int) pageFormat.getImageableX(),
                (int) pageFormat.getImageableY() );   
        pageGraphics.setClip( 0, 0, pageWidth, pageHeight);
        int x = 0;
        int y = fm.getAscent();
        ArrayList lines = (ArrayList) m_pages.get(pageIndex);
        for (int i = 0; i < lines.size(); i++) 
        {
            assert y < pageHeight;
            String str = (String) lines.get(i);
            pageGraphics.drawString(str, x, y);
            y += lineHeight;
        }        
        return PAGE_EXISTS;
    }
     **/
    
    private void calculateTextWidth(Graphics printGraphics,FontMetrics fm)
    {    
        String maxString = "";
        for( int i = 0; i < m_pages.size(); i++ )
        {
            ArrayList page = (ArrayList)m_pages.get(i);
            for( int j = 0; j < page.size(); j++ )
            {
                String line = (String)page.get(j);
                if( line.length() > maxString.length() )
                    maxString = line;
            }
        }
        m_textWidth = fm.getStringBounds( maxString , printGraphics ).getWidth();
    }
    
    private void correctTextWidth(Graphics printGraphics,FontMetrics fm,int pageWidth)
    {    
        while( pageWidth < m_textWidth )
        {
            Font font = printGraphics.getFont();
            m_fontSize = m_fontSize - 0.1f;
            printGraphics.setFont( font.deriveFont( m_fontSize ) );
            calculateTextWidth( printGraphics, printGraphics.getFontMetrics() );            
        }       
    }

  ////////////////////////////////////////////////////////////////////////////
  public int getNumberOfPages() 
  {
      return m_pages.size();
  }

  public PageFormat getPageFormat(int pageIndex) 
  {
      return m_pageFormat;
  }

  public Printable getPrintable(int pageIndex) throws IndexOutOfBoundsException 
  {
      return this;
  }
}

