/*
 * PrinterPagesGenerator.java
 *
 * Created on , 2005,  25, 0:12
 */
package medstore;

import java.util.*;

/**
 *
 * @author ait
 *
 */

public class PrinterPagesGenerator implements SharedConstants
{
    String m_text; 
    String m_header;
    int m_reportType;
    int m_linesPerPage;   
    int m_maxTextLength = 0;
    
    public PrinterPagesGenerator( String text, String header, int reportType, 
            int linesPerPage )
    {
        m_text = text; 
        m_header = header.replaceAll("\n","");
        m_reportType = reportType;
        m_linesPerPage = linesPerPage;
    }
    
    public ArrayList getPages()
    {
        ArrayList lines = getLines();
        if( lines.size() < m_linesPerPage )
        {
            ArrayList pages = new ArrayList();
            lines.add(0,getPageNumText(1,1));
            pages.add( lines );
            return pages;
        }
        
        switch( m_reportType )        
        {
            case ORDERS_AMMOUNT_QUERY:
                return handleOrderAnnount(lines);
            case ORDERS_HISTORY_ITEMS_QUERY:
            case MOVEMENT_REVISION_QUERY:
            case ORDERS_PENDING_QUERY:
            case ORDERS_UNPAID_QUERY:
            case ORDERS_HISTORY_QUERY:
            case ORDERS_USAGE:
            case ORDERS_USAGE_MONTHS:
            case ORDERS_NUMORDERS_DAILY:
            case ORDERS_NUMORDERS_MONTHLY:
            case ORDERS_ACTIVE_SALES_PEOPLE_DAILY:
            case ORDERS_ORDERS_ACTIVE_SALES_PEOPLE_MONTHLY:
            case ORDERS_ACTIVE_CUSTOMERS_DAILY:
            case ORDERS_ACTIVE_CUSTOMERS_MONTHLY:
            case ORDERS_COMMISSION_AMMOUNT:
            case TIME_TO_PURCHASE_QUERY: // TODO as simple report and test
                return handleSimpleReports( lines );
            case STOCK_ON_HAND_QUERY: 
            case INVENTORY_CHECKOUT_QUERY:
            case INVENTORY_CHECKOUT_DIFF_QUERY:
                return handleStoreReports( lines );                
            case ORDERS_DETAILS_QUERY: 
                return handleOrderDetails( lines );
            case STOCK_ON_HAND_TOTALS_QUERY:
                return handleOnHandTotals( lines );
            case ORDERS_ORDERS_COMMISIONS:
                return handleOrdersCommissions( lines );
            default:
                throw new java.lang.AssertionError();
        }
    }
    
    ////////////////////
    
    private ArrayList getLines()
    {
        ArrayList lines = new ArrayList();
        StringTokenizer tokenizer = new StringTokenizer(m_text, "\n", true);
        String prevToken = "";
        while ( tokenizer.hasMoreTokens() ) 
        {
            String line = tokenizer.nextToken();  
            if( m_maxTextLength < line.length() )
                m_maxTextLength = line.length();
            
            if ( line.equals("\n") && prevToken.equals("\n") )
            {
                lines.add("");
                prevToken = line;
            }
            else if( !line.equals("\n") )
            {
                line = line.replaceAll( "\t", "    " );
                lines.add( line );
                prevToken = line;
            }            
            else 
            {
                prevToken = line;
                continue;                
            }          
        }            
        return lines;
    }
    
    ////////////////////////////////////////////////////////////////
    
    private ArrayList handleSimpleReports( ArrayList lines )
    {
        ArrayList pages = new ArrayList();
        lines.add(""); //space for page num
        ArrayList antetka = new ArrayList();        
        //process first page
        ArrayList page = new ArrayList();
        for( int i = 0; i < m_linesPerPage; i++ )
        {
            String line = (String)lines.remove(0);
            if( line.equals( m_header ) )
            {
                antetka.add( line );           
                antetka.add( lines.get(0) );//get next line
            }            
            page.add(line);
        }
        pages.add(page);        
        //process remained pages
        while( lines.size() > 0 )
        {
            page = new ArrayList();
            page.add(""); //space for page num
            page.addAll( antetka );
            for( int i = 0; i < m_linesPerPage - 3 && lines.size() > 0; i++ )
            {
                String line = (String)lines.remove(0);
                page.add(line);
            }
            pages.add(page);
        }
        //put page numbers
        for( int i = 0; i < pages.size(); i++ )
        {
            page = (ArrayList)pages.get(i);
            page.set( 0, getPageNumText( i + 1, pages.size() ) );
        }
        return pages;
    }
    
    ///////////////////////////////////////////////////////////////////
    
    private String getPageNumText( int currentPage, int pagesCount )
    {
        String text = "page " + currentPage + " of " + pagesCount;
        int leftPadding = m_maxTextLength / 2;
        int rifgtPadding = leftPadding + m_maxTextLength % 2;
        for( int i = 0; i < leftPadding; i++ )
        {
            text = " " + text;
        }
        for( int i = 0; i < rifgtPadding; i++ )
        {
            text += " ";
        }
        return text;
    }
    
    //////////////////////////////////////////////////////////////////////
    
    private ArrayList handleOrderAnnount( ArrayList lines )
    {
        ArrayList pages = new ArrayList();
        lines.add(""); //space for page num
        ArrayList antetka = new ArrayList();        
        //process first page
        ArrayList page = new ArrayList();
        for( int i = 0; i < m_linesPerPage; i++ )
        {
            String line = (String)lines.remove(0);
            if( line.equals( m_header ) )
            {
                antetka.add( line );           
                antetka.add( lines.get(0) );//get next line
            }            
            page.add(line);
        }
        pages.add(page);        
        //process remained pages
        String strokes = null;
        String total = null;
        while( lines.size() > 0 )
        {
            int remaining = lines.size() - m_linesPerPage + 3 ;
            if( remaining > 0 && remaining <= 2  )
            {   
                total = (String)lines.remove( lines.size() - 1 );
                strokes = (String)lines.remove( lines.size() - 1 );
            }
            page = new ArrayList();
            page.add(""); //space for page num
            page.addAll( antetka );
            for( int i = 0; i < m_linesPerPage - 3 && lines.size() > 0; i++ )
            {
                String line = (String)lines.remove(0);
                page.add(line);
            }
            pages.add(page);
        }
        if( total != null )
        {
            page = new ArrayList();
            page.add(""); //space for page num
            page.add(strokes);
            page.add(total);
        }
        //put page numbers
        for( int i = 0; i < pages.size(); i++ )
        {
            page = (ArrayList)pages.get(i);
            page.set( 0, getPageNumText( i + 1, pages.size() ) );
        }
        return pages;
    }
    
    //////////////////////////////////////////////////////////////////
    
    private ArrayList handleStoreReports( ArrayList lines )
    {
        ArrayList pages = new ArrayList();
        lines.add(""); //space for page num
        ArrayList antetka = new ArrayList();        
        //process first page
        ArrayList page = new ArrayList();
        for( int i = 0; i < m_linesPerPage; i++ )
        {
            String line = (String)lines.remove(0);
            if( line.equals( m_header ) )
            {
                antetka.add( line );           
                antetka.add( lines.get(0) );//get next line
            }            
            page.add(line);
        }
        pages.add(page);
        //process remained pages
        while( lines.size() > 0 )
        {
            if( ((String)lines.get(0)).trim().startsWith("----") )
            {
                // eliminate strokes
                lines.remove(0);
            }
            page = new ArrayList();
            page.add(""); //space for page num
            page.addAll( antetka );
            for( int i = 0; i < m_linesPerPage - 3 && lines.size() > 0; i++ )
            {
                String line = (String)lines.remove(0);
                page.add(line);
            }
            pages.add(page);
        }
        //put page numbers
        for( int i = 0; i < pages.size(); i++ )
        {
            page = (ArrayList)pages.get(i);
            page.set( 0, getPageNumText( i + 1, pages.size() ) );
        }
        return pages;
    }
    ////////////////////////////////////////////////////////////////////////
    
    private ArrayList handleOrderDetails( ArrayList lines )
    {
        ArrayList pages = new ArrayList();
        while( lines.size() > 0 )
        {
            if( ((String)lines.get(0)).trim().startsWith("----") )
            {
                // eliminate strokes
                lines.remove(0);
            }
            ArrayList page = new ArrayList();
            page.add(""); //space for page num
            for( int i = 0; i < m_linesPerPage - 1 && lines.size() > 0; i++ )
            {
                String line = (String)lines.remove(0);
                page.add(line);
            }
            pages.add(page);
        }
        //put page numbers
        for( int i = 0; i < pages.size(); i++ )
        {
            ArrayList page = (ArrayList)pages.get(i);
            page.set( 0, getPageNumText( i + 1, pages.size() ) );
        }
        return pages;
    }
    ////////////////////////////////////////////////////////////////////////
    
    private ArrayList handleOnHandTotals( ArrayList lines )
    {
        m_linesPerPage -= 3;
        int remainder = m_linesPerPage % 4;
        if( remainder != 0 && remainder != 3 )
        {
            m_linesPerPage += 3 - remainder;            
        }
        
        ArrayList pages = new ArrayList();
        lines.add(""); //space for page num
        ArrayList antetka = new ArrayList();        
        //process first page
        ArrayList page = new ArrayList();
        for( int i = 0; i < m_linesPerPage; i++ )
        {
            String line = (String)lines.remove(0);
            if( line.equals( m_header ) )
            {
                antetka.add( line );           
                antetka.add( lines.get(0) );//get next line
            }            
            page.add(line);
        }
        pages.add(page);
        //process remained pages
        while( lines.size() > 0 )
        {
            if( ((String)lines.get(0)).trim().startsWith("----") )
            {
                // eliminate strokes
                lines.remove(0);
            }
            page = new ArrayList();
            page.add(""); //space for page num
            page.addAll( antetka );
            for( int i = 0; i < m_linesPerPage - 3 && lines.size() > 0; i++ )
            {
                String line = (String)lines.remove(0);
                page.add(line);
            }
            pages.add(page);
        }
        //put page numbers
        for( int i = 0; i < pages.size(); i++ )
        {
            page = (ArrayList)pages.get(i);
            page.set( 0, getPageNumText( i + 1, pages.size() ) );
        }
        return pages;
    }    
    ////////////////////////////////////////////////////////////////////
    
    private ArrayList handleOrdersCommissions( ArrayList lines )
    {
        m_linesPerPage -= 3;
        int remainder = m_linesPerPage % 4;
        if( remainder != 0 && remainder != 2 )
        {
            m_linesPerPage += 4 - remainder;            
        }
        
        ArrayList pages = new ArrayList();
        lines.add(""); //space for page num
        ArrayList antetka = new ArrayList();        
        //process first page
        ArrayList page = new ArrayList();
        for( int i = 0; i < m_linesPerPage; i++ )
        {
            String line = (String)lines.remove(0);
            if( line.equals( m_header ) )
            {
                antetka.add( line );           
                antetka.add( lines.get(0) );//get next line
            }            
            page.add(line);
        }
        pages.add(page);
        //process remained pages
        while( lines.size() > 0 )
        {
            if( ((String)lines.get(0)).trim().startsWith("----") )
            {
                // eliminate strokes
                lines.remove(0);
            }
            page = new ArrayList();
            page.add(""); //space for page num
            page.addAll( antetka );
            for( int i = 0; i < m_linesPerPage - 3 && lines.size() > 0; i++ )
            {
                String line = (String)lines.remove(0);
                page.add(line);
            }
            pages.add(page);
        }
        //put page numbers
        for( int i = 0; i < pages.size(); i++ )
        {
            page = (ArrayList)pages.get(i);
            page.set( 0, getPageNumText( i + 1, pages.size() ) );
        }
        return pages;
    }
}