/*
 * SimpleReportGenerator.java
 *
 * Created on , 2005,  14, 21:43
 */

package medstore;

import java.util.*;
import java.awt.print.*;

/**
 *
 * @author ait
 */
public class SimpleReportGenerator 
{    
    public static int ALLIGN_LEFT = 0;
    public static int ALLIGN_RIGHT = 1;
    //
    private int m_maxColumns = 85;
    private StringBuffer m_buffer = new StringBuffer();
    private ArrayList m_data;
    private int[] m_widths;
    private int[] m_alignments;
    private int m_rowsPerPage;    
    private String m_description;
    private String[] m_columns;
    private String m_firstColumnSpace = "     ";
    private int m_rowsCount = 0;   
    private boolean m_numbered = true;
    public String m_header = "";
    
    public SimpleReportGenerator( String description,String[] columns, 
            ArrayList data, int[] widths, int[] alignments, int rowsPerPage, boolean numbered )
    {
        m_data = data;
        m_description = description;
        m_columns = columns;
        m_widths = widths;
        m_alignments = alignments;
        m_rowsPerPage = rowsPerPage;        
        if( m_widths[0] < String.valueOf(data.size()).length() + 2  )
            m_widths[0] = String.valueOf(data.size()).length() + 2;
        m_numbered = numbered;
    }
    
    public SimpleReportGenerator( String description,String[] columns, 
            ArrayList data, int[] widths, int[] alignments, int rowsPerPage )
    {   
        m_data = data;
        m_description = description;
        m_columns = columns;
        m_widths = widths;
        m_alignments = alignments;
        m_rowsPerPage = rowsPerPage;        
        if( m_widths[0] < String.valueOf(data.size()).length() + 2  )
            m_widths[0] = String.valueOf(data.size()).length() + 2;        
    }
    
    public String generateReport()
        //throws Exception
    {
        m_buffer.append( m_description + "\n\n\n" );
        String str = m_buffer.toString();
        for( int i = 0; i < str.length(); i++ )
        {
            char c = str.charAt(i);
            if( c == '\n' )
                m_rowsCount++ ;
        }
        
        generateHeader();
        for( int i = 0; i < m_data.size(); i++ )                
        {
            generateRow( i );
        }
        return m_buffer.toString();
    }
    
    private void generateHeader()
    {   
        String[] columns = new String[ m_columns.length ];
        for( int i = 0; i < m_columns.length; i++ )
        {
            columns[i] = m_columns[i];
        }
        
        String header = "";        
        boolean isEmpty;
        do
        {
            isEmpty = true;
            for( int i = 0; i < columns.length; i++ )
            {
                String colText = columns[i];
                if( colText == null )
                    colText = "";
                int dif = m_widths[i] - colText.length() - 1;
                if( dif > 0 )
                {
                    colText = colText + getPadding(dif);                
                    columns[i] = "";
                }
                else if( dif < 0 )
                {
                    colText = colText.substring( 0, m_widths[i] - 1 );
                    columns[i] = columns[i].substring(m_widths[i] - 1 );
                    isEmpty = false;
                }                
                else
                {
                    columns[i] = "";
                }
                colText = " " + colText;
                header += colText;
            }
            header += "\n";
            m_buffer.append( header );
            m_header = header;
            checkNewPage();
            break; // fix
        }while( !isEmpty );       
        generateAntetka();        
    }
    
    private void generateAntetka()
    {
        String strokes = "";
        for( int i = 0; i < m_columns.length; i++ )
        {
            String colText = m_columns[i];
            if( colText == null )
                colText = "";
            String stroke = " ";
            String str = "-";
            if( i == 0 && m_numbered )
                str = " ";
            for( int j = 0; j < m_widths[i] - 1; j++ )
            {
                stroke += str;                
            }                
            if( i ==   m_columns.length - 1 )
                stroke += " ";
            strokes += stroke;
        }
        strokes += "\n";        
        m_buffer.append( strokes );
        checkNewPage();
    }
    
    private void generateRow( int rowIndex )
    {
        Object[] row = ((ArrayList)m_data.get(rowIndex)).toArray();
        boolean isEmpty = true;
        do
        {
            isEmpty = true;
            for( int i = 0; i < row.length; i++ )            
            {
                String colText = (String)row[i];
                if( colText == null )
                    colText = "";
                //colText = " " + colText;
                int dif = m_widths[i] - colText.length() - 1;
                if( dif > 0 )
                {
                    colText = getColText( colText, dif, m_alignments[i] );                
                    row[i] = "";
                }
                else if( dif < 0 )
                {
                    colText = colText.substring( 0, m_widths[i] - 1 );
                    row[i] = ((String)row[i]).substring(m_widths[i] - 1);
                    isEmpty = false;
                }                
                else
                {
                    row[i] = "";
                }    
                colText = " " + colText;
                m_buffer.append( colText );                
            }
            m_buffer.append( "\n" );
            checkNewPage();
            break; // fix ait
        }while( !isEmpty );
    }
    
    private String getColText( String text, int padding, int align )
    {
        String strPadding = getPadding( padding );
        if( align == ALLIGN_LEFT )
            return text + strPadding;
        return strPadding + text;
    }
    
    private String getPadding( int count )
    {
        String result = "";
        for( int i = 0; i < count; i++  )
        {
            result += " ";
        }
        return result;
    }
    
    private void checkNewPage()
    {
        m_rowsCount++;             
    }  
    
    /*public ArrayList getPagesForPrinting( int pageHeight, int lineHeight )
    {
        
    }*/
}



