package medstore.export;

import java.awt.Cursor;
import java.awt.EventQueue;
import java.awt.FileDialog;
import java.io.*;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.*;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;
import medstore.ComboDataModel;
import medstore.SharedConstants;
import medstore.SharedUtils;
import medstore.UniversalInputVerifier;
import medstore.UniversalTableModel;
import medstore.print.DateUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.Region;
import org.apache.poi.poifs.filesystem.*;

public class ActiveUsersForm extends javax.swing.JFrame implements SharedConstants {
    
    private Vector dateFilter;
    private Vector orderCountFilter;
    private Vector postCodeFilter;
    private Vector userTypeFilter;
    private JFileChooser fd;
    private Thread writeThred = null;
    private boolean write = false;
    
    public ActiveUsersForm() throws Exception{
        initComponents();        
        initFilters();
        afterInit();
        setVerifiers();
        setFilters();
        setPostCodeFilter();
        setDateFilter();
        setOrderCountFilter();
        setUserTypeFilter();
        setFilters();
        setSize(510,300);
    }
    private synchronized void buildSQL() throws Exception{
        setPostCodeFilter();
        setDateFilter();
        setOrderCountFilter();
        setUserTypeFilter();
        setFilters();
        if(!UniversalInputVerifier.DATE_Verifier(txtDateFrom)){
            throw new Exception("Date From is not valid date!");
        }
        if(!UniversalInputVerifier.DATE_Verifier(txtDateTo)){
            throw new Exception("Date To is not valid date!");
        }
        
        if(!UniversalInputVerifier.Universal_Integer_Verifier(txtOrdersFrom)){
            throw new Exception("Orders Count From is not valid integer value!");
        }
        if(!UniversalInputVerifier.Universal_Integer_Verifier(txtOrdersTo)){
            throw new Exception("Orders Count To is not valid integer value!");
        }
        
        if(!UniversalInputVerifier.Universal_Integer_Verifier(txtPCodeFrom)){
            throw new Exception("Post Code From is not valid integer value!");
        }
        if(!UniversalInputVerifier.Universal_Integer_Verifier(txtPCodeTo)){
            throw new Exception("Post Code To is not valid integer value!");
        }
        boolean datefilter = Boolean.parseBoolean(String.valueOf(dateFilter.get(0)));
        boolean orderfilter = Boolean.parseBoolean(String.valueOf(orderCountFilter.get(0)));
        boolean pcodefilter = Boolean.parseBoolean(String.valueOf(postCodeFilter.get(0)));
        boolean usertypefilter = Boolean.parseBoolean(String.valueOf(userTypeFilter.get(0)));
        
        int usertype = Integer.parseInt(String.valueOf(userTypeFilter.get(1)));
                
        boolean nullorders = false; // if order count = 0
        boolean nonnullunion = true;
        
        final String base_prsn = " prsn_key, " +
                "          type_abbr,             " +
                "          prsn_appl,            " +
                "          prsn_fnam,            " +
                "          prsn_lnam,            " +
                "          prsn_city,            " +
                "          city_code,            " +
                "          prsn_strt,            " +
                "          cntr_name            "
                ;
        
        String orderby = " br desc, type_abbr,  cntr_name, city_code, prsn_city, prsn_strt, prsn_appl, prsn_fnam, prsn_lnam ";
        
        //SQL HOLDERS
        String base_prsn_pcode = "";
        String vorgang_date = "";
        String order_count = "";
        String base_prsn_usertype = "";
        
        if(!datefilter && !orderfilter && !pcodefilter && !usertypefilter) {
            throw new Exception("Select filter!");
        }
        
        //IF DATE FILTER CHECKED - dates from vorgang
        if(datefilter){
            vorgang_date =
                    "       (" +
                    "           (" +
                    "               vorgang.invc_date>='" + getFormatDate(dateFilter.get(1).toString()) + "' AND " +
                    "               vorgang.invc_date<'" + getFormatDate(dateFilter.get(2).toString()) + " 23:59:59'" +
                    "           ) OR " +
                    "           (" +
                    "               vorgang.pay_date>='" + getFormatDate(dateFilter.get(1).toString()) + "'  AND " +
                    "               vorgang.pay_date<='" + getFormatDate(dateFilter.get(2).toString()) + " 23:59:59' " +
                    "           )" +
                    "         ) ";
        }
        
        
        //ORDER COUNT FILTER CHECKED
        if(orderfilter){
            int from = 0;
            int to = 0;
            try{
                from = Integer.parseInt(orderCountFilter.get(1).toString());
            } catch(Exception e){
                
            }
            try{
                to = Integer.parseInt(orderCountFilter.get(2).toString());
            } catch(Exception e){
                
            }
            nullorders = (from <= 0 && to <= 0);
            order_count = " ( COUNT(dbo.base_prsn.prsn_key) >= " + from + "  and COUNT(dbo.base_prsn.prsn_key) <= " + to + " )";
            nonnullunion = (from <= 0);
        }
        
        //POST CODE FILTER CHECKED
        if(pcodefilter){
            int from = 0;
            int to = 0;
            try{
                from = Integer.parseInt(postCodeFilter.get(1).toString());
            } catch(Exception e){
                
            }
            try{
                to = Integer.parseInt(postCodeFilter.get(2).toString());
            } catch(Exception e){
                
            }
            
            
            base_prsn_pcode =
                    " ( " +
                    "   (" +
                    "       city_code = CASE when ISNUMERIC(city_code)!= 0 then case when city_code>=" + from + " then city_code end else '0' end" +
                    "   ) " +
                    "   and " +
                    "   (" +
                    "       city_code = CASE when ISNUMERIC(city_code)!= 0 then case when city_code <= " + to + " then city_code end else '0' end" +
                    "   ) " +
                    " )";
            
        }
        
        //if USER TYPE FILTER CHECKED
        if(usertypefilter){
            base_prsn_usertype= " type_idnt = " + usertype;            
            base_prsn_usertype =  "(" + base_prsn_usertype + ")";
        }
        
        Connection conn = getConnection();
        Statement stmt = conn.createStatement();
        String vorgang_usertype = " inner join vorgang on " +
                " ( " +
                "   prsn_key = vorgang.prv1_idnt  or " +
                "   prsn_key = vorgang.prv2_idnt  or " +
                "   prsn_key = vorgang.prv3_idnt  or " +
                "   prsn_key = vorgang.patn_idnt " +
                " ) ";
        String sql = " SELECT " + base_prsn + ", COUNT(dbo.base_prsn.prsn_key) br";
        sql+= (orderfilter ? " " : "");
        String from = " FROM base_prsn " + vorgang_usertype;
        String join = " INNER JOIN base_cntr ON base_prsn.cntr_idnt = base_cntr.cntr_key " +
                " INNER JOIN base_prsn_type ON base_prsn.type_idnt = base_prsn_type.type_key ";
        
        String where = "";
        where = base_prsn_usertype.length()<=0 ? "" : base_prsn_usertype;
        where+= datefilter ? (where.trim().length()<=0 ? vorgang_date : " and " + vorgang_date) : "";
        where+= pcodefilter ? (where.trim().length()<=0 ? base_prsn_pcode : " and " + base_prsn_pcode) : "";
        
        String groupby = " group by " + base_prsn;
        groupby+= nullorders || order_count.trim().length() <= 0 ? "" : " HAVING " + order_count;
        
        String nullorders_sql = "select " + base_prsn + ",'0' br";
        nullorders_sql+= " FROM base_prsn " + join;
        nullorders_sql+= " where " +
                "   prsn_key not in " +
                "       (" +
                "   SELECT  prsn_key " + from + (where.trim().length()<=0 ? "" : " where " + where) +
                "       ) ";
        nullorders_sql+= usertypefilter ? " and " + base_prsn_usertype : "";
        nullorders_sql+= pcodefilter ? " and " + base_prsn_pcode : "";
        
        sql = nonnullunion ? nullorders_sql + " union " + sql : sql;
        
        
        sql = (nullorders ? nullorders_sql : sql + from + join + (where.trim().length()<=0 ? "" : " where " + where));
        sql+=groupby;
        sql+=" order by " + orderby;
        System.out.println(sql);
        ResultSet rs = null;
        try{
            rs = stmt.executeQuery(sql);
        } catch(Exception e){
            throw new Exception("Error:");
        }
        String couumns = base_prsn + ",Orders";
        String[] columnNames = couumns.split(",");
        UniversalTableModel tm = new UniversalTableModel(rs, columnNames);
        
        
        
        String filePath = editFileLocation.getText();
        HSSFWorkbook wb = new HSSFWorkbook();
        
        HSSFFont f = wb.createFont();
        f.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        
        HSSFCellStyle cs = wb.createCellStyle();
        cs.setFont(f);
        cs.setWrapText(true);
        
        HSSFCellStyle cs1 = wb.createCellStyle();
        cs1.setFont(f);
        cs1.setBorderBottom(HSSFCellStyle.BORDER_THIN);
        cs1.setBorderLeft(HSSFCellStyle.BORDER_THIN);
        cs1.setBorderRight(HSSFCellStyle.BORDER_THIN);
        cs1.setBorderTop(HSSFCellStyle.BORDER_THIN);
        cs1.setWrapText(true);
        
        HSSFSheet sheet = wb.createSheet("Sheet 1");
        sheet.setColumnWidth( (short) 3, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        sheet.setColumnWidth( (short) 4, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        sheet.setColumnWidth( (short) 5, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        sheet.setColumnWidth( (short) 6, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        sheet.setColumnWidth( (short) 7, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        sheet.setColumnWidth( (short) 8, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        sheet.setColumnWidth( (short) 9, (short) ( ( 50 * 8 ) / ( (double) 1 / 10 ) ) );
        
        int rows = tm.getRowCount();
        if(rows>10000)
            throw new Exception("Too many rows (" + tm.getRowCount() + "), max (10000)!");
        
        int header_rows = 0;
        int header_cols = 0;
        //Header
        HSSFRow row = sheet.createRow(header_rows++);
        
        HSSFCell cell = row.createCell((short)header_cols);
        cell.setCellStyle(cs);
        cell.setCellValue("Active Persons Report - " + DateUtils.ddmmyyy(new Date()));
        sheet.addMergedRegion(new Region(0,(short)0,0,(short)9));
        
        row = sheet.createRow(header_rows++);
        header_cols = 0;
        
        if(datefilter){
            row = sheet.createRow(header_rows++);
            
            cell = row.createCell((short)0);
            cell.setCellValue("Date: ");
            cell.setCellStyle(cs);
            
            cell = row.createCell((short)2);
            cell.setCellStyle(cs);
            cell.setCellValue("From " + dateFilter.get(1).toString() + " To " + dateFilter.get(2).toString());
            
            sheet.addMergedRegion(new Region(header_rows - 1,(short)0,header_rows - 1,(short)1));
            sheet.addMergedRegion(new Region(header_rows - 1,(short)2,header_rows - 1,(short)9));
            
        }
        if(orderfilter){
            row = sheet.createRow(header_rows++);
            
            cell = row.createCell((short)0);
            cell.setCellStyle(cs);
            cell.setCellValue("Number of orders: ");
            
            cell = row.createCell((short)2);
            cell.setCellStyle(cs);
            cell.setCellValue("From " + orderCountFilter.get(1).toString() + " To " + orderCountFilter.get(2).toString());
            
            sheet.addMergedRegion(new Region(header_rows - 1,(short)0,header_rows - 1,(short)1));
            sheet.addMergedRegion(new Region(header_rows - 1,(short)2,header_rows - 1,(short)9));
        }
        
        if(pcodefilter){
            row = sheet.createRow(header_rows++);
            
            cell = row.createCell((short)0);
            cell.setCellStyle(cs);
            cell.setCellValue("Post Code: ");
            
            cell = row.createCell((short)2);
            cell.setCellStyle(cs);
            cell.setCellValue("From " + postCodeFilter.get(1).toString() + " To " + postCodeFilter.get(2).toString());
            
            sheet.addMergedRegion(new Region(header_rows - 1,(short)0,header_rows - 1,(short)1));
            sheet.addMergedRegion(new Region(header_rows - 1,(short)2,header_rows - 1,(short)9));
        }
        
        if(usertypefilter){
            row = sheet.createRow(header_rows++);
            header_cols = 0;
            
            cell = row.createCell((short)0);
            cell.setCellStyle(cs);
            cell.setCellValue("Person type: ");
            
            header_cols = 1;
            String celval = ""+cboUserType.getSelectedItem();
            
            cell = row.createCell((short)2);
            cell.setCellStyle(cs);
            cell.setCellValue(celval);
            
            sheet.addMergedRegion(new Region(header_rows - 1,(short)0,header_rows - 1,(short)1));
            sheet.addMergedRegion(new Region(header_rows - 1,(short)2,header_rows - 1,(short)9));
        }
        header_rows++;
        row = sheet.createRow(header_rows++);
        header_cols = 0;
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("UIN ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Type ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Appl. ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("First Name ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Last Name ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("City ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Post Code ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Street ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Country ");
        
        cell = row.createCell((short)header_cols++);
        cell.setCellStyle(cs1);
        cell.setCellValue("Orders count");
        
        int counter = 0;
        for(int i = 0; i < rows; i++){
            
            row = sheet.createRow(header_rows+i);
            for(int j= 0; j < tm.getColumnCount(); j++){
                cell = row.createCell((short)j);
                cell.setCellValue(""+tm.getValueAt(i, j));
            }
        }
        
        
        FileOutputStream out = new FileOutputStream(filePath);
        wb.write(out);
        out.close();
        
        
    }
    
    private void setFilters(){
        chkDate.setSelected(Boolean.parseBoolean(String.valueOf(dateFilter.get(0))));
        txtDateFrom.setText(String.valueOf(dateFilter.get(1)));
        txtDateTo.setText(String.valueOf(dateFilter.get(2)));
        
        chkOrdCount.setSelected(Boolean.parseBoolean(String.valueOf(orderCountFilter.get(0))));
        txtOrdersFrom.setText(String.valueOf(orderCountFilter.get(1)));
        txtOrdersTo.setText(String.valueOf(orderCountFilter.get(2)));
        
        chkPostCode.setSelected(Boolean.parseBoolean(String.valueOf(postCodeFilter.get(0))));
        txtPCodeFrom.setText(String.valueOf(postCodeFilter.get(1)));
        txtPCodeTo.setText(String.valueOf(postCodeFilter.get(2)));
        
    }
    
    private void initFilters(){
        
        dateFilter = new Vector(3);
        orderCountFilter = new Vector(3);
        postCodeFilter = new Vector(3);
        userTypeFilter = new Vector(5);
        
        dateFilter.add(0, new Boolean(true));
        dateFilter.add(1, DateUtils.ddmmyyy(new Date()));
        dateFilter.add(2, DateUtils.ddmmyyy(new Date()));
        
        orderCountFilter.add(0, new Boolean(false));
        orderCountFilter.add(1, new Integer(0));
        orderCountFilter.add(2, new Integer(0));
        
        postCodeFilter.add(0, new Boolean(false));
        postCodeFilter.add(1, new Integer(0));
        postCodeFilter.add(2, new Integer(0));
        
        userTypeFilter.add(0, new Boolean(true));
        
        /*userTypeFilter.add(1, new Boolean(true)); //DR
        userTypeFilter.add(2, new Boolean(false)); //MA
        userTypeFilter.add(3, new Boolean(false)); //PAT
        userTypeFilter.add(4, new Boolean(false)); //APO*/
    }
    private void setVerifiers(){
        txtDateFrom.setInputVerifier(new UniversalInputVerifier(FilterQueryDialog_DATE,new Vector()));
        txtDateTo.setInputVerifier(new UniversalInputVerifier(FilterQueryDialog_DATE,new Vector()));
        
        txtOrdersFrom.setInputVerifier(new UniversalInputVerifier(SharedConstants.Universal_Integer,new Vector()));
        txtOrdersTo.setInputVerifier(new UniversalInputVerifier(SharedConstants.Universal_Integer,new Vector()));
    }
    private void setDateFilter(){
        txtDateFrom.setEnabled(chkDate.isSelected());
        txtDateTo.setEnabled(chkDate.isSelected());
        
        dateFilter.set(0, new Boolean(chkDate.isSelected()));
        dateFilter.set(1, txtDateFrom.getText());
        dateFilter.set(2, txtDateTo.getText());
    }
    private void setOrderCountFilter(){
        txtOrdersFrom.setEnabled(chkOrdCount.isSelected());
        txtOrdersTo.setEnabled(chkOrdCount.isSelected());
        
        orderCountFilter.set(0, new Boolean(chkOrdCount.isSelected()));
        orderCountFilter.set(1, txtOrdersFrom.getText());
        orderCountFilter.set(2, txtOrdersTo.getText());
    }
    private void setPostCodeFilter(){
        txtPCodeFrom.setEnabled(chkPostCode.isSelected());
        txtPCodeTo.setEnabled(chkPostCode.isSelected());
        
        
        postCodeFilter.set(0, new Boolean(chkPostCode.isSelected()));
        postCodeFilter.set(1, txtPCodeFrom.getText());
        postCodeFilter.set(2, txtPCodeTo.getText());
    }
    private void setUserTypeFilter(){       
        int nTypeID = ((ComboDataModel)cboUserType.getModel()).getCObjectAt(cboUserType.getSelectedIndex()).nItemDBKey;
        userTypeFilter.add(1, new Integer(nTypeID));
    }
    private String getFormatDate(String date){
        String dateFields[] = date.split("\\.");
        return dateFields[1] + "." + dateFields[0] + "." + dateFields[2];
    }
    
    public Connection getConnection() {
        java.sql.Connection  con = null;
        String url = "jdbc:microsoft:sqlserver://";
        String fullUrl = url + SharedUtils.databaseUrl + ":" + SharedUtils.databaseUrlPort +
                ";databaseName=" + SharedUtils.databaseName + ";";
        try {
            Class.forName("com.microsoft.jdbc.sqlserver.SQLServerDriver");
            Properties props = new Properties();
            props.setProperty("user", SharedUtils.dbUserName );
            props.setProperty("password", SharedUtils.dbPassword );
            props.setProperty("charSet", "ISO-8859-1" );
            props.setProperty("useUnicode", "true");
            props.setProperty("SendStringParametersAsUnicode", "true");
            con = java.sql.DriverManager.getConnection(fullUrl,props);
        } catch(Exception e) {
            e.printStackTrace();
        }
        return con;
    }
    
    public void checkExist() throws Exception{
        
        if( editFileLocation.getText().trim().equals("")) {
            throw new Exception("Please select File Name!");
        }
        File f = new File(editFileLocation.getText());
        while(f.exists()){
            int choose = JOptionPane.showConfirmDialog(this, "File already "
                    + "exist overwrite data in the file ?", "File exist"
                    , JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE);
            if(choose == JOptionPane.YES_OPTION){
                f = fd.getSelectedFile();
                break;
            } else{
                int res = fd.showSaveDialog(this);
                if(res == JFileChooser.APPROVE_OPTION) {
                    f = new File(fd.getSelectedFile().getAbsolutePath());
                } else{
                    return;
                }
            }
        }
    }
    private void afterInit() throws Exception{
        Connection connDB = getConnection();
        String strSQL = "SELECT TYPE_ABBR, TYPE_KEY  FROM BASE_PRSN_TYPE ORDER BY TYPE_ABBR";
        Statement stm = connDB.createStatement();
        ResultSet rsDBData=stm.executeQuery(strSQL);
        ComboDataModel cdbClients = new ComboDataModel(rsDBData);
        cboUserType.setModel(cdbClients);
        cboUserType.setSelectedIndex(1);
        cboUserType.updateUI();
        rsDBData.close();
        stm.close();
    }
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        panDate = new javax.swing.JPanel();
        txtDateFrom = new javax.swing.JTextField();
        lblDateFrom = new javax.swing.JLabel();
        lblDateTo = new javax.swing.JLabel();
        txtDateTo = new javax.swing.JTextField();
        chkDate = new javax.swing.JCheckBox();
        panOrderCount = new javax.swing.JPanel();
        txtOrdersFrom = new javax.swing.JTextField();
        lblOrdersFrom = new javax.swing.JLabel();
        lblOrdersTo = new javax.swing.JLabel();
        txtOrdersTo = new javax.swing.JTextField();
        chkOrdCount = new javax.swing.JCheckBox();
        panPersonType = new javax.swing.JPanel();
        cboUserType = new javax.swing.JComboBox();
        lblUserType = new javax.swing.JLabel();
        panPostCode = new javax.swing.JPanel();
        txtPCodeFrom = new javax.swing.JTextField();
        lblPCodeFrom = new javax.swing.JLabel();
        lblPCodeTo = new javax.swing.JLabel();
        txtPCodeTo = new javax.swing.JTextField();
        chkPostCode = new javax.swing.JCheckBox();
        panPersonType1 = new javax.swing.JPanel();
        editFileLocation = new javax.swing.JTextField();
        btnBrowse = new javax.swing.JButton();
        btnSave = new javax.swing.JButton();

        getContentPane().setLayout(null);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setLocationByPlatform(true);
        setMaximizedBounds(new java.awt.Rectangle(0, 0, 500, 700));
        panDate.setLayout(null);

        panDate.setBorder(new javax.swing.border.TitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
        txtDateFrom.setFont(new java.awt.Font("Arial", 1, 12));
        panDate.add(txtDateFrom);
        txtDateFrom.setBounds(150, 5, 80, 21);

        lblDateFrom.setFont(new java.awt.Font("Arial", 1, 12));
        lblDateFrom.setText("From");
        panDate.add(lblDateFrom);
        lblDateFrom.setBounds(110, 5, 40, 21);

        lblDateTo.setFont(new java.awt.Font("Arial", 1, 12));
        lblDateTo.setText("To");
        panDate.add(lblDateTo);
        lblDateTo.setBounds(240, 5, 30, 21);

        txtDateTo.setFont(new java.awt.Font("Arial", 1, 12));
        panDate.add(txtDateTo);
        txtDateTo.setBounds(270, 5, 80, 21);

        chkDate.setFont(new java.awt.Font("Arial", 1, 12));
        chkDate.setText("Date");
        chkDate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chkDateActionPerformed(evt);
            }
        });

        panDate.add(chkDate);
        chkDate.setBounds(2, 5, 100, 21);

        getContentPane().add(panDate);
        panDate.setBounds(10, 10, 480, 30);

        panOrderCount.setLayout(null);

        panOrderCount.setBorder(new javax.swing.border.TitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
        txtOrdersFrom.setFont(new java.awt.Font("Arial", 1, 12));
        panOrderCount.add(txtOrdersFrom);
        txtOrdersFrom.setBounds(150, 5, 80, 21);

        lblOrdersFrom.setFont(new java.awt.Font("Arial", 1, 12));
        lblOrdersFrom.setText("From");
        panOrderCount.add(lblOrdersFrom);
        lblOrdersFrom.setBounds(110, 5, 40, 21);

        lblOrdersTo.setFont(new java.awt.Font("Arial", 1, 12));
        lblOrdersTo.setText("To");
        panOrderCount.add(lblOrdersTo);
        lblOrdersTo.setBounds(240, 5, 30, 21);

        txtOrdersTo.setFont(new java.awt.Font("Arial", 1, 12));
        panOrderCount.add(txtOrdersTo);
        txtOrdersTo.setBounds(270, 5, 80, 21);

        chkOrdCount.setFont(new java.awt.Font("Arial", 1, 12));
        chkOrdCount.setText("Order Count");
        chkOrdCount.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chkOrdCountActionPerformed(evt);
            }
        });

        panOrderCount.add(chkOrdCount);
        chkOrdCount.setBounds(2, 5, 100, 21);

        getContentPane().add(panOrderCount);
        panOrderCount.setBounds(10, 40, 480, 30);

        panPersonType.setLayout(null);

        panPersonType.setBorder(new javax.swing.border.TitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
        cboUserType.setFont(new java.awt.Font("Arial", 1, 12));
        cboUserType.setMinimumSize(new java.awt.Dimension(29, 21));
        cboUserType.setPreferredSize(new java.awt.Dimension(29, 21));
        panPersonType.add(cboUserType);
        cboUserType.setBounds(100, 5, 120, 21);

        lblUserType.setFont(new java.awt.Font("Arial", 1, 12));
        lblUserType.setText("User Type");
        panPersonType.add(lblUserType);
        lblUserType.setBounds(10, 5, 80, 21);

        getContentPane().add(panPersonType);
        panPersonType.setBounds(10, 100, 480, 30);

        panPostCode.setLayout(null);

        panPostCode.setBorder(new javax.swing.border.TitledBorder(null, "", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
        txtPCodeFrom.setFont(new java.awt.Font("Arial", 1, 12));
        panPostCode.add(txtPCodeFrom);
        txtPCodeFrom.setBounds(150, 5, 80, 21);

        lblPCodeFrom.setFont(new java.awt.Font("Arial", 1, 12));
        lblPCodeFrom.setText("From");
        panPostCode.add(lblPCodeFrom);
        lblPCodeFrom.setBounds(110, 5, 40, 21);

        lblPCodeTo.setFont(new java.awt.Font("Arial", 1, 12));
        lblPCodeTo.setText("To");
        panPostCode.add(lblPCodeTo);
        lblPCodeTo.setBounds(240, 5, 30, 21);

        txtPCodeTo.setFont(new java.awt.Font("Arial", 1, 12));
        panPostCode.add(txtPCodeTo);
        txtPCodeTo.setBounds(270, 5, 80, 21);

        chkPostCode.setFont(new java.awt.Font("Arial", 1, 12));
        chkPostCode.setText("Post Code");
        chkPostCode.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chkPostCodeActionPerformed(evt);
            }
        });

        panPostCode.add(chkPostCode);
        chkPostCode.setBounds(2, 5, 100, 21);

        getContentPane().add(panPostCode);
        panPostCode.setBounds(10, 70, 480, 30);

        panPersonType1.setLayout(null);

        panPersonType1.setBorder(new javax.swing.border.TitledBorder(null, "Save to", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Tahoma", 1, 12)));
        editFileLocation.setFont(new java.awt.Font("Arial", 1, 12));
        panPersonType1.add(editFileLocation);
        editFileLocation.setBounds(10, 40, 280, 21);

        btnBrowse.setFont(new java.awt.Font("Arial", 1, 12));
        btnBrowse.setText("Browse");
        btnBrowse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnBrowseActionPerformed(evt);
            }
        });

        panPersonType1.add(btnBrowse);
        btnBrowse.setBounds(300, 40, 90, 21);

        btnSave.setFont(new java.awt.Font("Arial", 1, 12));
        btnSave.setText("Save");
        btnSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSaveActionPerformed(evt);
            }
        });

        panPersonType1.add(btnSave);
        btnSave.setBounds(400, 40, 70, 21);

        getContentPane().add(panPersonType1);
        panPersonType1.setBounds(10, 140, 480, 70);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents
            
    private void btnSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveActionPerformed
        
        try{
            
            final ActiveUsersForm parent = this;
            EventQueue.invokeLater(new Runnable(){
                public void run(){
                    try{
                        checkExist();
                        btnSave.setEnabled(false);
                        editFileLocation.setEnabled(false);
                        btnBrowse.setEnabled(false);
                        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR)); 
                        buildSQL();
                        JOptionPane.showMessageDialog(null,"File saved!");
                    } catch(Exception e){                        
                        JOptionPane.showMessageDialog(parent,"Error: " +e.getMessage());
                        e.printStackTrace();
                    } finally{
                         setCursor(Cursor.getDefaultCursor()); 
                        btnSave.setEnabled(true);
                        editFileLocation.setEnabled(true);
                        btnBrowse.setEnabled(true);
                    }
                }
            });
            
            
        } catch(Exception e){
            JOptionPane.showMessageDialog(this,"Error: " +e.getMessage());
        } finally{
            
        }
    }//GEN-LAST:event_btnSaveActionPerformed
    
    private void btnBrowseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnBrowseActionPerformed
        
        fd = new JFileChooser();
        fd.setSelectedFile(new File(DateUtils.ddmmyyy(new Date())+".xls"));
        fd.setFileFilter(new FileFilter(){
            public boolean accept(java.io.File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".xls");
            }
            
            public String getDescription() {
                return "Excel (*.xls)";
            }
        });
        int res = fd.showSaveDialog(this);
        if(res == JFileChooser.APPROVE_OPTION) {
            editFileLocation.setText(fd.getSelectedFile().getAbsolutePath());
        } else{
            return;
        }
    }//GEN-LAST:event_btnBrowseActionPerformed
    
    private void chkPostCodeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chkPostCodeActionPerformed
        setPostCodeFilter();
    }//GEN-LAST:event_chkPostCodeActionPerformed
    
    private void chkDateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chkDateActionPerformed
        setDateFilter();
    }//GEN-LAST:event_chkDateActionPerformed
    
    private void chkOrdCountActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chkOrdCountActionPerformed
        setOrderCountFilter();
    }//GEN-LAST:event_chkOrdCountActionPerformed
    public static void main(String args[]){
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                try{
                    new ActiveUsersForm().setVisible(true);
                } catch(Exception e){
                    e.printStackTrace();
                }
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnBrowse;
    private javax.swing.JButton btnSave;
    private javax.swing.JComboBox cboUserType;
    private javax.swing.JCheckBox chkDate;
    private javax.swing.JCheckBox chkOrdCount;
    private javax.swing.JCheckBox chkPostCode;
    private javax.swing.JTextField editFileLocation;
    private javax.swing.JLabel lblDateFrom;
    private javax.swing.JLabel lblDateTo;
    private javax.swing.JLabel lblOrdersFrom;
    private javax.swing.JLabel lblOrdersTo;
    private javax.swing.JLabel lblPCodeFrom;
    private javax.swing.JLabel lblPCodeTo;
    private javax.swing.JLabel lblUserType;
    private javax.swing.JPanel panDate;
    private javax.swing.JPanel panOrderCount;
    private javax.swing.JPanel panPersonType;
    private javax.swing.JPanel panPersonType1;
    private javax.swing.JPanel panPostCode;
    private javax.swing.JTextField txtDateFrom;
    private javax.swing.JTextField txtDateTo;
    private javax.swing.JTextField txtOrdersFrom;
    private javax.swing.JTextField txtOrdersTo;
    private javax.swing.JTextField txtPCodeFrom;
    private javax.swing.JTextField txtPCodeTo;
    // End of variables declaration//GEN-END:variables
    
}
