package medstore.print.letters;
import java.awt.PrintJob;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import java.net.URL;
import java.util.*;
import java.sql.*;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;
import medstore.PrintUtilities;

import medstore.SharedConstants;
import medstore.SharedUtils;
import medstore.print.CustomPrintUtil;
import medstore.print.DateUtils;
import medstore.print.gui.PrintPreview;
import org.jfree.report.JFreeReport;
import org.jfree.report.modules.gui.print.PrintUtil;
import org.jfree.report.modules.misc.tablemodel.ResultSetTableModelFactory;
import org.jfree.report.modules.parser.base.ReportGenerator;


public class GenerateLetter implements SharedConstants{
    
    DefaultTableModel tm = new DefaultTableModel();
    Object[] columnNames = {
        "strAppeal",
                "strDate",
                "strClientIdnt",
                "strTableOrderNo",
                "strOrderDate",
                "strDocType",
                "strNetto",
                "strTotal",
                "strPayed",
                "strDayLag",
                "strPenalty",
                "strTotalPenalty",
                "beforeAntetkaText",
                "underAntetkaText"
    };
    public GenerateLetter() {
    }
    
    public String showReport( final int letterNum ) throws Exception{
        tm = new DefaultTableModel(columnNames, 0);
        
        String letterDateIdent = "letter" + letterNum + "date";
        final String sql = "select  TOP 100 PERCENT vorgang.invc_date, " +
                " base_prsn.prsn_appl, base_prsn.prsn_lnam, base_prsn.prsn_fnam, base_prsn.prsn_strt, " +
                " base_prsn.city_code, base_prsn.prsn_city, base_prsn.prsn_key, base_prsn.altr_name, " +
                " vorgang.v_nr, base_doc_type.type_name, vorgang.neto_sum, " +
                " vorgang.totl_sum, vorgang.pay_sum, cntry_short, DATEDIFF(day, vorgang.invc_date, getdate()) diff " +
                " from base_prsn inner join base_cntr on base_prsn.cntr_idnt = base_cntr.cntr_key,vorgang, base_doc_type " +
                " where pay_sum < totl_sum AND vorgang." + letterDateIdent + " is null " +
                " and base_doc_type.type_key=vorgang.doc_type_idnt " +
                " and vorgang.patn_idnt=base_prsn.prsn_key " +
                " and vorgang.doc_type_idnt=1 " +
                " and vorgang.v_nr not in(select distinct invoiceid from vorgang where doc_type_idnt=2) " +
                " AND DATEDIFF([day], dbo.vorgang.invc_date, GETDATE()) >= (SELECT     TOP 1 letter" + letterNum + "days FROM settings)" +
                
                " order by base_prsn.prsn_key, vorgang.v_nr ";
        System.out.println(sql);
        Connection conn = null;
        Calendar currentDate = Calendar.getInstance();
        currentDate.add( Calendar.DAY_OF_MONTH ,1 );
        SharedUtils.set2BeginOfTheDay(currentDate);
        Calendar endDate = Calendar.getInstance();
        try {
            conn = SharedUtils.getConnection();
            Statement stm = conn.createStatement();
            ResultSet rs = stm.executeQuery(sql);
            boolean isFirst = true;
            while( rs.next() ) {
                // Timestamp orderTimeStamp = rs.getTimestamp(1);
                // endDate.setTimeInMillis( orderTimeStamp.getTime() );
                //endDate.add( Calendar.DAY_OF_MONTH, SharedUtils.letterSendDays[letterNum-1] );
                // if( endDate.getTimeInMillis() >= currentDate.getTimeInMillis() )
                //     continue;
                //load person info
                ArrayList personInfo = new ArrayList();
                String strAppeal = rs.getString(2);
                String strLastName = rs.getString(3);
                String strFirstName = rs.getString(4);
                String strStreet = rs.getString(5);
                String strCode = rs.getString(6);
                String strCity = rs.getString(7);
                String strDate = SharedUtils.getCurrentOnlyDateStringFormatted();
                String strClientIdnt = rs.getString(8);
                String altName = rs.getString(9);
                altName = rs.wasNull() || altName==null ? "" : altName;
                
                String strTableOrderNo = rs.getString(10);
                String strDocType = rs.getString(11);
                String strOrderDate = DateUtils.ddmmyyy(rs.getTimestamp(1));
                String strNetto = rs.getString(12);
                String strTotal = rs.getString(13);
                String strPayed = rs.getString(14);
                
                
                
                String penalty = getPenalty(letterNum);
                String totalPenalty = "" + ( Double.parseDouble( penalty ) + Double.parseDouble(strTotal) );
                
                String cntry_short = rs.getString("cntry_short");
                cntry_short = rs.wasNull() ? "" : cntry_short;
                
                String diff = rs.getString("diff");
                tm.addRow(new Object[]{
                    strAppeal+"\n" +
                            strFirstName + " " + strLastName +
                            (altName.trim().length() <=0 ? "" : "\n" + altName) +
                            "\n" + strStreet +
                            "\n" + cntry_short + " " + strCode + " " + strCity,
                            strDate,
                            strClientIdnt,
                            strTableOrderNo,
                            strOrderDate,
                            strDocType,
                            SharedUtils.formatFloat(strNetto, '.' ) + " EUR",
                            SharedUtils.formatFloat(strTotal, '.' ) + " EUR",
                            SharedUtils.formatFloat(strPayed, '.' )+ " EUR",
                            diff,
                            SharedUtils.formatFloat(penalty, '.' )+ " EUR",
                            SharedUtils.formatFloat(totalPenalty, '.' )+ " EUR",
                            getBeforTableText(letterNum),
                            getAfterTableText(strDate, letterNum)
                            
                });
            }
            rs.close();
            stm.close();
        } catch( SQLException e ) {
            e.printStackTrace();
        } finally {
            SharedUtils.closeConnection(conn);
        }
        
        final ReportGenerator generator = ReportGenerator.getInstance();
        final ResultSetTableModelFactory rstmf =  ResultSetTableModelFactory.getInstance();
        URL urlReportXml= getClass().getResource("/medstore/print/letters/letter"+letterNum+".xml");
        
        final JFreeReport report = generator.parseReport(urlReportXml);
        report.setData(tm);
        report.setName("Generate Letter " + letterNum);
        JButton print = new JButton("Print");
        
        final PrintPreview preview = new PrintPreview(report, print);
        
        print.addActionListener(new ActionListener(){
            public void actionPerformed(final ActionEvent e) {
                try{
                    if(PrintUtil.print(report)){
                        update(sql, letterNum);
                    }
                    
                } catch(Exception ex){
                    JOptionPane.showMessageDialog(null,"Error: " + ex.getMessage());
                    ex.printStackTrace();
                }
            }
        });
        preview.setSize(750, 750);
        preview.setVisible(true);
        
        return "";
    }
    public String showReport( final String v_nr, final int letterNum ) throws Exception{
        String letterDateIdent = "letter" + letterNum + "date";
        final String sql = " select  " +
                "   vorgang.invc_date,  " +
                "   base_prsn.prsn_appl, " +
                "   base_prsn.prsn_lnam, " +
                "   base_prsn.prsn_fnam, " +
                "   base_prsn.prsn_strt,  " +
                "   base_prsn.city_code, " +
                "   base_prsn.prsn_city, " +
                "   base_prsn.prsn_key, " +
                "   base_prsn.altr_name,  " +
                "   vorgang.v_nr, " +
                "   base_doc_type.type_name, " +
                "   vorgang.neto_sum,  " +
                "   vorgang.totl_sum, " +
                "   vorgang.pay_sum, " +
                "   cntry_short, " +
                "   DATEDIFF(day, vorgang.invc_date, getdate()) diff   " +
                
                " from base_prsn " +
                "   inner join base_cntr on base_prsn.cntr_idnt = base_cntr.cntr_key " +
                "   inner join vorgang on vorgang.patn_idnt=base_prsn.prsn_key" +
                "   inner join base_doc_type on base_doc_type.type_key = vorgang.doc_type_idnt " +
                " where " +
                "   vorgang.v_nr = " + v_nr + " " +
                "   and vorgang.doc_type_idnt = 1 " +
                "   and vorgang.v_nr not in" +
                "       (" +
                "           select distinct invoiceid from vorgang where doc_type_idnt=2" +
                "       ) " +
                " order by " +
                "   base_prsn.prsn_key, vorgang.v_nr  ";
        System.out.println(sql);
        Connection conn = null;
        tm = new DefaultTableModel(columnNames, 0);
        try {
            conn = SharedUtils.getConnection();
            Statement stm = conn.createStatement();
            ResultSet rs = stm.executeQuery(sql);
            boolean isFirst = true;
            while( rs.next() ) {
                
                String strAppeal = rs.getString(2);
                String strLastName = rs.getString(3);
                String strFirstName = rs.getString(4);
                String strStreet = rs.getString(5);
                String strCode = rs.getString(6);
                String strCity = rs.getString(7);
                String strDate = SharedUtils.getCurrentOnlyDateStringFormatted();
                String strClientIdnt = rs.getString(8);
                String altName = rs.getString(9);
                altName = rs.wasNull() || altName==null ? "" : altName;
                
                String strTableOrderNo = rs.getString(10);
                String strDocType = rs.getString(11);
                String strOrderDate = DateUtils.ddmmyyy(rs.getTimestamp(1));
                String strNetto = rs.getString(12);
                String strTotal = rs.getString(13);
                String strPayed = rs.getString(14);
                
                String penalty = getPenalty(letterNum);
                String totalPenalty = "" + ( Double.parseDouble( penalty ) + Double.parseDouble(strTotal) );
                
                String cntry_short = rs.getString("cntry_short");
                cntry_short = rs.wasNull() ? "" : cntry_short;
                
                String diff = rs.getString("diff");
                tm.addRow(new Object[]{
                    strAppeal+"\n" +
                            strFirstName + " " + strLastName +
                            (altName.trim().length() <=0 ? "" : "\n" + altName) +
                            "\n" + strStreet +
                            "\n" + cntry_short + " " + strCode + " " + strCity,
                            strDate,
                            strClientIdnt,
                            strTableOrderNo,
                            strOrderDate,
                            strDocType,
                            SharedUtils.formatFloat(strNetto, '.' ) + " EUR",
                            SharedUtils.formatFloat(strTotal, '.' ) + " EUR",
                            SharedUtils.formatFloat(strPayed, '.' )+ " EUR",
                            diff,
                            SharedUtils.formatFloat(penalty, '.' )+ " EUR",
                            SharedUtils.formatFloat(totalPenalty, '.' )+ " EUR",
                            getBeforTableText(letterNum),
                            getAfterTableText(strDate, letterNum)
                            
                });
            }
            rs.close();
            stm.close();
        } catch( SQLException e ) {
            e.printStackTrace();
        } finally {
            SharedUtils.closeConnection(conn);
        }
        
        final ReportGenerator generator = ReportGenerator.getInstance();
        final ResultSetTableModelFactory rstmf =  ResultSetTableModelFactory.getInstance();
        URL urlReportXml= getClass().getResource("/medstore/print/letters/letter"+letterNum+".xml");
        
        final JFreeReport report = generator.parseReport(urlReportXml);
        report.setData(tm);
        report.setName("Generate Letter " + letterNum);
        JButton print = new JButton("Print");
        
        final PrintPreview preview = new PrintPreview(report, print);
        
        print.addActionListener(new ActionListener(){
            public void actionPerformed(final ActionEvent e) {
                try{
                    if(PrintUtil.print(report)){
                        update("select " + v_nr + " as v_nr ", letterNum);
                    }
                    
                } catch(Exception ex){
                    JOptionPane.showMessageDialog(null,"Error: " + ex.getMessage());
                    ex.printStackTrace();
                }
            }
        });
        preview.setSize(750, 750);
        preview.setVisible(true);
        
        return "";
    }
    private String getPenalty( int letterMode) {
        return "" + SharedUtils.letterPenalties[letterMode-1];
    }
    private String getBeforTableText(int letterMode ) {
        if( letterMode == 1 )
            return
                "Sicherlich ist es Ihnen entgangen , die im folgenden aufgef" + (char)252 + "hrten" + "\n" +
                    "Rechnungen zubegleichen:";
        
        if( letterMode == 2 )
            return
                "Sicher haben Sie " + (char)252 + "bersehen, dab die unten aufgef" + (char)252 + "hrten Rechnungen " +
                    "zur Zahlung f" + (char)228 + "llig geworden sind.\nWir bitten Sie deshalb um sofortige " +
                    "" + (char)220 + "berweisung des Gesamtbetrages auf unser Konto.";
        
        if( letterMode == 3 )
            return
                "nachdem unsere 1. und 2. Mahnung ohne Erfolg blieb, setzen wir Ihnen " +
                    "nun eine letzte Zahlungsfrist." + "\n" +
                    "Sollte der von Ihnen Geschuldete Betrag nicht bis zu dem, der Zahlungsaufstellung, " +
                    "folgendem Datum auf unser Konto eingegangen sein, so werden wir ein " +
                    "Inkassoinstitut damit beauftragen, unsere Forderungen von Ihnen einzuziehen." + "\n" +
                    "Die damit verbundenen Kosten gehen zu Ihren Lasten " +
                    "";
        
        throw new java.lang.AssertionError();
    }
    private String getAfterTableText(String date, int letterMode ) {
        if( letterMode == 1 )
            return
                " Wir bitten Sie h" + (char)246 + "flichst den o.g. Betrag innerhalb der n" + (char)228 + "chsten 7 Tage" + "\n" +
                    " auf unser Konto zu " + (char)252 + "berweisen." +  "\n\n" +
                    " Sollten Sie inzwischen den Betrag an uns " + (char)252 + "berwiesen haben, so" +  "\n" +
                    " sehen Sie dieses Schreiben als gegenstandslos an." +  "\n\n" +
                    " Mit freundlichen Gr" + (char)252 + "ssen" +  "\n\n" +
                    " Ihr Apotheke Vita Natura Team ";
        
        if( letterMode == 2 )
            return
                "Wir erwarten Ihre Zahlung bis zum " +
                    SharedUtils.incDate(date, 10) + "\n\n" +
                    "Sollten Sie inzwischen den Betrag an uns " + (char)252 + "berwiesen haben , so sehen " +
                    "Sie dieses Schreiben \nals gegenstandslos an." + "\n\n" +
                    "Mit freundlichen Gr" + (char)252 + "ssen " + "\n\n" +
                    "Ihr Apotheke Vita Natura Team";
        
        if( letterMode == 3 )
            return
                "Wir gew" + (char)228 + "hren Ihnen eine letzte Zahlungsfrist bis zum " + SharedUtils.incDate(date, 10) + "\n\n" +
                    "Sollte der Betrag bis zu diesem Datum nicht auf unser Konto " +
                    "eingegangen sein, werden \nwir rechtliche Schritte gegen Sie einleiten, " +
                    "die mit weiteren Kosten f" + (char)252 + "r Sie verbunden sind." + "\n\n\n" +
                    "Mit freundlichen Gr" + (char)252 + "ssen" + "\n\n\n" +
                    "Ihr Apotheke Vita Natura Team";
        
        throw new java.lang.AssertionError();
    }
    
    public void update(String selected_sql, int m_letterNum) throws Exception{
        Connection conn = null;
        try {
            Timestamp tmspCurrentDate = SharedUtils.getCurrentTimestamp();
            String  sql = "update vorgang set letter" + m_letterNum + "date= ? " +
                    " where v_nr in ( select v_nr from (" + selected_sql + ") subquery )";
            System.out.println(sql);
            conn = SharedUtils.getConnection();
            PreparedStatement pstm = conn.prepareStatement(sql);
            pstm.setTimestamp(1, tmspCurrentDate );
            pstm.executeUpdate();
            pstm.close();
        } catch(SQLException e) {
            throw new Exception("Error: " + e.getMessage());
        } finally {
            SharedUtils.closeConnection(conn);
        }
    }
}
