package medstore.print.reports.commissions.personcommperoffer;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class PCPOTableModel extends AbstractTableModel{
    
    private static Class[] COLUMN_TYPES =
    {
        PCPOReportData.class,
                String.class, String.class, String.class, String.class,                 
                String.class, String.class, String.class, String.class,
                Double.class, Double.class, Double.class
                
    };
    private static String[] COLUMN_NAMES =
    {
                "report",
                        "report.date", "report.query", "report.interval", "report.producer",
                        "col.name",  "col.type",     "col.date", "col.ordno", 
                        "col.netto", "col.percent", "col.sum", "col.uid", "col.rel_name", "col.invc_addr", "col.key"
                         
    };
  
   private transient PCPOReportData[] reportsPerRow;
   private transient PCPOData[] itemsDataPerRow; 
   private transient int[] itemsDataCountPerRow;
   
   private ArrayList reports;
   private int totalSize;
    
    public PCPOTableModel() {
        reports = new ArrayList();
    }
    public void addReport(final PCPOReportData report) {
        reports.add(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public void removeReport(final PCPOReportData report) {
        reports.remove(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public PCPOReportData getReport(final int i) {
        return (PCPOReportData) reports.get(i);
    }
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < reports.size(); i++) {
            final PCPOReportData report = getReport(i);
            size += report.getDataCount();
        }
        this.totalSize = size;
        this.reportsPerRow = null;
        this.itemsDataPerRow = null;
        this.itemsDataCountPerRow = null;
    }
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    private void fillCache() {
        if (reportsPerRow != null && itemsDataPerRow != null) {            
            return;
        }
        
        this.reportsPerRow = new PCPOReportData[totalSize];
        this.itemsDataPerRow = new PCPOData[totalSize];
        this.itemsDataCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = reports.size();
        for (int i = 0; i < invoiceSize; i++) {
            final PCPOReportData inv = (PCPOReportData) reports.get(i);
            final int articleCount = inv.getDataCount();
            for (int ac = 0; ac < articleCount; ac++) {
                reportsPerRow[currentRow] = inv;
                itemsDataPerRow[currentRow] = inv.getData(ac);
                itemsDataCountPerRow[currentRow] = inv.getDataCount(ac);
                currentRow += 1;
            }
        }
    }
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        fillCache();
        final PCPOReportData report = reportsPerRow[rowIndex];
        final PCPOData data = itemsDataPerRow[rowIndex];
        switch (columnIndex) {
            case 0: return report;
            case 1: return report.getReportDate();
            case 2: return report.getReportType();
            case 3: return report.getReportDateInterval();
            case 4: return report.getReportProducer();
            case 5: return data.getName();
            case 6: return data.getType();
            case 7: return data.getDate();
            case 8: return data.getOrderNo();
            case 9: return data.getNetto();
            case 10: return data.getPercent();
            case 11: return data.getSum();
            case 12: return data.getRelKey();
            case 13: return data.getRelName();
            case 14: return data.getInvcAddr();
            case 15: return data.getKey();
        }
        throw new IndexOutOfBoundsException("OrderHistoryReport ColumnIndex is invalid: " + columnIndex);
    }
}
