package medstore.print.reports.inventory.stockonhandtotal;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class SOHTTableModel extends AbstractTableModel{
    
    private static Class[] COLUMN_TYPES =
    {
        SOHTReportData.class,
                String.class, String.class, String.class, String.class, 
                String.class, String.class, String.class, String.class, Integer.class
    };
    private static String[] COLUMN_NAMES =
    {
                "report",
                        "report.date","report.query","report.interval","report.producer",
                        "col.barcode","col.stockname", "col.storename", "col.stockcount",
                        "col.checkout","col.diff"
    };
  
   private transient SOHTReportData[] orderHistoryReportPerRow;
   private transient SOHTData[] orderHistoryDataPerRow; 
   private transient int[] orderHistoryDataCountPerRow;
   
   private ArrayList reports;
   private int totalSize;
    
    public SOHTTableModel() {
        reports = new ArrayList();
    }
    public void addOrderHistoryReport(final SOHTReportData report) {
        reports.add(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public void removeOrderHistoryReport(final SOHTReportData report) {
        reports.remove(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public SOHTReportData getOrderHistoryReport(final int i) {
        return (SOHTReportData) reports.get(i);
    }
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < reports.size(); i++) {
            final SOHTReportData report = getOrderHistoryReport(i);
            size += report.getDataCount();
        }
        this.totalSize = size;
        this.orderHistoryReportPerRow = null;
        this.orderHistoryDataPerRow = null;
        this.orderHistoryDataCountPerRow = null;
    }
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    private void fillCache() {
        if (orderHistoryReportPerRow != null && orderHistoryDataPerRow != null) {            
            return;
        }
        
        this.orderHistoryReportPerRow = new SOHTReportData[totalSize];
        this.orderHistoryDataPerRow = new SOHTData[totalSize];
        this.orderHistoryDataCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = reports.size();
        for (int i = 0; i < invoiceSize; i++) {
            final SOHTReportData inv = (SOHTReportData) reports.get(i);
            final int articleCount = inv.getDataCount();
            for (int ac = 0; ac < articleCount; ac++) {
                orderHistoryReportPerRow[currentRow] = inv;
                orderHistoryDataPerRow[currentRow] = inv.getData(ac);
                orderHistoryDataCountPerRow[currentRow] = inv.getDataCount(ac);
                currentRow += 1;
            }
        }
    }
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        fillCache();
        final SOHTReportData report = orderHistoryReportPerRow[rowIndex];
        final SOHTData data = orderHistoryDataPerRow[rowIndex];
        switch (columnIndex) {
            case 0: return report;
            case 1: return report.getReportDate();
            case 2: return report.getReportType();
            case 3: return report.getReportDateInterval();
            case 4: return report.getReportProducer();
            case 5: return data.getBarcode();
            case 6: return data.getStockName();
            case 7: return data.getStoreName();
            case 8: return new Integer(data.getCount());            
            case 9: return data.getCheckout();
            case 10: return data.getDiffrence();
        }
        throw new IndexOutOfBoundsException("OrderHistoryReport ColumnIndex is invalid: " + columnIndex);
    }
}
