package medstore.print.reports.monthly;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class OMonthTableModel extends AbstractTableModel{
    
    private static Class[] COLUMN_TYPES =
    {
        OMonthReportData.class,
                String.class, String.class, String.class, String.class,                 
                String.class, String.class, String.class
                
    };
    private static String[] COLUMN_NAMES =
    {
                "report",
                        "report.date", "report.query", "report.interval", "report.producer",
                        "col.number",  "col.date",     "col.ordercount"
                        
    };
  
   private transient OMonthReportData[] reportsPerRow;
   private transient OMonthData[] itemsDataPerRow; 
   private transient int[] itemsDataCountPerRow;
   
   private ArrayList reports;
   private int totalSize;
    
    public OMonthTableModel() {
        reports = new ArrayList();
    }
    public void addReport(final OMonthReportData report) {
        reports.add(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public void removeReport(final OMonthReportData report) {
        reports.remove(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public OMonthReportData getReport(final int i) {
        return (OMonthReportData) reports.get(i);
    }
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < reports.size(); i++) {
            final OMonthReportData report = getReport(i);
            size += report.getDataCount();
        }
        this.totalSize = size;
        this.reportsPerRow = null;
        this.itemsDataPerRow = null;
        this.itemsDataCountPerRow = null;
    }
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    private void fillCache() {
        if (reportsPerRow != null && itemsDataPerRow != null) {            
            return;
        }
        
        this.reportsPerRow = new OMonthReportData[totalSize];
        this.itemsDataPerRow = new OMonthData[totalSize];
        this.itemsDataCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = reports.size();
        for (int i = 0; i < invoiceSize; i++) {
            final OMonthReportData inv = (OMonthReportData) reports.get(i);
            final int articleCount = inv.getDataCount();
            for (int ac = 0; ac < articleCount; ac++) {
                reportsPerRow[currentRow] = inv;
                itemsDataPerRow[currentRow] = inv.getData(ac);
                itemsDataCountPerRow[currentRow] = inv.getDataCount(ac);
                currentRow += 1;
            }
        }
    }
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        fillCache();
        final OMonthReportData report = reportsPerRow[rowIndex];
        final OMonthData data = itemsDataPerRow[rowIndex];
        switch (columnIndex) {
            case 0: return report;
            case 1: return report.getReportDate();
            case 2: return report.getReportType();
            case 3: return report.getReportDateInterval();
            case 4: return report.getReportProducer();
            case 5: return data.getRowNumber();
            case 6: return data.getDate();
            case 7: return data.getOrderCount();            
        }
        throw new IndexOutOfBoundsException("OrderHistoryReport ColumnIndex is invalid: " + columnIndex);
    }
}
