package medstore.print.reports.orderhistory;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.HashSet;
import java.util.Hashtable;
import medstore.SharedConstants;
import medstore.SharedUtils;
import medstore.print.DateUtils;
import medstore.print.reports.ReportTop;
import org.jfree.report.JFreeReport;



public class OHReport implements SharedConstants{
    private Hashtable htbFilterParams;
    
    public OHReport(Hashtable htbFilterParams) {
        this.htbFilterParams = htbFilterParams;
    }
    
    public JFreeReport getReport() throws Exception{
        String strTextDateFrom = getFromDate();
        String strTextDateTo = (String)htbFilterParams.get(TEXT_DATE_TO);
        strTextDateTo = SharedUtils.getToDate( strTextDateTo );
        String rep_date = "Date: " + DateUtils.ddmmyyy_hhmmss_current();
        String type = "Query: 'Orders History'\n";
        String rep_date_interval = "date - from: " + (String)htbFilterParams.get(TEXT_DATE_FROM) + " to: " + (String)htbFilterParams.get(TEXT_DATE_TO);
        String producer = "produced by user: " + SharedUtils.strUsername;
        
        
        
        ReportTop top = new ReportTop(rep_date, type, rep_date_interval, producer);
        OHReportData rep = new OHReportData(top);
        
        String strSQL = " SELECT " +
                " vorgang.v_nr, vorgang.invc_date, base_prsn.prsn_lnam, " +
                " base_prsn.prsn_fnam , " +
                " vorgang.neto_sum,  vorgang.vat_pers, vorgang.invc_prto, vorgang.patn_dscn " +
                " FROM " +
                " base_prsn, vorgang, conn_evnt_quer, base_evnt " +
                " WHERE vorgang.invc_date >='" + strTextDateFrom + "' " +
                " AND vorgang.invc_date <'" + strTextDateTo + "'" +
                " AND (v_nr NOT IN (SELECT invoiceid FROM vorgang WHERE      doc_type_idnt = 2)) " +
                " AND vorgang.patn_idnt=base_prsn.prsn_key " +
                " AND vorgang.v_nr = conn_evnt_quer.quer_idnt " +
                " AND base_evnt.evnt_key=conn_evnt_quer.evnt_idnt " +
                " AND doc_type_idnt = 1 " +
                " ORDER BY vorgang.invc_date ASC, vorgang.v_nr ASC ";
        
        Connection conn = null;
        int count = 0;
        try {
            conn = SharedUtils.getConnection();
            Statement stm = conn.createStatement();
            ResultSet rs = stm.executeQuery(strSQL);
            
            HashSet vnrs = new HashSet();
            while( rs.next() ) {
                
                String orderNum = rs.getString(1).trim();
                        if( !vnrs.add(orderNum)  )
                            continue;
                count++;
                SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");                
                String datetime = sdf.format(rs.getTimestamp("invc_date"));
                
                String name2 = rs.getString(3);
                String name1 = rs.getString(4);
                
                float fNeto = rs.getFloat("neto_sum");                
                float fVat = rs.getFloat("vat_pers");
                float fPorto = rs.getFloat("invc_prto");
                float iDiscount = rs.getFloat( "patn_dscn" );
                
                
                
                float fDiscount =  fNeto * 0.01f * iDiscount;
                float fTotal = fNeto + fPorto - fDiscount;
                fVat = fTotal * 0.01f * fVat;
                fTotal +=fVat;
                
                float total = fTotal;
                String strTotal = SharedUtils.formatFloat("" + total, '.');
                
                String name = "";
                if( name1 != null && !name1.trim().equals("") &&
                        name2 != null && !name2.trim().equals("") ) {
                    name = name1.trim() + " , " + name2.trim();
                } else if( name1!=null && !name1.trim().equals("") ) {
                    name = name1.trim();
                } else if( name2!=null && !name2.trim().equals("") ) {
                    name = name2.trim();
                }
                
                OHData rep_data = new OHData(count + ")", datetime, name, orderNum, strTotal);
                rep.addOrderHistoryData(rep_data);                
            }
            
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            SharedUtils.closeConnection(conn);
        }
        if(count==0)
            rep.addOrderHistoryData(new OHData("", "", "", "", ""));
        
        
        OHTableModel tm = new OHTableModel();
        tm.addOrderHistoryReport(rep);
        OHPrintUtils spUtils = new OHPrintUtils(tm,"/medstore/print/reports/orderhistory/report.xml","Orders History");        
        return spUtils.createReport();
        
    }
    private String getFromDate(){
        String strTextDateFrom = (String)htbFilterParams.get(TEXT_DATE_FROM);
        String dateFields[] = strTextDateFrom.split("\\.");
        strTextDateFrom = dateFields[1] + "." + dateFields[0] + "." + dateFields[2];
        return strTextDateFrom;
    }
}
