package medstore.print.reports.orderhistory;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class OHTableModel extends AbstractTableModel{
    
    private static Class[] COLUMN_TYPES =
    {
        OHReportData.class,
                String.class, String.class, String.class, String.class, 
                String.class, String.class, String.class, String.class, String.class
    };
    private static String[] COLUMN_NAMES =
    {
                "report",
                        "report.date","report.query","report.interval","report.producer",
                        "col.number","col.date", "col.name", "col.ordnum", "col.total"
    };
  
   private transient OHReportData[] orderHistoryReportPerRow;
   private transient OHData[] orderHistoryDataPerRow; 
   private transient int[] orderHistoryDataCountPerRow;
   
   private ArrayList reports;
   private int totalSize;
    
    public OHTableModel() {
        reports = new ArrayList();
    }
    public void addOrderHistoryReport(final OHReportData report) {
        reports.add(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public void removeOrderHistoryReport(final OHReportData report) {
        reports.remove(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public OHReportData getOrderHistoryReport(final int i) {
        return (OHReportData) reports.get(i);
    }
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < reports.size(); i++) {
            final OHReportData report = getOrderHistoryReport(i);
            size += report.getOrderHistoryDataCount();
        }
        this.totalSize = size;
        this.orderHistoryReportPerRow = null;
        this.orderHistoryDataPerRow = null;
        this.orderHistoryDataCountPerRow = null;
    }
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    private void fillCache() {
        if (orderHistoryReportPerRow != null && orderHistoryDataPerRow != null) {            
            return;
        }
        
        this.orderHistoryReportPerRow = new OHReportData[totalSize];
        this.orderHistoryDataPerRow = new OHData[totalSize];
        this.orderHistoryDataCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = reports.size();
        for (int i = 0; i < invoiceSize; i++) {
            final OHReportData inv = (OHReportData) reports.get(i);
            final int articleCount = inv.getOrderHistoryDataCount();
            for (int ac = 0; ac < articleCount; ac++) {
                orderHistoryReportPerRow[currentRow] = inv;
                orderHistoryDataPerRow[currentRow] = inv.getOrderHistoryData(ac);
                orderHistoryDataCountPerRow[currentRow] = inv.getOrderHistoryDataCount(ac);
                currentRow += 1;
            }
        }
    }
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        fillCache();
        final OHReportData report = orderHistoryReportPerRow[rowIndex];
        final OHData data = orderHistoryDataPerRow[rowIndex];
        switch (columnIndex) {
            case 0: return report;
            case 1: return report.getReportDate();
            case 2: return report.getReportType();
            case 3: return report.getReportDateInterval();
            case 4: return report.getReportProducer();
            case 5: return data.getRowNumber();
            case 6: return data.getDate();
            case 7: return data.getName();
            case 8: return data.getOrderNo();
            case 9: return data.getTotal();            
        }
        throw new IndexOutOfBoundsException("OrderHistoryReport ColumnIndex is invalid: " + columnIndex);
    }
}
