package medstore.print.reports.ordersamount;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Hashtable;
import medstore.SharedConstants;
import medstore.SharedUtils;
import medstore.math.BigDec;
import medstore.print.DateUtils;
import medstore.print.reports.ReportTop;
import org.jfree.report.JFreeReport;



public class OAReport implements SharedConstants{
    private Hashtable htbFilterParams;
    
    public OAReport(Hashtable htbFilterParams) {
        this.htbFilterParams = htbFilterParams;
    }
    
    public JFreeReport getReport() throws Exception{
        String strTextDateFrom = getFromDate();
        String strTextDateTo = (String)htbFilterParams.get(TEXT_DATE_TO);
        strTextDateTo = SharedUtils.getToDate( strTextDateTo );
        String rep_date = "Date: " + DateUtils.ddmmyyy_hhmmss_current();
        String type = "Query: 'Orders Ammount'\n";
        String rep_date_interval = "date - from: " + (String)htbFilterParams.get(TEXT_DATE_FROM) + " to: " + (String)htbFilterParams.get(TEXT_DATE_TO);
        String producer = "produced by user: " + SharedUtils.strUsername;
        
        
        
        ReportTop top = new ReportTop(rep_date, type, rep_date_interval, producer);
        OAReportData rep = new OAReportData(top);
        
        String strSQL = " SELECT base_prsn.prsn_lnam , base_prsn.prsn_fnam , " +
                " vorgang.invc_date, vorgang.v_nr, vorgang.neto_sum, " +
                " vorgang.vat_pers, vorgang.invc_prto, " +
                " vorgang.patn_dscn, vorgang.doc_type_idnt " +
                " FROM base_prsn, vorgang " +
                " WHERE invc_date>='" + strTextDateFrom + "'" +
                " AND invc_date<'" + strTextDateTo + "'" +
                " AND vorgang.patn_idnt=base_prsn.prsn_key " +
                " ORDER BY vorgang.invc_date asc, v_nr ASC ";
        
        Connection conn = null;
        int count = 0;
        try {
            conn = SharedUtils.getConnection();
            Statement stm = conn.createStatement();
            ResultSet rs = stm.executeQuery(strSQL);
            
            float fSumNetto = 0,
                    fSumPorto = 0,
                    fSumVAT = 0,
                    fSumTotal = 0;
            
            while( rs.next() ) {
                count++;
                
                String orderNum = rs.getString("v_nr").trim();
                SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");
                String datetime = sdf.format(rs.getTimestamp("invc_date"));
                
                String name2 = rs.getString(1);
                String name1 = rs.getString(2);
                
                float fNeto = rs.getFloat("neto_sum");
                float fVat = rs.getFloat("vat_pers");
                float fPorto = rs.getFloat("invc_prto");
                float iDiscount = rs.getFloat( "patn_dscn" );
                
                int iDocType = rs.getInt( "doc_type_idnt" );
                
                float fDiscount =  fNeto * 0.01f * iDiscount;
                float fTotal = fNeto + fPorto - fDiscount;
                fVat = fTotal * 0.01f * fVat;
                fTotal +=fVat;
                
                if( iDocType == 2 ) //credit note
                {
                    fNeto  *= -1;
                    fTotal *= -1;
                    iDiscount *= -1;
                    fPorto *= -1;
                    fVat*=-1;                    
                    fDiscount*=-1;
                    
                }
                
                
                
                String name = "";
                if( name1 != null && !name1.trim().equals("") &&
                        name2 != null && !name2.trim().equals("") ) {
                    name = name1.trim() + " , " + name2.trim();
                } else if( name1!=null && !name1.trim().equals("") ) {
                    name = name1.trim();
                } else if( name2!=null && !name2.trim().equals("") ) {
                    name = name2.trim();
                }
                
                String strNetto = BigDec.scaledRoundHalfUp("" + fNeto, 2);
                String strPorto = BigDec.scaledRoundHalfUp("" + fPorto, 2);
                String strVAT = BigDec.scaledRoundHalfUp("" + fVat, 2);
                String strTotal = BigDec.scaledRoundHalfUp("" + fTotal, 2);
                
                /*Set rounded values to original Float values*/
                fNeto = Float.parseFloat(strNetto);
                fPorto = Float.parseFloat(strPorto);
                fVat = Float.parseFloat(strVAT);
                fTotal = Float.parseFloat(strTotal);
                /*---------------------------------*/
                
                fSumNetto += fNeto;
                fSumPorto += fPorto;
                fSumVAT += fVat;
                fSumTotal += fTotal;
                
                OAData rep_data = new OAData(count + ")", datetime, name, orderNum, strNetto, strPorto, strVAT, strTotal, fDiscount);
                rep.addData(rep_data);
                
            }
            rep.setTotals(fSumNetto, fSumPorto, fSumVAT, fSumTotal);
            
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            SharedUtils.closeConnection(conn);
        }
        if(count==0)
            rep.addData(new OAData("", "", "", "", "", "", "", "", 0.00f));
        
        
        OATableModel tm = new OATableModel();
        tm.addReport(rep);
        OAPrintUtils spUtils = new OAPrintUtils(tm,"/medstore/print/reports/ordersamount/report.xml","Orders Ammount");
        return spUtils.createReport();
        
    }
    private String getFromDate(){
        String strTextDateFrom = (String)htbFilterParams.get(TEXT_DATE_FROM);
        String dateFields[] = strTextDateFrom.split("\\.");
        strTextDateFrom = dateFields[1] + "." + dateFields[0] + "." + dateFields[2];
        return strTextDateFrom;
    }
}
