package medstore.print.reports.ordersamount;
import java.util.ArrayList;
import medstore.SharedUtils;
import medstore.math.BigDec;
import medstore.print.reports.ReportTop;

public class OAReportData extends ReportTop{
    private ArrayList rowdata;
    private ArrayList rowdataCounts;
    private float fSumNetto = 0, 
            fSumPorto = 0, 
            fSumVAT = 0, 
            fSumTotal = 0;
    
    public OAReportData(ReportTop top) {
        super(top.getReportDate(), top.getReportType(), top.getReportDateInterval(), top.getReportProducer());
        rowdata = new ArrayList();
        rowdataCounts = new ArrayList();
    }
    
    public synchronized void addData( OAData data) {
        final int index = rowdata.indexOf(data);
        if (index == -1) {
            
            rowdata.add(data);
            rowdataCounts.add(new Integer(1));
        } else {
            final Integer oldCount = (Integer) rowdataCounts.get(index);
            rowdataCounts.set(index, new Integer(oldCount.intValue() + 1));
        }        
    }
    
    public synchronized void removeData(final OAData data) {
        final int index = rowdata.indexOf(data);
        if (index != -1) {
            final Integer oldCount = (Integer) rowdataCounts.get(index);
            if (oldCount.intValue() == 1) {
                rowdataCounts.remove(index);
                rowdata.remove(index);
            } else {
                rowdataCounts.set(index, new Integer(oldCount.intValue() - 1));
            }
        }        
    }
    public synchronized void setData(ArrayList rowdata){
        for(int i = 0; i<rowdata.size(); i++){
            addData((OAData)rowdata.get(i));
        }
    }
    
    public OAData getData(final int index) {
        return (OAData) rowdata.get(index);
    }
    
    public int getDataCount(final int index) {
        final Integer i = (Integer) rowdataCounts.get(index);
        return i.intValue();
    }
    
    public int getDataCount() {
        return rowdata.size();
    }
    public void setTotals(float fSumNetto, float fSumPorto, float fSumVAT, float fSumTotal){
        this.fSumNetto = fSumNetto;
        this.fSumPorto = fSumPorto;
        this.fSumVAT = fSumVAT;
        this.fSumTotal = fSumTotal;
    }
    public String getTotalNetto(){
        return SharedUtils.formatFloat("" + fSumNetto, '.');
    }
    public String getTotalPorto(){
        return  SharedUtils.formatFloat("" + fSumPorto, '.');
    }
    public String getTotalVat(){
        return SharedUtils.formatFloat("" + fSumVAT, '.');
    }
    public String getTotalTotal(){
        return BigDec.scaledRoundHalfUp(fSumTotal,2);
    }
}
