package medstore.print.reports.ordersamount;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class OATableModel extends AbstractTableModel{
    
    private static Class[] COLUMN_TYPES =
    {
        OAReportData.class,
                String.class, String.class, String.class, String.class, 
                String.class, String.class, String.class, String.class, 
                String.class, String.class, String.class,
                String.class, String.class, String.class, String.class, Double.class
    };
    private static String[] COLUMN_NAMES =
    {
                "report",
                        "report.date", "report.query", "report.interval", "report.producer",
                        "col.number",  "col.date",     "col.name",        "col.ordnum", "col.netto", 
                        "col.porto",   "col.vat",      "col.total",
                        "total.netto", "total.porto", "total.vat", "total.total", "discount"
    };
  
   private transient OAReportData[] reportsPerRow;
   private transient OAData[] itemsDataPerRow; 
   private transient int[] itemsDataCountPerRow;
   
   private ArrayList reports;
   private int totalSize;
    
    public OATableModel() {
        reports = new ArrayList();
    }
    public void addReport(final OAReportData report) {
        reports.add(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public void removeReport(final OAReportData report) {
        reports.remove(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public OAReportData getReport(final int i) {
        return (OAReportData) reports.get(i);
    }
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < reports.size(); i++) {
            final OAReportData report = getReport(i);
            size += report.getDataCount();
        }
        this.totalSize = size;
        this.reportsPerRow = null;
        this.itemsDataPerRow = null;
        this.itemsDataCountPerRow = null;
    }
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    private void fillCache() {
        if (reportsPerRow != null && itemsDataPerRow != null) {            
            return;
        }
        
        this.reportsPerRow = new OAReportData[totalSize];
        this.itemsDataPerRow = new OAData[totalSize];
        this.itemsDataCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = reports.size();
        for (int i = 0; i < invoiceSize; i++) {
            final OAReportData inv = (OAReportData) reports.get(i);
            final int articleCount = inv.getDataCount();
            for (int ac = 0; ac < articleCount; ac++) {
                reportsPerRow[currentRow] = inv;
                itemsDataPerRow[currentRow] = inv.getData(ac);
                itemsDataCountPerRow[currentRow] = inv.getDataCount(ac);
                currentRow += 1;
            }
        }
    }
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        fillCache();
        final OAReportData report = reportsPerRow[rowIndex];
        final OAData data = itemsDataPerRow[rowIndex];
        switch (columnIndex) {
            case 0: return report;
            case 1: return report.getReportDate();
            case 2: return report.getReportType();
            case 3: return report.getReportDateInterval();
            case 4: return report.getReportProducer();
            case 5: return data.getRowNumber();
            case 6: return data.getDate();
            case 7: return data.getName();
            case 8: return data.getOrderNo();
            case 9: return data.getNetto();
            case 10: return data.getPorto();
            case 11: return data.getVat();
            case 12: return data.getTotal();
            case 13: return report.getTotalNetto();
            case 14: return report.getTotalPorto();
            case 15: return report.getTotalVat();
            case 16: return report.getTotalTotal();
            case 17: return data.getDiscount();
        }
        throw new IndexOutOfBoundsException("OrderHistoryReport ColumnIndex is invalid: " + columnIndex);
    }
}
