package medstore.print.reports.orderspending;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Hashtable;
import medstore.SharedConstants;
import medstore.SharedUtils;
import medstore.math.BigDec;
import medstore.print.DateUtils;
import medstore.print.reports.ReportTop;
import org.jfree.report.JFreeReport;



public class OPReport implements SharedConstants{
    private Hashtable htbFilterParams;
    
    public OPReport(Hashtable htbFilterParams) {
        this.htbFilterParams = htbFilterParams;
    }
    
    public JFreeReport getReport() throws Exception{
        String strTextDateFrom = getFromDate();
        String strTextDateTo = (String)htbFilterParams.get(TEXT_DATE_TO);
        strTextDateTo = SharedUtils.getToDate( strTextDateTo );
        String rep_date = "Date: " +  DateUtils.ddmmyyy_hhmmss_current();
        String type = "Query: 'Pending Orders'\n";
        String rep_date_interval = "date - from: " + (String)htbFilterParams.get(TEXT_DATE_FROM) + " to: " + (String)htbFilterParams.get(TEXT_DATE_TO);
        String producer = "produced by user: " + SharedUtils.strUsername;
        
        
        
        ReportTop top = new ReportTop(rep_date, type, rep_date_interval, producer);
        OPReportData rep = new OPReportData(top);
                        
        String strSQL = " SELECT  cast(vorgang.invc_date as smalldatetime) as invc_date, " +
                                " base_prsn.prsn_fnam , base_prsn.prsn_lnam, " +
                                " vorgang.v_nr, vorgang.neto_sum, " +
                                " vorgang.vat_pers, vorgang.invc_prto, vorgang.patn_dscn " +
                                " FROM base_prsn, vorgang " +
                                " WHERE v_nr not in(select quer_idnt from conn_evnt_quer) " + 
                                " AND vorgang.INVC_DATE >='" + strTextDateFrom + "' AND " +
                                " vorgang.INVC_DATE <'" + strTextDateTo + "'" +
                                " AND vorgang.doc_type_idnt=1 " +
                                " AND vorgang.patn_idnt=base_prsn.prsn_key " + 
                " AND (v_nr NOT IN (SELECT invoiceid FROM vorgang WHERE      doc_type_idnt = 2)) " +
                                " ORDER BY vorgang.INVC_DATE ASC, vorgang.v_nr ASC";
        
        Connection conn = null;
        int count = 0;
        try {
            conn = SharedUtils.getConnection();
            Statement stm = conn.createStatement();
            ResultSet rs = stm.executeQuery(strSQL);
            
            float fSumNetto = 0, 
            fSumPorto = 0, 
            fSumVAT = 0, 
            fSumTotal = 0;
            
            while( rs.next() ) {
                count++;
                
                String orderNum = rs.getString("v_nr").trim();
                SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");
                String datetime = sdf.format(rs.getTimestamp("invc_date"));
                
                String name2 = rs.getString("prsn_lnam");
                String name1 = rs.getString("prsn_fnam");
                
                float fNeto = rs.getFloat("neto_sum");                
                float fVat = rs.getFloat("vat_pers");
                float fPorto = rs.getFloat("invc_prto");
                float iDiscount = rs.getFloat( "patn_dscn" );
                
                
                float fDiscount =  fNeto * 0.01f * iDiscount;
                float fTotal = fNeto + fPorto - fDiscount;
                fVat = fTotal * 0.01f * fVat;
                fTotal +=fVat;
                
                
                String name = "";
                if( name1 != null && !name1.trim().equals("") &&
                        name2 != null && !name2.trim().equals("") ) {
                    name = name1.trim() + " , " + name2.trim();
                } else if( name1!=null && !name1.trim().equals("") ) {
                    name = name1.trim();
                } else if( name2!=null && !name2.trim().equals("") ) {
                    name = name2.trim();
                }
                
                String strNetto = BigDec.scaledRoundHalfUp("" + fNeto, 2);
                String strPorto = BigDec.scaledRoundHalfUp("" + fPorto, 2);
                String strVAT = BigDec.scaledRoundHalfUp("" + fVat, 2);
                String strTotal = BigDec.scaledRoundHalfUp("" + fTotal, 2);
                
                OPData rep_data = new OPData(count + ")", datetime, name, orderNum, strNetto, strPorto, strVAT, strTotal);
                rep.addData(rep_data);
                
            }
            
            
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            SharedUtils.closeConnection(conn);
        }
        if(count==0)
            rep.addData(new OPData("", "", "", "", "", "", "", ""));
        
        
        OPTableModel tm = new OPTableModel();
        tm.addReport(rep);
        OPPrintUtils spUtils = new OPPrintUtils(tm,"/medstore/print/reports/orderspending/report.xml","Pending Orders");
        return spUtils.createReport();
        
    }
    private String getFromDate(){
        String strTextDateFrom = (String)htbFilterParams.get(TEXT_DATE_FROM);
        String dateFields[] = strTextDateFrom.split("\\.");
        strTextDateFrom = dateFields[1] + "." + dateFields[0] + "." + dateFields[2];
        return strTextDateFrom;
    }
}
