package medstore.print.reports.salesdaily;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashSet;
import java.util.Hashtable;
import medstore.SharedConstants;
import medstore.SharedUtils;
import medstore.print.DateUtils;
import medstore.print.reports.ReportTop;
import org.jfree.report.JFreeReport;



public class SADayReport implements SharedConstants{
    private Hashtable htbFilterParams;
    private String[] m_strMonths = {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG",
            "SEP","OCT","NOV","DEC"};
    public SADayReport(Hashtable htbFilterParams) {
        this.htbFilterParams = htbFilterParams;
    }
    
    public JFreeReport getReport() throws Exception{
        String strTextDateFrom = getFromDate();
        String strTextDateTo = (String)htbFilterParams.get(TEXT_DATE_TO);
        strTextDateTo = SharedUtils.getToDate( strTextDateTo );
        String rep_date = "Date: " + DateUtils.ddmmyyy_hhmmss_current();
        String type = "Query: 'Active sales people(Daily)'\n";
        String rep_date_interval = "date - from: " + (String)htbFilterParams.get(TEXT_DATE_FROM) + " to: " + (String)htbFilterParams.get(TEXT_DATE_TO);
        String producer = "produced by user: " + SharedUtils.strUsername;
        
        
        
        //get person's create dates
        ArrayList createTimes = getOrdersCreateTimes4SalesPeople(strTextDateFrom, strTextDateTo, 2 );//dealers
        
        ArrayList data = getActivePersonsData( strTextDateFrom, strTextDateTo,
                createTimes,Calendar.DAY_OF_MONTH, "", false );
        
        ReportTop top = new ReportTop(rep_date, type, rep_date_interval, producer);
        SADayReportData rep = new SADayReportData(top);
       int count = 0;
        for( int i = 0; i < data.size(); i++ ) {
           count++;
            ArrayList row = (ArrayList)data.get(i);            
            String date = row.get(1).toString();
            String itemcount = row.get(2).toString();
            
            SADayData rep_data = new SADayData((i+1) + ")", date, itemcount);
            rep.addData(rep_data);
        }
       if(count==0)
            rep.addData(new SADayData("", "", ""));
        SADayTableModel tm = new SADayTableModel();
        tm.addReport(rep);
        SADayPrintUtils spUtils = new SADayPrintUtils(tm,"/medstore/print/reports/salesdaily/report.xml","Active sales people(Daily)");
        return spUtils.createReport();
        
    }
    private String getFromDate(){
        String strTextDateFrom = (String)htbFilterParams.get(TEXT_DATE_FROM);
        String dateFields[] = strTextDateFrom.split("\\.");
        strTextDateFrom = dateFields[1] + "." + dateFields[0] + "." + dateFields[2];
        return strTextDateFrom;
    }
    
    private ArrayList getOrdersCreateTimes4SalesPeople(String strTextDateFrom, String strTextDateTo, int pesrsonType) {
        String strSQL = " select  " +
                " invc_date, prv2_idnt, prv3_idnt " +
                " from vorgang " +
                " where " +
                " invc_date>='" + strTextDateFrom + "'" +
                " AND invc_date<'" + strTextDateTo + "'" +
                " AND vorgang.doc_type_idnt = 1 " +
                " order by invc_date";
        ArrayList createTimes = new ArrayList();
        Connection conn = null;
        try {
            conn = SharedUtils.getConnection();
            Statement stm = conn.createStatement();
            ResultSet rs = stm.executeQuery(strSQL);
            while( rs.next() ) {
                ArrayList list = new ArrayList();
                Long createTime = new Long( rs.getTimestamp(1).getTime() );
                String persId = rs.getString(2);  //prov2
                list.add( persId );
                list.add(createTime);
                createTimes.add(list);
                
                persId = rs.getString(3);//prov3
                if( persId != null ) {
                    list = new ArrayList();
                    list.add( persId );
                    list.add(createTime);
                    createTimes.add(list);
                }
            }
        } catch(Exception e) {
            e.printStackTrace();
        } finally {
            SharedUtils.closeConnection( conn );
        }
        return createTimes;
    }
    ArrayList getActivePersonsData( String strTextDateFrom, String strTextDateTo,
            ArrayList createTimesPersons,int rate,String year, boolean hasEmptyPeriods )
    {
        ArrayList data = new ArrayList();                
        Calendar calEnd = Calendar.getInstance();
        String[] fields = strTextDateTo.split( "\\." );
        calEnd.set( Calendar.YEAR, 2000 + Integer.parseInt( fields[2] ) );
        calEnd.set( Calendar.MONTH, Integer.parseInt( fields[0] ) - 1 );
        calEnd.set( Calendar.DAY_OF_MONTH, Integer.parseInt( fields[1] ) );
        SharedUtils.set2BeginOfTheDay( calEnd );
        
        Calendar calBeg = Calendar.getInstance();
        fields = strTextDateFrom.split( "\\." );
        calBeg.set( Calendar.YEAR, 2000 + Integer.parseInt( fields[2] ) );
        calBeg.set( Calendar.MONTH, Integer.parseInt( fields[0] ) - 1 );
        calBeg.set( Calendar.DAY_OF_MONTH, Integer.parseInt( fields[1] ) );
        SharedUtils.set2BeginOfTheDay( calBeg );
        int num = 1;
        int beginJ = 0;
        while( calEnd.getTimeInMillis() > calBeg.getTimeInMillis() )
        {
            long begin = calBeg.getTimeInMillis();
            calBeg.add( rate, 1 );
            long end = calBeg.getTimeInMillis();
            int count = 0;
            HashSet persons = new HashSet();
            for( int j = beginJ; j < createTimesPersons.size(); j++ )
            {
                ArrayList row = (ArrayList)createTimesPersons.get(j);
                String persId = (String)row.get(0);
                long createTime = ((Long)row.get(1)).longValue(); 
                
                if( createTime < begin )
                {
                    beginJ = j + 1;
                    continue;
                }
                
                if( /*createTime>=begin &&*/ createTime<end  )
                {
                    beginJ = j + 1;
                    if( persons.add(persId) )
                        count++;
                }                
            }
            
            if( !hasEmptyPeriods && count == 0 )
                continue;
            ArrayList row = new ArrayList();            
            row.add( num + ")" );
            if( rate == Calendar.DAY_OF_MONTH )
            {
                row.add( SharedUtils.getDateFromMillis( 
                        begin,  SharedUtils.PARSE_ONLY_DATE ) );
            }
            else if( rate == Calendar.MONTH )
            {
                row.add( m_strMonths[num-1] + " " + year );                                
            }
            else
                throw new java.lang.AssertionError();
            row.add( "" + count );
            data.add(row);
            num++;
        }         
        System.out.println("\n\n\n");
        return data;
    }    
}
