package medstore.print.reports.usage;

import java.util.ArrayList;
import java.util.Date;
import javax.swing.table.AbstractTableModel;

public class USAGETableModel extends AbstractTableModel{
    
    private static Class[] COLUMN_TYPES =
    {
        USAGEReportData.class,
                String.class, String.class, String.class, String.class, 
                String.class, String.class, String.class, String.class, String.class
    };
    private static String[] COLUMN_NAMES =
    {
                "report",
                        "report.date","report.query","report.interval","report.producer",
                        "col.number","col.date", "col.name", "col.barcode", "col.count"
    };
  
   private transient USAGEReportData[] reportsPerRow;
   private transient USAGEData[] itemsDataPerRow; 
   private transient int[] itemsDataCountPerRow;
   
   private ArrayList reports;
   private int totalSize;
    
    public USAGETableModel() {
        reports = new ArrayList();
    }
    public void addReport(final USAGEReportData report) {
        reports.add(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public void removeReport(final USAGEReportData report) {
        reports.remove(report);
        invalidateCaches();
        fireTableDataChanged();
    }
    public USAGEReportData getReport(final int i) {
        return (USAGEReportData) reports.get(i);
    }
    public void invalidateCaches() {
        int size = 0;
        for (int i = 0; i < reports.size(); i++) {
            final USAGEReportData report = getReport(i);
            size += report.getItemDataCount();
        }
        this.totalSize = size;
        this.reportsPerRow = null;
        this.itemsDataPerRow = null;
        this.itemsDataCountPerRow = null;
    }
    public int getColumnCount() {
        return COLUMN_NAMES.length;
    }
    
    public int getRowCount() {
        return totalSize;
    }
    
    public String getColumnName(final int column) {
        return COLUMN_NAMES[column];
    }
    
    public Class getColumnClass(final int columnIndex) {
        return COLUMN_TYPES[columnIndex];
    }
    private void fillCache() {
        if (reportsPerRow != null && itemsDataPerRow != null) {            
            return;
        }
        
        this.reportsPerRow = new USAGEReportData[totalSize];
        this.itemsDataPerRow = new USAGEData[totalSize];
        this.itemsDataCountPerRow = new int[totalSize];
        
        int currentRow = 0;
        final int invoiceSize = reports.size();
        for (int i = 0; i < invoiceSize; i++) {
            final USAGEReportData rep = (USAGEReportData) reports.get(i);
            final int articleCount = rep.getItemDataCount();
            for (int ac = 0; ac < articleCount; ac++) {
                reportsPerRow[currentRow] = rep;
                itemsDataPerRow[currentRow] = rep.getItemData(ac);
                itemsDataCountPerRow[currentRow] = rep.getItemCount(ac);
                currentRow += 1;
            }
        }
    }
    public Object getValueAt(final int rowIndex, final int columnIndex) {
        fillCache();
        final USAGEReportData report = reportsPerRow[rowIndex];
        final USAGEData data = itemsDataPerRow[rowIndex];
        switch (columnIndex) {
            case 0: return report;
            case 1: return report.getReportDate();
            case 2: return report.getReportType();
            case 3: return report.getReportDateInterval();
            case 4: return report.getReportProducer();
            case 5: return data.getRowNumber();            
            case 7: return data.getName();
            case 8: return data.getBarcode();
            case 9: return data.getStockCount();            
            
        }
        throw new IndexOutOfBoundsException("OrderHistoryReport ColumnIndex is invalid: " + columnIndex);
    }
}
