package medstore.products.composite;

import java.lang.reflect.Array;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import medstore.SharedUtils;
import medstore.StockManagerAdmin;
import medstore.UniversalTableModel;
import medstore.UniversalTableRow;



public class CompositeProduct {
    private String name;
    private String barcode;
    private String count;
    private String contentBarcode;
    private String limit;
    private String bPrice;
    private String mPrice;
    private String sPrice;
    private String abbr;
    private String stockGroup;
    private String quote;
    private int parent;
    private int stck_key = -1;
    
    private UniversalTableModel childs;
    
    private String[] prodCols = {"Name","Count"};
    private String[] prodAll = {"Name"};
    
    public static final int name_index = 0;
    public static final int count_index = 1;
    public static final int poduct_index = 2;
    public static final int poduct_barcode = 3;
    String[] dbcols = {"id","stck_key","stck_parent_key","cnt"};
    
    public CompositeProduct(String stck_key) {
        this.stck_key = Integer.valueOf(stck_key);
    }
    public CompositeProduct(UniversalTableRow tableRow) {
        if(tableRow==null){
            return;
        }
        name = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_NAME_INDEX));
        barcode  =  ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_BARCODE_INDEX));
        count = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_PCOUNT_INDEX));
        contentBarcode = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_CNTN_BARCODE_INDEX));
        limit = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_LIMIT_INDEX));
        bPrice = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_BPRICE_INDEX));
        mPrice = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_MPRICE_INDEX));
        sPrice = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_SPRICE_INDEX));
        abbr = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_ABBR_INDEX));
        stockGroup = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_GROUP_INDEX));
        quote = ((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_QUOTE_INDEX));
        stck_key = Integer.valueOf((String)tableRow.getDataVector().get(StockManagerAdmin.STOCK_KEY_INDEX));
        childs = null;
    }
    public void setStckKey(String id) throws Exception{
        stck_key = Integer.valueOf(id);
    }
    
    public UniversalTableModel loadChilds(){
        if(stck_key<=0)
            return new UniversalTableModel(null,prodCols);
        String sql =
                " SELECT " +
                "   base_stck.stck_name," +
                "   composite_stck.cnt, " +
                "   composite_stck.stck_key, " +
                /* -for fullfillment ----*/
                "   base_stck.stck_brcd " +
                /*----------------------*/
                " FROM BASE_STCK INNER JOIN composite_stck on base_stck.stck_key = composite_stck.stck_key" +
                " WHERE composite_stck.stck_parent_key = " + stck_key + "" +
                " ORDER BY STCK_NAME";
        
        ResultSet rs = SharedUtils.executeSelect(SharedUtils.getConnection(), sql);
        return new UniversalTableModel(rs, prodCols);
        
        
    }
    public UniversalTableModel loadProducts(){
        String sql =
                " SELECT " +
                "   base_stck.stck_name," +
                "   0, " +
                "   stck_key, " +
                "   base_stck.stck_brcd " +
                " FROM BASE_STCK " +
                (stck_key>0 ? " where composite = 1 and stck_key!=" + stck_key : " where composite = 1" ) +
                " ORDER BY STCK_NAME";
        
        ResultSet rs = SharedUtils.executeSelect(SharedUtils.getConnection(), sql);
        return new UniversalTableModel(rs, prodAll);
        
        
    }
    public boolean hasChilds(){
        UniversalTableModel arr = loadChilds();
        if(arr!=null && arr.getRowCount()>0){
            return true;
        }
        return false;
    }
    public UniversalTableModel getChilds(){
        if(childs==null){
            childs = loadChilds();
        }
        return childs;
    }
    
    public void insertComposite(MixedDataModel selTabl, boolean b) throws Exception{
        int stat = (b == true ? 0 : 1);
        
        Connection conn = SharedUtils.getConnection();//SharedUtils.DBConnect(SharedUtils.connectionString,true);
        conn.setAutoCommit(false);
        try{
            int rows = selTabl.getRowCount();
            
            String del = "delete from composite_stck where stck_parent_key =" + stck_key;
            
            String updsql = "update base_stck set composite = " + stat + " where stck_key = " + stck_key ;
            
            Statement ps = conn.createStatement();
            
            ps.addBatch(del);
            ps.addBatch(updsql);
            int[] cnt = ps.executeBatch();
            
            if(b==false){
                conn.commit();
                return;
            }
            
            
            for(int i = 0; i < rows; i++){
                UniversalTableRow row = selTabl.getRow(i);
                
                Object prod_name = row.getRowElement(name_index);
                Object prod_id = row.getRowElement( poduct_index);
                Object parent_id = stck_key;
                Object count = row.getRowElement(count_index);
                
                
                
                String sql =
                        " insert into composite_stck " +
                        " (" +
                        " stck_key," +
                        " stck_parent_key," +
                        " cnt" +
                        " ) " +
                        " values( " +
                        prod_id + "," +
                        parent_id + "," +
                        count +
                        ")";
                
                ps.executeUpdate(sql);
            }
            
            conn.commit();
        } catch(Exception e){
            SharedUtils.rollback(conn);
            throw new Exception(e.getMessage());
        } finally{
            SharedUtils.closeConnection(conn);
        }
        
    }
    
    public static boolean isPartOfComposite(String stck_key) throws Exception{
        String sql = "select stck_key from composite_stck where stck_key = " + stck_key;                
        ResultSet hasChilds = SharedUtils.executeSelect(SharedUtils.getConnection(),sql);
        
        
        System.out.println(sql);
                
        boolean b = hasChilds.next();
        
        return b;
    }
    public static boolean isComposite(String stck_key) throws Exception{
        String sql = "select stck_parent_key from composite_stck where stck_parent_key = " + stck_key;
        String sql_stat = "select composite from base_stck where composite = 0 and stck_key = " + stck_key;
        
        ResultSet hasChilds = SharedUtils.executeSelect(SharedUtils.getConnection(),sql);
        ResultSet stat = SharedUtils.executeSelect(SharedUtils.getConnection(),sql_stat);
        
        System.out.println(sql);
        System.out.println(sql_stat);
        
        boolean b2 = hasChilds.next();
        boolean b3 = stat.next();
        System.out.println(" " + b2 + " " + b3 +" ");
        return (b2 || b3);        
    }
    
    public static boolean isCompositeForBarcode(String stck_brcd){
        String sql =
                " select " +
                "   stck_parent_key " +
                " from composite_stck " +
                "   inner join base_stck on composite_stck.stck_parent_key = base_stck.stck_key " +
                " where base_stck.stck_brcd ='" + stck_brcd + "'";
        
        return SharedUtils.executeSqlScalarList(sql).size()>0;
    }
    public static String getCompositeKeyFromBarcode(String stck_brcd){
        String sql =
                " select " +
                "   stck_key " +
                " from base_stck " +                
                " where base_stck.stck_brcd ='" + stck_brcd + "'";
        String stck_key = SharedUtils.executeSqlScalar(sql);
        return  stck_key;
    }
    public static boolean isRegistered(String id){
        String sql = "select nstk_idnt from position where nstk_idnt = " + id;
        return SharedUtils.executeSqlScalarList(sql).size()>0;
    }
}
