package medstore.products.composite;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import medstore.InventoryCountTableRenderer;
import medstore.UniversalTableRow;


public class MixedActions {
    private JList srcList;
    private JTable destList;
    
    public MixedActions(JList srcList, JTable destList, final JCheckBox chkBox) {
        this.srcList = srcList;
        this.destList = destList;
        srcList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                if(chkBox.isSelected())
                    lstSrcProdsMouseClicked(evt);
                else{
                    JOptionPane.showMessageDialog(null,"Component is disabled!");
                }
            }
        });
        destList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                if(chkBox.isSelected())
                    lstSelProdsMouseClicked(evt);
                else{
                    JOptionPane.showMessageDialog(null,"Component is disabled!");
                }
            }
        });
        TableColumn col = destList.getColumnModel().getColumn(1);
        
        final JTextField txtEditorField = new JTextField();
        txtEditorField.setFont( new java.awt.Font("Tahoma", 1, 13) );
        
        txtEditorField.addFocusListener(new FocusListener(){
            public void focusLost(FocusEvent tev){
                try{
                    
                    JTextField f = ((JTextField)tev.getComponent());
                    f.postActionEvent();
                    
                } catch(Exception e){
                    e.printStackTrace();
                }
            }
            public void focusGained(FocusEvent evt){
                
            }
        });
        
        DefaultCellEditor cellEditor = new DefaultCellEditor( txtEditorField );
        col.setCellEditor(cellEditor);
        
        TableCellRenderer ren = new TableCellRenderer(){
            public Component getTableCellRendererComponent(
                    JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                JTextField renderedComponent = new JTextField();
                renderedComponent.setFont(new java.awt.Font("Tahoma", 0, 13));
                renderedComponent.setText((String)value);
                return renderedComponent;
            }
        };
        col.setCellRenderer(ren);
    }
    private void lstSrcProdsMouseClicked(java.awt.event.MouseEvent evt){
        if(evt.getClickCount() % 2 != 0 || !SwingUtilities.isLeftMouseButton(evt))
            return;
        if(evt.getSource()==srcList){
            if(srcList.getSelectedIndex()<0)
                return;
            UniversalTableRow el = ((MixedDataModel)srcList.getModel()).getRow(srcList.getSelectedIndex());
            if(((MixedDataModel)destList.getModel()).addRow(el)){
                
                int curRow = Math.max(0,destList.getRowCount()-1);
                destList.setRowSelectionInterval(curRow,curRow);
                destList.updateUI();
                Rectangle rect = destList.getCellRect(curRow,1,true);
                destList.scrollRectToVisible(rect);
            }
        }
        
    }
    private void lstSelProdsMouseClicked(java.awt.event.MouseEvent evt){
        if(evt.getClickCount() % 2 != 0 || !SwingUtilities.isLeftMouseButton(evt))
            return;
        
        if(evt.getSource()==destList){
            if(destList.getSelectedRow()<0 || destList.getSelectedColumn()==1)
                return;
            ((MixedDataModel)destList.getModel()).removeRow(destList.getSelectedRow());
            destList.updateUI();
            destList.clearSelection();
            return;
        }
        
    }
    
}
