package adminmodul.campaign;

import java.sql.*;
import java.util.*;
import adminmodul.*;
import adminmodul.SharedUtils.*;

public class CampUtils extends adminmodul.SharedUtils{
    
    private CampDataObject data;
    
    private Vector tableData;
    
    private String sqlQuery = "select base_stck.stck_brcd, base_stck.stck_name," +
            "CONVERT(CHAR(8),campaign.fromdate,4) as fromdate, " +
            "CONVERT(CHAR(8),campaign.todate,4) as todate, " +
            "campaign.lownum, campaign.highnum, " +
            "campaign.discount, campaign.stockid, campaign.id from campaign " +
            "INNER JOIN base_stck ON campaign.stockid = base_stck.stck_key";
    
    private String sqlParamQuery = "select base_stck.stck_brcd, base_stck.stck_name," +
            "CONVERT(CHAR(8),campaign.fromdate,4) as fromdate, " +
            "CONVERT(CHAR(8),campaign.todate,4) as todate, " +
            "campaign.lownum, campaign.highnum, " +
            "campaign.discount, campaign.stockid, campaign.id from campaign " +
            "INNER JOIN base_stck ON campaign.stockid = base_stck.stck_key " +
            "where campaign.id = ?";
    
    private String sqlInsert = "insert into campaign(stockid, fromdate, todate, lownum, highnum, discount) values(?,?,?,?,?,?)";
    
    private String sqlUpdate = "update campaign set stockid = ?, fromdate = ?, todate = ?, lownum = ?, highnum = ?, discount = ? where id  = ?";
    
    private String sqlLastInsertId = "SELECT max(id) from campaign where stockid = ? and fromdate = ? and todate = ? and lownum = ? and highnum = ? and discount = ?";
    
    public CampUtils(){
        this.data = null;
        tableData = null;        
    }
    
    public UniversalTableModel getTableModel() throws Exception{
        Connection conn = getConnection();
        Statement stmt = conn.createStatement();
        
        ResultSet rs = stmt.executeQuery(sqlQuery);
        String[] strColNames = {"Barcode","Stock Name","Date From","Date To","Low Qty","High Qty", "Discount"};
        UniversalTableModel utmodet = new UniversalTableModel(rs,strColNames);
        return utmodet;
    }
    
    public CampDataObject getParamCamp(String id) throws Exception{
        if(id==null)
            throw new Exception("Record Not Found");
        
        Connection conn = msConnect();
        PreparedStatement stmt = conn.prepareStatement(sqlParamQuery);
        stmt.setString(1, id);
        ResultSet rs = stmt.executeQuery();
                
        
        if(rs.next()){
            
           data = new CampDataObject(rs.getInt("stockid"), rs.getString("fromdate"), rs.getString("todate"), rs.getInt("lownum"), rs.getInt("highnum"), rs.getInt("discount"), rs.getInt("stck_brcd"));
           data.setId(rs.getInt("id"));
           data.setProdName(rs.getString("stck_name"));            
        }
        else{
            throw new Exception("Empty Record");
        }
        return data;
    }
    
    public String insertData(CampDataObject data) throws Exception{
        Connection conn = getConnection();
        
        PreparedStatement stmt = conn.prepareStatement(sqlInsert);
        
        String strLastOrdrID = null;            
        String fromDate = getToDate(data.getFromDate());
        String toDate = getToDate(data.getToDate());
       
        try{
            stmt.setInt(1, data.getStockId());
            stmt.setString(2, fromDate);
            stmt.setString(3, toDate);
            stmt.setInt(4, data.getLowNum());
            stmt.setInt(5, data.getHighNum());
            stmt.setInt(6, data.getDiscount());
            if(stmt.executeUpdate()<=0)
                throw new Exception("0 Records was Inserted");
                     
            stmt = conn.prepareStatement(sqlLastInsertId);
            stmt.setInt(1, data.getStockId());
            stmt.setString(2, fromDate);
            stmt.setString(3, toDate);
            stmt.setInt(4, data.getLowNum());
            stmt.setInt(5, data.getHighNum());
            stmt.setInt(6, data.getDiscount());
            
            ResultSet rsDBData=stmt.executeQuery(); 
            
            if(rsDBData.next()){ 
                strLastOrdrID = rsDBData.getString(1);                 
            }                       
            else{
                throw new Exception ("There is a problem with the database server!Please try again later");                
            }
           
           return strLastOrdrID;
        }
        catch(Exception e){
            rollConnection(conn);
            throw new Exception(e.getMessage());
        }
        finally{
            try{
                closeConnection(conn);
            }
            catch(Exception e){
            
            }
        }
    }
    
    public int updateData(CampDataObject data) throws Exception{
        Connection conn = getConnection();
        
        PreparedStatement stmt = conn.prepareStatement(sqlUpdate);
        String fromDate = getToDate(data.getFromDate());
        String toDate = getToDate(data.getToDate());
        
        try{     
            stmt.setInt(1, data.getStockId());
            stmt.setString(2, fromDate);
            stmt.setString(3, toDate);
            stmt.setInt(4, data.getLowNum());
            stmt.setInt(5, data.getHighNum());
            stmt.setInt(6, data.getDiscount());
            stmt.setInt(7, data.getId());
            int ins = stmt.executeUpdate();
            conn.commit();            
            conn.close();
            return ins;
        }
        catch(Exception e){
            rollConnection(conn);
            throw new Exception(e.getMessage());
        }
        finally{
            try{
                closeConnection(conn);
            }
            catch(Exception e){
            
            }
        }
    }
}
